@@ -41,11 +41,11 @@
 #include <netinet/tcp.h>
 
 #include "cache.h"
+#include "cache_transport.h"
 #include "cache_pool.h"
 #include "common/heritage.h"
 
-#include "vcli.h"
-#include "vcli_priv.h"
+#include "vcli_serve.h"
 #include "vsa.h"
 #include "vtcp.h"
 #include "vtim.h"
@@ -292,6 +292,7 @@ static void __match_proto__(task_func_t)
 vca_make_session(struct worker *wrk, void *arg)
 {
 	struct sess *sp;
+	struct req *req;
 	struct wrk_accept *wa;
 	struct sockaddr_storage ss;
 	struct suckaddr *sa;
@@ -304,6 +305,13 @@ vca_make_session(struct worker *wrk, void *arg)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CAST_OBJ_NOTNULL(wa, arg, WRK_ACCEPT_MAGIC);
 
+	if (VTCP_blocking(wa->acceptsock)) {
+		AZ(close(wa->acceptsock));
+		wrk->stats->sess_drop++;	// XXX Better counter ?
+		WS_Release(wrk->aws, 0);
+		return;
+	}
+
 	/* Turn accepted socket into a session */
 	AN(wrk->aws->r);
 	sp = SES_New(wrk->pool);
@@ -329,7 +337,6 @@ vca_make_session(struct worker *wrk, void *arg)
 
 	sp->fd = wa->acceptsock;
 	wa->acceptsock = -1;
-	sp->sess_step = wa->acceptlsock->first_step;
 
 	assert(wa->acceptaddrlen <= vsa_suckaddr_len);
 	SES_Reserve_remote_addr(sp, &sa);
@@ -348,7 +355,8 @@ vca_make_session(struct worker *wrk, void *arg)
 
 	VTCP_name(sa, laddr, sizeof laddr, lport, sizeof lport);
 
-	VSL(SLT_Begin, sp->vxid, "sess 0 %s", wa->acceptlsock->proto_name);
+	VSL(SLT_Begin, sp->vxid, "sess 0 %s",
+	    wa->acceptlsock->transport->name);
 	VSL(SLT_SessOpen, sp->vxid, "%s %s %s %s %s %.6f %d",
 	    raddr, rport, wa->acceptlsock->name, laddr, lport,
 	    sp->t_open, sp->fd);
@@ -364,14 +372,15 @@ vca_make_session(struct worker *wrk, void *arg)
 	}
 	vca_tcp_opt_set(sp->fd, 0);
 
-	/* SES_Proto_Sess() must be sceduled with reserved WS */
-	assert(8 <= WS_Reserve(sp->ws, 8));
-	wrk->task.func = SES_Proto_Sess;
-	wrk->task.priv = sp;
+	req = Req_New(wrk, sp);
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	req->htc->fd = sp->fd;
+
+	SES_SetTransport(wrk, sp, req, wa->acceptlsock->transport);
 }
 
 /*--------------------------------------------------------------------
- * This function accepts on a single socket for a single session pool.
+ * This function accepts on a single socket for a single thread pool.
  *
  * As long as we can stick the accepted connection to another thread
  * we do so, otherwise we put the socket back on the "BACK" pool
@@ -459,7 +468,7 @@ vca_accept_task(struct worker *wrk, void *arg)
 }
 
 /*--------------------------------------------------------------------
- * Called when a worker and attached session pool is created, to
+ * Called when a worker and attached thread pool is created, to
  * allocate the tasks which will listen to sockets for that pool.
  */
 
@@ -494,6 +503,7 @@ vca_acct(void *arg)
 	(void)vca_tcp_opt_init();
 
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
+		CHECK_OBJ_NOTNULL(ls->transport, TRANSPORT_MAGIC);
 		assert (ls->sock > 0);		// We know where stdin is
 		if (cache_param->tcp_fastopen) {
 			int i;
@@ -578,10 +588,8 @@ ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
 /*--------------------------------------------------------------------*/
 
 static struct cli_proto vca_cmds[] = {
-	{ CLI_SERVER_START,	"i", ccf_start },
-	{ "debug.listen_address", "debug.listen_address",
-	    "\tReport the actual listen address.", 0, 0,
-	    "d", ccf_listen_address, NULL },
+	{ CLICMD_SERVER_START,		"", ccf_start },
+	{ CLICMD_DEBUG_LISTEN_ADDRESS,	"d", ccf_listen_address },
 	{ NULL }
 };
 
@@ -605,3 +613,52 @@ VCA_Shutdown(void)
 		(void)close(i);
 	}
 }
+
+/*--------------------------------------------------------------------
+ * Transport protocol registration
+ *
+ */
+
+static VTAILQ_HEAD(,transport)	transports =
+    VTAILQ_HEAD_INITIALIZER(transports);
+
+void
+XPORT_Init(void)
+{
+	uint16_t n;
+	struct transport *xp;
+
+	ASSERT_MGT();
+
+	VTAILQ_INSERT_TAIL(&transports, &PROXY_transport, list);
+	VTAILQ_INSERT_TAIL(&transports, &HTTP1_transport, list);
+	VTAILQ_INSERT_TAIL(&transports, &H2_transport, list);
+
+	n = 0;
+	VTAILQ_FOREACH(xp, &transports, list)
+		xp->number = ++n;
+}
+
+const struct transport *
+XPORT_Find(const char *name)
+{
+	const struct transport *xp;
+
+	ASSERT_MGT();
+
+	VTAILQ_FOREACH(xp, &transports, list)
+		if (!strcasecmp(xp->name, name))
+			return (xp);
+	return (NULL);
+}
+
+const struct transport *
+XPORT_ByNumber(uint16_t no)
+{
+	const struct transport *xp;
+
+	VTAILQ_FOREACH(xp, &transports, list)
+		if (xp->number == no)
+			return (xp);
+	return (NULL);
+}
