@@ -44,7 +44,7 @@
 #include "cache.h"
 
 #include "binary_heap.h"
-#include "vcli_priv.h"
+#include "vcli_serve.h"
 #include "vrt.h"
 #include "vsa.h"
 #include "vtcp.h"
@@ -474,7 +474,7 @@ vbp_build_req(struct vbp_target *vt, const struct vrt_backend_probe *vbp,
 	vt->req = strdup(VSB_data(vsb));
 	AN(vt->req);
 	vt->req_len = VSB_len(vsb);
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 }
 
 /*--------------------------------------------------------------------
@@ -627,5 +627,5 @@ VBP_Init(void)
 	vbp_heap = binheap_new(NULL, vbp_cmp, vbp_update);
 	AN(vbp_heap);
 	AZ(pthread_cond_init(&vbp_cond, NULL));
-	WRK_BgThread(&thr, "Backend poller", vbp_thread, NULL);
+	WRK_BgThread(&thr, "backend-poller", vbp_thread, NULL);
 }
