@@ -36,7 +36,8 @@
 #include "hash/hash_slinger.h"
 #include "vtim.h"
 
-static struct objcore oc_marker = { .magic = OBJCORE_MAGIC, };
+static struct objcore oc_mark_cnt = { .magic = OBJCORE_MAGIC, };
+static struct objcore oc_mark_end = { .magic = OBJCORE_MAGIC, };
 static unsigned ban_batch;
 static unsigned ban_generation;
 
@@ -51,13 +52,27 @@ ban_kick_lurker(void)
 	AZ(pthread_cond_signal(&ban_lurker_cond));
 }
 
-static void
-ban_cleantail(void)
-{
-	struct ban *b;
+/*
+ * ban_cleantail: clean the tail of the ban list up to the first ban which is
+ * still referenced. For already completed bans, we update statistics
+ * accordingly, but otherwise just skip the completion step and remove directly
+ *
+ * return 1 if we removed the victim, 0 otherwise
+ */
 
+static int
+ban_cleantail(const struct ban *victim)
+{
+	struct ban *b, *bt;
+	struct banhead_s freelist = VTAILQ_HEAD_INITIALIZER(freelist);
+	int r = 0;
+
+	/* handle the zero-length tail unprotected */
+	if (VTAILQ_LAST(&ban_head, banhead_s) == VTAILQ_FIRST(&ban_head))
+		return (r);
+
+	Lck_Lock(&ban_mtx);
 	do {
-		Lck_Lock(&ban_mtx);
 		b = VTAILQ_LAST(&ban_head, banhead_s);
 		if (b != VTAILQ_FIRST(&ban_head) && b->refcount == 0) {
 			assert(VTAILQ_EMPTY(&b->objcore));
@@ -70,16 +85,24 @@ ban_cleantail(void)
 			VSC_C_main->bans--;
 			VSC_C_main->bans_deleted++;
 			VTAILQ_REMOVE(&ban_head, b, list);
+			VTAILQ_INSERT_TAIL(&freelist, b, list);
 			VSC_C_main->bans_persisted_fragmentation +=
 			    ban_len(b->spec);
-			ban_info(BI_DROP, b->spec, ban_len(b->spec));
+			ban_info_drop(b->spec, ban_len(b->spec));
 		} else {
 			b = NULL;
 		}
-		Lck_Unlock(&ban_mtx);
-		if (b != NULL)
-			BAN_Free(b);
 	} while (b != NULL);
+
+	Lck_Unlock(&ban_mtx);
+
+	VTAILQ_FOREACH_SAFE(b, &freelist, list, bt) {
+		if (b == victim)
+			r = 1;
+		BAN_Free(b);
+	}
+
+	return (r);
 }
 
 /*--------------------------------------------------------------------
@@ -88,23 +111,56 @@ ban_cleantail(void)
  * makes most sense in HSH_Lookup(), but we come the other way.
  * We optimistically try to get them the other way, and get out of
  * the way if that fails, and retry again later.
+ *
+ * To avoid hammering on contested ocs, we first move those behind a marker
+ * once. When we only have contested ocs left, we stop moving them around and
+ * re-try them in order.
  */
 
 static struct objcore *
 ban_lurker_getfirst(struct vsl_log *vsl, struct ban *bt)
 {
 	struct objhead *oh;
-	struct objcore *oc;
+	struct objcore *oc, *noc;
+	int move_oc = 1;
 
+	Lck_Lock(&ban_mtx);
+
+	oc = VTAILQ_FIRST(&bt->objcore);
 	while (1) {
-		Lck_Lock(&ban_mtx);
-		oc = VTAILQ_FIRST(&bt->objcore);
 		CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
-		if (oc == &oc_marker) {
-			VTAILQ_REMOVE(&bt->objcore, oc, ban_list);
+
+		if (oc == &oc_mark_cnt) {
+			if (VTAILQ_NEXT(oc, ban_list) == &oc_mark_end) {
+				/* done with this ban's oc list */
+				VTAILQ_REMOVE(&bt->objcore, &oc_mark_cnt,
+				    ban_list);
+				VTAILQ_REMOVE(&bt->objcore, &oc_mark_end,
+				    ban_list);
+				oc = NULL;
+				break;
+			}
+			oc = VTAILQ_NEXT(oc, ban_list);
+			CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+			move_oc = 0;
+		} else if (oc == &oc_mark_end) {
+			assert(move_oc == 0);
+
+			/* hold off to give lookup a chance and reiterate */
 			Lck_Unlock(&ban_mtx);
-			return (NULL);
+			VSC_C_main->bans_lurker_contention++;
+			VSL_Flush(vsl, 0);
+			VTIM_sleep(cache_param->ban_lurker_holdoff);
+			Lck_Lock(&ban_mtx);
+
+			oc = VTAILQ_FIRST(&bt->objcore);
+			assert(oc == &oc_mark_cnt);
+			continue;
 		}
+
+		assert(oc != &oc_mark_cnt);
+		assert(oc != &oc_mark_end);
+
 		oh = oc->objhead;
 		CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
 		if (!Lck_Trylock(&oh->mtx)) {
@@ -113,26 +169,28 @@ ban_lurker_getfirst(struct vsl_log *vsl, struct ban *bt)
 			} else {
 				/*
 				 * We got the lock, and the oc is not being
-				 * dismantled under our feet.
-				 * Take it off the ban and (optimistically)
-				 * put it on the * destination ban
+				 * dismantled under our feet - grab a ref
 				 */
 				AZ(oc->flags & OC_F_BUSY);
 				oc->refcnt += 1;
 				VTAILQ_REMOVE(&bt->objcore, oc, ban_list);
 				VTAILQ_INSERT_TAIL(&bt->objcore, oc, ban_list);
 				Lck_Unlock(&oh->mtx);
-				Lck_Unlock(&ban_mtx);
 				break;
 			}
 		}
 
-		/* Try again, later */
-		Lck_Unlock(&ban_mtx);
-		VSC_C_main->bans_lurker_contention++;
-		VSL_Flush(vsl, 0);
-		VTIM_sleep(cache_param->ban_lurker_sleep);
+		noc = VTAILQ_NEXT(oc, ban_list);
+
+		if (move_oc) {
+			/* contested ocs go between the two markers */
+			VTAILQ_REMOVE(&bt->objcore, oc, ban_list);
+			VTAILQ_INSERT_BEFORE(&oc_mark_end, oc, ban_list);
+		}
+
+		oc = noc;
 	}
+	Lck_Unlock(&ban_mtx);
 	return (oc);
 }
 
@@ -146,12 +204,14 @@ ban_lurker_test_ban(struct worker *wrk, struct vsl_log *vsl, struct ban *bt,
 	int i;
 
 	/*
-	 * First see if there is anything to do, and if so, insert marker
+	 * First see if there is anything to do, and if so, insert markers
 	 */
 	Lck_Lock(&ban_mtx);
 	oc = VTAILQ_FIRST(&bt->objcore);
-	if (oc != NULL)
-		VTAILQ_INSERT_TAIL(&bt->objcore, &oc_marker, ban_list);
+	if (oc != NULL) {
+		VTAILQ_INSERT_TAIL(&bt->objcore, &oc_mark_cnt, ban_list);
+		VTAILQ_INSERT_TAIL(&bt->objcore, &oc_mark_end, ban_list);
+	}
 	Lck_Unlock(&ban_mtx);
 	if (oc == NULL)
 		return;
@@ -166,6 +226,13 @@ ban_lurker_test_ban(struct worker *wrk, struct vsl_log *vsl, struct ban *bt,
 			return;
 		i = 0;
 		VTAILQ_FOREACH_REVERSE_SAFE(bl, obans, banhead_s, l_list, bln) {
+			if (oc->ban != bt) {
+				/*
+				 * HSH_Lookup() grabbed this oc, killed
+				 * it or tested it to top.  We're done.
+				 */
+				break;
+			}
 			if (bl->flags & BANS_FLAG_COMPLETED) {
 				/* Ban was overtaken by new (dup) ban */
 				VTAILQ_REMOVE(obans, bl, l_list);
@@ -176,34 +243,40 @@ ban_lurker_test_ban(struct worker *wrk, struct vsl_log *vsl, struct ban *bt,
 			i = ban_evaluate(wrk, bl->spec, oc, NULL, &tests);
 			VSC_C_main->bans_lurker_tested++;
 			VSC_C_main->bans_lurker_tests_tested += tests;
-			if (i)
+			if (i) {
+				VSLb(vsl, SLT_ExpBan, "%u banned by lurker",
+				    ObjGetXID(wrk, oc));
+				HSH_Kill(oc);
+				VSC_C_main->bans_lurker_obj_killed++;
 				break;
+			}
 		}
-		if (i) {
-			VSLb(vsl, SLT_ExpBan, "%u banned by lurker",
-			    ObjGetXID(wrk, oc));
-
-			EXP_Rearm(oc, oc->exp.t_origin, 0, 0, 0);
-					// XXX ^ fake now
-			VSC_C_main->bans_lurker_obj_killed++;
-		} else {
-			if (oc->ban != bd) {
-				Lck_Lock(&ban_mtx);
-				oc->ban->refcount--;
-				VTAILQ_REMOVE(&oc->ban->objcore, oc, ban_list);
+		if (i == 0 && oc->ban == bt) {
+			Lck_Lock(&ban_mtx);
+			if (oc->ban == bt) {
+				bt->refcount--;
+				VTAILQ_REMOVE(&bt->objcore, oc, ban_list);
 				oc->ban = bd;
 				bd->refcount++;
 				VTAILQ_INSERT_TAIL(&bd->objcore, oc, ban_list);
-				Lck_Unlock(&ban_mtx);
-				ObjUpdateMeta(wrk, oc);
+				i = 1;
 			}
+			Lck_Unlock(&ban_mtx);
+			if (i)
+				ObjSendEvent(wrk, oc, OEV_BANCHG);
 		}
-		(void)HSH_DerefObjCore(wrk, &oc);
+		(void)HSH_DerefObjCore(wrk, &oc, 0);
 	}
 }
 
 /*--------------------------------------------------------------------
- * Ban lurker thread
+ * Ban lurker thread:
+ *
+ * try to move ocs as far up the ban list as possible (to bd)
+ *
+ * BANS_FLAG_REQ bans act as barriers, for bans further down, ocs get moved to
+ * them. But still all bans up to the initial bd get checked and marked
+ * completed.
  */
 
 static double
@@ -214,8 +287,10 @@ ban_lurker_work(struct worker *wrk, struct vsl_log *vsl)
 	double d, dt, n;
 
 	dt = 49.62;		// Random, non-magic
-	if (cache_param->ban_lurker_sleep == 0)
+	if (cache_param->ban_lurker_sleep == 0) {
+		(void) ban_cleantail(NULL);
 		return (dt);
+	}
 
 	Lck_Lock(&ban_mtx);
 	b = ban_start;
@@ -243,9 +318,25 @@ ban_lurker_work(struct worker *wrk, struct vsl_log *vsl)
 		}
 	}
 
+	/*
+	 * conceptually, all obans are now completed. Remove the tail. If it
+	 * containted the first oban, all obans were on the tail and we're
+	 * done.
+	 */
+	if (ban_cleantail(VTAILQ_FIRST(&obans)))
+		return (dt);
+
+	/*
+	 * Mark remaining bans completed: the tail of the obans list is now
+	 * removed, but iterating over it is safe until we hit the new tail ban
+	 */
 	Lck_Lock(&ban_mtx);
-	VTAILQ_FOREACH(b, &obans, l_list)
+	bd = VTAILQ_LAST(&ban_head, banhead_s);
+	VTAILQ_FOREACH(b, &obans, l_list) {
 		ban_mark_completed(b);
+		if (b == bd)
+			break;
+	}
 	Lck_Unlock(&ban_mtx);
 	return (dt);
 }
@@ -264,7 +355,6 @@ ban_lurker(struct worker *wrk, void *priv)
 
 	while (!ban_shutdown) {
 		d = ban_lurker_work(wrk, &vsl);
-		ban_cleantail();
 		if (DO_DEBUG(DBG_LURKER))
 			VSLb(&vsl, SLT_Debug, "lurker: sleep = %lf", d);
 		d += VTIM_real();
