@@ -33,8 +33,7 @@
 
 #include "cache.h"
 #include "cache_filter.h"
-
-#include "vcli_priv.h"
+#include "vcli_serve.h"
 
 static unsigned fetchfrag;
 
@@ -72,7 +71,6 @@ VFP_GetStorage(struct vfp_ctx *vc, ssize_t *sz, uint8_t **ptr)
 {
 	ssize_t l;
 
-
 	CHECK_OBJ_NOTNULL(vc, VFP_CTX_MAGIC);
 	AN(sz);
 	assert(*sz >= 0);
@@ -94,6 +92,14 @@ VFP_GetStorage(struct vfp_ctx *vc, ssize_t *sz, uint8_t **ptr)
 	return (VFP_OK);
 }
 
+void
+VFP_Extend(const struct vfp_ctx *vc, ssize_t sz)
+{
+	CHECK_OBJ_NOTNULL(vc, VFP_CTX_MAGIC);
+
+	ObjExtend(vc->wrk, vc->oc, sz);
+}
+
 /**********************************************************************
  */
 
@@ -196,8 +202,11 @@ VFP_Push(struct vfp_ctx *vc, const struct vfp *vfp, int top)
 
 	CHECK_OBJ_NOTNULL(vc, VFP_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vc->http, HTTP_MAGIC);
+
 	vfe = WS_Alloc(vc->http->ws, sizeof *vfe);
-	AN(vfe);
+	if (vfe == NULL)
+		return (NULL);
+
 	INIT_OBJ(vfe, VFP_ENTRY_MAGIC);
 	vfe->vfp = vfp;
 	vfe->closed = VFP_OK;
@@ -223,8 +232,7 @@ debug_fragfetch(struct cli *cli, const char * const *av, void *priv)
 }
 
 static struct cli_proto debug_cmds[] = {
-	{ "debug.fragfetch", "debug.fragfetch",
-		"\tEnable fetch fragmentation.", 1, 1, "d", debug_fragfetch },
+	{ CLICMD_DEBUG_FRAGFETCH,		"d", debug_fragfetch },
 	{ NULL }
 };
 
