@@ -285,7 +285,7 @@ VDP_gunzip(struct req *req, enum vdp_action act, void **priv,
 	const void *dp;
 	struct worker *wrk;
 	struct vgz *vg;
-	char *p;
+	const char *p;
 	uint64_t u;
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -307,10 +307,10 @@ VDP_gunzip(struct req *req, enum vdp_action act, void **priv,
 		http_Unset(req->resp, H_Content_Encoding);
 
 		req->resp_len = -1;
-		if (req->objcore->flags & OC_F_INCOMPLETE)
-			return (0);	/* No idea about length */
+		if (req->objcore->boc != NULL)
+			return (0);	/* No idea about length (yet) */
 
-		p = ObjGetattr(req->wrk, req->objcore, OA_GZIPBITS, &dl);
+		p = ObjGetAttr(req->wrk, req->objcore, OA_GZIPBITS, &dl);
 		if (p == NULL || dl != 32)
 			return (0); /* No OA_GZIPBITS yet */
 
@@ -369,7 +369,7 @@ VGZ_UpdateObj(const struct vfp_ctx *vc, struct vgz *vg, enum vgz_ua_e e)
 	if (e == VUA_UPDATE && ii == vg->bits)
 		return;
 	vg->bits = ii;
-	p = ObjSetattr(vc->wrk, vc->oc, OA_GZIPBITS, 32, NULL);
+	p = ObjSetAttr(vc->wrk, vc->oc, OA_GZIPBITS, 32, NULL);
 	AN(p);
 	vbe64enc(p, vg->vz.start_bit);
 	vbe64enc(p + 8, vg->vz.last_bit);
