@@ -56,28 +56,23 @@
 #include <stdlib.h>
 
 #include "cache.h"
-
+#include "cache/cache_transport.h"
 
 #include "hash/hash_slinger.h"
 #include "vsha256.h"
 #include "vtim.h"
 
+struct rush {
+	unsigned		magic;
+#define RUSH_MAGIC		0xa1af5f01
+	VTAILQ_HEAD(,req)	reqs;
+};
+
 static const struct hash_slinger *hash;
 static struct objhead *private_oh;
 
-/*---------------------------------------------------------------------*/
-
-static struct objcore *
-hsh_NewObjCore(struct worker *wrk)
-{
-	struct objcore *oc;
-
-	ALLOC_OBJ(oc, OBJCORE_MAGIC);
-	XXXAN(oc);
-	wrk->stats->n_objectcore++;
-	oc->flags |= OC_F_BUSY | OC_F_INCOMPLETE;
-	return (oc);
-}
+static void hsh_rush1(struct worker *, struct objhead *, struct rush *, int);
+static void hsh_rush2(struct worker *, struct rush *);
 
 /*---------------------------------------------------------------------*/
 
@@ -90,6 +85,7 @@ hsh_newobjhead(void)
 	XXXAN(oh);
 	oh->refcnt = 1;
 	VTAILQ_INIT(&oh->objcs);
+	VTAILQ_INIT(&oh->waitinglist);
 	Lck_New(&oh->mtx, lck_objhdr);
 	return (oh);
 }
@@ -99,12 +95,11 @@ hsh_newobjhead(void)
 static void
 hsh_prealloc(struct worker *wrk)
 {
-	struct waitinglist *wl;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 
 	if (wrk->nobjcore == NULL)
-		wrk->nobjcore = hsh_NewObjCore(wrk);
+		wrk->nobjcore = ObjNew(wrk);
 	CHECK_OBJ_NOTNULL(wrk->nobjcore, OBJCORE_MAGIC);
 
 	if (wrk->nobjhead == NULL) {
@@ -113,15 +108,6 @@ hsh_prealloc(struct worker *wrk)
 	}
 	CHECK_OBJ_NOTNULL(wrk->nobjhead, OBJHEAD_MAGIC);
 
-	if (wrk->nwaitinglist == NULL) {
-		ALLOC_OBJ(wl, WAITINGLIST_MAGIC);
-		XXXAN(wl);
-		VTAILQ_INIT(&wl->list);
-		wrk->nwaitinglist = wl;
-		wrk->stats->n_waitinglist++;
-	}
-	CHECK_OBJ_NOTNULL(wrk->nwaitinglist, WAITINGLIST_MAGIC);
-
 	if (hash->prep != NULL)
 		hash->prep(wrk);
 }
@@ -135,13 +121,13 @@ HSH_Private(struct worker *wrk)
 
 	CHECK_OBJ_NOTNULL(private_oh, OBJHEAD_MAGIC);
 
-	oc = hsh_NewObjCore(wrk);
+	oc = ObjNew(wrk);
 	AN(oc);
 	oc->refcnt = 1;
 	oc->objhead = private_oh;
-	oc->flags |= OC_F_PRIVATE | OC_F_BUSY;
+	oc->flags |= OC_F_PRIVATE;
 	Lck_Lock(&private_oh->mtx);
-	VTAILQ_INSERT_TAIL(&private_oh->objcs, oc, list);
+	VTAILQ_INSERT_TAIL(&private_oh->objcs, oc, hsh_list);
 	private_oh->refcnt++;
 	Lck_Unlock(&private_oh->mtx);
 	return (oc);
@@ -152,22 +138,15 @@ void
 HSH_Cleanup(struct worker *wrk)
 {
 
-	if (wrk->nobjcore != NULL) {
-		FREE_OBJ(wrk->nobjcore);
-		wrk->stats->n_objectcore--;
-		wrk->nobjcore = NULL;
-	}
+	if (wrk->nobjcore != NULL)
+		ObjDestroy(wrk, &wrk->nobjcore);
+
 	if (wrk->nobjhead != NULL) {
 		Lck_Delete(&wrk->nobjhead->mtx);
 		FREE_OBJ(wrk->nobjhead);
 		wrk->nobjhead = NULL;
 		wrk->stats->n_objecthead--;
 	}
-	if (wrk->nwaitinglist != NULL) {
-		FREE_OBJ(wrk->nwaitinglist);
-		wrk->nwaitinglist = NULL;
-		wrk->stats->n_waitinglist--;
-	}
 	if (wrk->nhashpriv != NULL) {
 		/* XXX: If needed, add slinger method for this */
 		free(wrk->nhashpriv);
@@ -181,6 +160,7 @@ HSH_DeleteObjHead(struct worker *wrk, struct objhead *oh)
 
 	AZ(oh->refcnt);
 	assert(VTAILQ_EMPTY(&oh->objcs));
+	assert(VTAILQ_EMPTY(&oh->waitinglist));
 	Lck_Delete(&oh->mtx);
 	wrk->stats->n_objecthead--;
 	FREE_OBJ(oh);
@@ -281,13 +261,20 @@ hsh_testmagic(void *result)
  */
 
 void
-HSH_Insert(struct worker *wrk, const void *digest, struct objcore *oc)
+HSH_Insert(struct worker *wrk, const void *digest, struct objcore *oc,
+    struct ban *ban)
 {
 	struct objhead *oh;
+	struct rush rush;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	AN(digest);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+	AN(ban);
+	AN(oc->flags & OC_F_BUSY);
+	AZ(oc->flags & OC_F_PRIVATE);
+	assert(oc->refcnt == 1);
+	INIT_OBJ(&rush, RUSH_MAGIC);
 
 	hsh_prealloc(wrk);
 
@@ -297,17 +284,27 @@ HSH_Insert(struct worker *wrk, const void *digest, struct objcore *oc)
 	Lck_AssertHeld(&oh->mtx);
 	assert(oh->refcnt > 0);
 
-	/* Insert (precreated) objcore in objecthead */
-	oc->refcnt = 1;
-	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
-	AZ(oc->flags & OC_F_BUSY);
-
-	VTAILQ_INSERT_HEAD(&oh->objcs, oc, list);
-	/* NB: do not deref objhead the new object inherits our reference */
+	/* Mark object busy and insert (precreated) objcore in
+	   objecthead. The new object inherits our objhead reference. */
 	oc->objhead = oh;
+	VTAILQ_INSERT_TAIL(&oh->objcs, oc, hsh_list);
+	oc->refcnt++;				// For EXP_Insert
 	Lck_Unlock(&oh->mtx);
-	wrk->stats->n_objectcore++;
-	wrk->stats->n_vampireobject++;
+
+	BAN_RefBan(oc, ban);
+	AN(oc->ban);
+	EXP_Insert(wrk, oc);
+
+	/* Move the object first in the oh list, unbusy it and run the
+	   waitinglist if necessary */
+	Lck_Lock(&oh->mtx);
+	VTAILQ_REMOVE(&oh->objcs, oc, hsh_list);
+	VTAILQ_INSERT_HEAD(&oh->objcs, oc, hsh_list);
+	oc->flags &= ~OC_F_BUSY;
+	if (!VTAILQ_EMPTY(&oh->waitinglist))
+		hsh_rush1(wrk, oh, &rush, HSH_RUSH_POLICY);
+	Lck_Unlock(&oh->mtx);
+	hsh_rush2(wrk, &rush);
 }
 
 /*---------------------------------------------------------------------
@@ -328,7 +325,7 @@ hsh_insert_busyobj(struct worker *wrk, struct objhead *oh)
 	AN(oc->flags & OC_F_BUSY);
 	oc->refcnt = 1;		/* Owned by busyobj */
 	oc->objhead = oh;
-	VTAILQ_INSERT_TAIL(&oh->objcs, oc, list);
+	VTAILQ_INSERT_TAIL(&oh->objcs, oc, hsh_list);
 	return (oc);
 }
 
@@ -337,7 +334,7 @@ hsh_insert_busyobj(struct worker *wrk, struct objhead *oh)
 
 enum lookup_e
 HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
-    int wait_for_busy, int always_insert)
+    int always_insert)
 {
 	struct worker *wrk;
 	struct objhead *oh;
@@ -346,7 +343,7 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 	double exp_t_origin;
 	int busy_found;
 	enum lookup_e retval;
-	uint8_t *vary;
+	const uint8_t *vary;
 
 	AN(ocp);
 	*ocp = NULL;
@@ -393,61 +390,80 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 	busy_found = 0;
 	exp_oc = NULL;
 	exp_t_origin = 0.0;
-	VTAILQ_FOREACH(oc, &oh->objcs, list) {
+	VTAILQ_FOREACH(oc, &oh->objcs, hsh_list) {
 		/* Must be at least our own ref + the objcore we examine */
 		assert(oh->refcnt > 1);
 		CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 		assert(oc->objhead == oh);
+		assert(oc->refcnt > 0);
 
-		if (oc->exp_flags & OC_EF_DYING)
+		if (oc->flags & OC_F_DYING)
 			continue;
 		if (oc->flags & OC_F_FAILED)
 			continue;
 
-		if (oc->flags & OC_F_BUSY) {
-			CHECK_OBJ_ORNULL(oc->busyobj, BUSYOBJ_MAGIC);
+		if (oc->boc != NULL && oc->boc->state < BOS_STREAM) {
+			CHECK_OBJ_ORNULL(oc->boc, BOC_MAGIC);
+
 			if (req->hash_ignore_busy)
 				continue;
 
-			if (oc->busyobj != NULL &&
-			    oc->busyobj->vary != NULL &&
-			    !VRY_Match(req, oc->busyobj->vary))
+			if (oc->boc->vary != NULL &&
+			    !VRY_Match(req, oc->boc->vary))
 				continue;
 
 			busy_found = 1;
 			continue;
 		}
 
-		if (oc->exp.ttl <= 0.)
+		if (oc->ttl <= 0.)
 			continue;
 
-		if (BAN_CheckObject(wrk, oc, req))
+		if (BAN_CheckObject(wrk, oc, req)) {
+			oc->flags |= OC_F_DYING;
+			EXP_Remove(oc);
 			continue;
+		}
 
-		vary = ObjGetattr(wrk, oc, OA_VARY, NULL);
+		if (ObjHasAttr(wrk, oc, OA_VARY)) {
+			vary = ObjGetAttr(wrk, oc, OA_VARY, NULL);
+			if (!VRY_Match(req, vary))
+				continue;
+		}
 
-		if (vary != NULL && !VRY_Match(req, vary))
-			continue;
-
-		if (EXP_Ttl(req, &oc->exp) >= req->t_req) {
+		if (EXP_Ttl(req, oc) >= req->t_req) {
 			/* If still valid, use it */
 			assert(oh->refcnt > 1);
 			assert(oc->objhead == oh);
-			oc->refcnt++;
-			if (oc->hits < LONG_MAX)
-				oc->hits++;
+			if (oc->flags & OC_F_PASS) {
+				wrk->stats->cache_hitpass++;
+				oc = NULL;
+				*bocp = hsh_insert_busyobj(wrk, oh);
+			} else {
+				oc->refcnt++;
+				if (oc->hits < LONG_MAX)
+					oc->hits++;
+			}
 			Lck_Unlock(&oh->mtx);
+			if (oc == NULL)
+				return (HSH_MISS);
 			assert(HSH_DerefObjHead(wrk, &oh));
 			*ocp = oc;
 			return (HSH_HIT);
 		}
-		if (oc->exp.t_origin > exp_t_origin) {
+		if (oc->t_origin > exp_t_origin) {
 			/* record the newest object */
 			exp_oc = oc;
-			exp_t_origin = oc->exp.t_origin;
+			exp_t_origin = oc->t_origin;
 		}
 	}
 
+	if (exp_oc != NULL && exp_oc->flags & OC_F_PASS) {
+		wrk->stats->cache_hitpass++;
+		exp_oc = NULL;
+		busy_found = 0;
+	}
+
 	if (exp_oc != NULL) {
 		assert(oh->refcnt > 1);
 		assert(exp_oc->objhead == oh);
@@ -481,20 +497,9 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 
 	AZ(req->hash_ignore_busy);
 
-	if (wait_for_busy) {
-		CHECK_OBJ_NOTNULL(wrk->nwaitinglist, WAITINGLIST_MAGIC);
-		if (oh->waitinglist == NULL) {
-			oh->waitinglist = wrk->nwaitinglist;
-			wrk->nwaitinglist = NULL;
-		}
-		VTAILQ_INSERT_TAIL(&oh->waitinglist->list,
-		    req, w_list);
-		if (DO_DEBUG(DBG_WAITINGLIST))
-			VSLb(req->vsl, SLT_Debug, "on waiting list <%p>", oh);
-	} else {
-		if (DO_DEBUG(DBG_WAITINGLIST))
-			VSLb(req->vsl, SLT_Debug, "hit busy obj <%p>", oh);
-	}
+	VTAILQ_INSERT_TAIL(&oh->waitinglist, req, w_list);
+	if (DO_DEBUG(DBG_WAITINGLIST))
+		VSLb(req->vsl, SLT_Debug, "on waiting list <%p>", oh);
 
 	wrk->stats->busy_sleep++;
 	/*
@@ -504,67 +509,64 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 	 */
 	req->hash_objhead = oh;
 	req->wrk = NULL;
+	req->waitinglist = 1;
 	Lck_Unlock(&oh->mtx);
 	return (HSH_BUSY);
 }
 
 /*---------------------------------------------------------------------
+ * Pick the req's we are going to rush from the waiting list
  */
 
 static void
-hsh_rush(struct worker *wrk, struct objhead *oh)
+hsh_rush1(struct worker *wrk, struct objhead *oh, struct rush *r, int max)
 {
 	unsigned u;
 	struct req *req;
-	struct sess *sp;
-	struct waitinglist *wl;
+
+	if (max == 0)
+		return;
+	if (max == HSH_RUSH_POLICY)
+		max = cache_param->rush_exponent;
+	assert(max > 0);
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
+	CHECK_OBJ_NOTNULL(r, RUSH_MAGIC);
+	VTAILQ_INIT(&r->reqs);
 	Lck_AssertHeld(&oh->mtx);
-	wl = oh->waitinglist;
-	CHECK_OBJ_ORNULL(wl, WAITINGLIST_MAGIC);
-	if (wl == NULL)
-		return;
-	for (u = 0; u < cache_param->rush_exponent; u++) {
-		req = VTAILQ_FIRST(&wl->list);
+	for (u = 0; u < max; u++) {
+		req = VTAILQ_FIRST(&oh->waitinglist);
 		if (req == NULL)
 			break;
 		CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 		wrk->stats->busy_wakeup++;
 		AZ(req->wrk);
-		VTAILQ_REMOVE(&wl->list, req, w_list);
-		DSL(DBG_WAITINGLIST, req->vsl->wid, "off waiting list");
-		if (SES_Reschedule_Req(req)) {
-			/*
-			 * In case of overloads, we ditch the entire
-			 * waiting list.
-			 */
-			wrk->stats->busy_wakeup--;
-			while (1) {
-				wrk->stats->busy_killed++;
-				AN (req->vcl);
-				VCL_Rel(&req->vcl);
-				sp = req->sp;
-				CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-				CNT_AcctLogCharge(wrk->stats, req);
-				Req_Release(req);
-				SES_Delete(sp, SC_OVERLOAD, NAN);
-				req = VTAILQ_FIRST(&wl->list);
-				if (req == NULL)
-					break;
-				CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-				VTAILQ_REMOVE(&wl->list, req, w_list);
-				DSL(DBG_WAITINGLIST, req->vsl->wid,
-				    "kill from waiting list");
-			}
-			break;
-		}
+		VTAILQ_REMOVE(&oh->waitinglist, req, w_list);
+		VTAILQ_INSERT_TAIL(&r->reqs, req, w_list);
+		req->waitinglist = 0;
 	}
-	if (VTAILQ_EMPTY(&wl->list)) {
-		oh->waitinglist = NULL;
-		FREE_OBJ(wl);
-		wrk->stats->n_waitinglist--;
+}
+
+/*---------------------------------------------------------------------
+ * Rush req's that came from waiting list.
+ */
+
+static void
+hsh_rush2(struct worker *wrk, struct rush *r)
+{
+	struct req *req;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(r, RUSH_MAGIC);
+
+	while(!VTAILQ_EMPTY(&r->reqs)) {
+		req = VTAILQ_FIRST(&r->reqs);
+		CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+		VTAILQ_REMOVE(&r->reqs, req, w_list);
+		DSL(DBG_WAITINGLIST, req->vsl->wid, "off waiting list");
+		AN(req->transport->reembark);
+		req->transport->reembark(wrk, req);
 	}
 }
 
@@ -593,7 +595,7 @@ double keep)
 		Lck_Lock(&oh->mtx);
 		assert(oh->refcnt > 0);
 		now = VTIM_real();
-		VTAILQ_FOREACH(oc, &oh->objcs, list) {
+		VTAILQ_FOREACH(oc, &oh->objcs, hsh_list) {
 			CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 			assert(oc->objhead == oh);
 			if (oc->flags & OC_F_BUSY) {
@@ -606,7 +608,7 @@ double keep)
 				 */
 				continue;
 			}
-			if (oc->exp_flags & OC_EF_DYING)
+			if (oc->flags & OC_F_DYING)
 				continue;
 			if (spc < sizeof *ocp) {
 				/* Iterate if aws is not big enough */
@@ -623,7 +625,7 @@ double keep)
 			oc = ocp[n];
 			CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 			EXP_Rearm(oc, now, ttl, grace, keep);
-			(void)HSH_DerefObjCore(wrk, &oc);
+			(void)HSH_DerefObjCore(wrk, &oc, 0);
 		}
 	} while (more);
 	WS_Release(wrk->aws, 0);
@@ -652,17 +654,15 @@ HSH_Fail(struct objcore *oc)
 
 	Lck_Lock(&oh->mtx);
 	oc->flags |= OC_F_FAILED;
-	oc->flags &= ~OC_F_INCOMPLETE;
-	oc->busyobj = NULL;
 	Lck_Unlock(&oh->mtx);
 }
 
 /*---------------------------------------------------------------------
- * Remove the busyobj from an objcore
+ * Abandon a fetch we will not need
  */
 
 void
-HSH_Complete(struct objcore *oc)
+HSH_Abandon(struct objcore *oc)
 {
 	struct objhead *oh;
 
@@ -671,8 +671,7 @@ HSH_Complete(struct objcore *oc)
 	CHECK_OBJ(oh, OBJHEAD_MAGIC);
 
 	Lck_Lock(&oh->mtx);
-	oc->busyobj = NULL;
-	oc->flags &= ~OC_F_INCOMPLETE;
+	oc->flags |= OC_F_ABANDON;
 	Lck_Unlock(&oh->mtx);
 }
 
@@ -684,35 +683,90 @@ void
 HSH_Unbusy(struct worker *wrk, struct objcore *oc)
 {
 	struct objhead *oh;
+	struct rush rush;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	oh = oc->objhead;
 	CHECK_OBJ(oh, OBJHEAD_MAGIC);
+	INIT_OBJ(&rush, RUSH_MAGIC);
 
 	AN(oc->stobj->stevedore);
 	AN(oc->flags & OC_F_BUSY);
 	assert(oh->refcnt > 0);
+	assert(oc->refcnt > 0);
 
 	if (!(oc->flags & OC_F_PRIVATE)) {
 		BAN_NewObjCore(oc);
-		EXP_Insert(wrk, oc);
-		AN(oc->exp_flags & OC_EF_EXP);
 		AN(oc->ban);
 	}
 
 	/* XXX: pretouch neighbors on oh->objcs to prevent page-on under mtx */
 	Lck_Lock(&oh->mtx);
 	assert(oh->refcnt > 0);
+	assert(oc->refcnt > 0);
+	if (!(oc->flags & OC_F_PRIVATE))
+		oc->refcnt++;			// For EXP_Insert
 	/* XXX: strictly speaking, we should sort in Date: order. */
-	VTAILQ_REMOVE(&oh->objcs, oc, list);
-	VTAILQ_INSERT_HEAD(&oh->objcs, oc, list);
+	VTAILQ_REMOVE(&oh->objcs, oc, hsh_list);
+	VTAILQ_INSERT_HEAD(&oh->objcs, oc, hsh_list);
 	oc->flags &= ~OC_F_BUSY;
-	if (oh->waitinglist != NULL)
-		hsh_rush(wrk, oh);
+	if (!VTAILQ_EMPTY(&oh->waitinglist))
+		hsh_rush1(wrk, oh, &rush, HSH_RUSH_POLICY);
 	Lck_Unlock(&oh->mtx);
+	if (!(oc->flags & OC_F_PRIVATE))
+		EXP_Insert(wrk, oc);
+	hsh_rush2(wrk, &rush);
 }
 
+/*====================================================================
+ * HSH_Kill()
+ *
+ * It's dead Jim, kick it...
+ */
+
+void
+HSH_Kill(struct objcore *oc)
+{
+
+	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+	CHECK_OBJ_NOTNULL(oc->objhead, OBJHEAD_MAGIC);
+
+	Lck_Lock(&oc->objhead->mtx);
+	oc->flags |= OC_F_DYING;
+	Lck_Unlock(&oc->objhead->mtx);
+	EXP_Remove(oc);
+}
+
+/*====================================================================
+ * HSH_Snipe()
+ *
+ * If objcore is idle, gain a ref and mark it dead.
+ */
+
+int
+HSH_Snipe(const struct worker *wrk, struct objcore *oc)
+{
+	int retval = 0;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+	CHECK_OBJ_NOTNULL(oc->objhead, OBJHEAD_MAGIC);
+
+	if (oc->refcnt == 1 && !Lck_Trylock(&oc->objhead->mtx)) {
+		if (oc->refcnt == 1 && !(oc->flags & OC_F_DYING)) {
+			oc->flags |= OC_F_DYING;
+			oc->refcnt++;
+			retval = 1;
+		}
+		Lck_Unlock(&oc->objhead->mtx);
+	}
+	if (retval)
+		EXP_Remove(oc);
+	return (retval);
+}
+
+
 /*---------------------------------------------------------------------
  * Gain a reference on an objcore
  */
@@ -735,23 +789,51 @@ HSH_Ref(struct objcore *oc)
  * Gain a reference on the busyobj, if the objcore has one
  */
 
-struct busyobj *
-HSH_RefBusy(const struct objcore *oc)
+struct boc *
+HSH_RefBoc(const struct objcore *oc)
 {
 	struct objhead *oh;
-	struct busyobj *bo;
+	struct boc *boc;
 
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	oh = oc->objhead;
 	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
+	if (oc->boc == NULL)
+		return (NULL);
 	Lck_Lock(&oh->mtx);
 	assert(oc->refcnt > 0);
-	bo = oc->busyobj;
-	CHECK_OBJ_ORNULL(bo, BUSYOBJ_MAGIC);
-	if (bo != NULL)
-		bo->refcount++;
+	boc = oc->boc;
+	CHECK_OBJ_ORNULL(boc, BOC_MAGIC);
+	if (boc != NULL) {
+		assert(boc->refcount > 0);
+		if (boc->state < BOS_FINISHED)
+			boc->refcount++;
+		else
+			boc = NULL;
+	}
 	Lck_Unlock(&oh->mtx);
-	return (bo);
+	return (boc);
+}
+
+void
+HSH_DerefBoc(struct worker *wrk, struct objcore *oc)
+{
+	struct boc *boc;
+	unsigned r;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+	boc = oc->boc;
+	CHECK_OBJ_NOTNULL(boc, BOC_MAGIC);
+	Lck_Lock(&oc->objhead->mtx);
+	assert(oc->refcnt > 0);
+	assert(boc->refcount > 0);
+	r = --boc->refcount;
+	if (r == 0)
+		oc->boc = NULL;
+	Lck_Unlock(&oc->objhead->mtx);
+	if (r == 0)
+		ObjBocDone(wrk, oc, &boc);
 }
 
 /*--------------------------------------------------------------------
@@ -761,10 +843,11 @@ HSH_RefBusy(const struct objcore *oc)
  */
 
 int
-HSH_DerefObjCore(struct worker *wrk, struct objcore **ocp)
+HSH_DerefObjCore(struct worker *wrk, struct objcore **ocp, int rushmax)
 {
 	struct objcore *oc;
 	struct objhead *oh;
+	struct rush rush;
 	unsigned r;
 
 	AN(ocp);
@@ -774,6 +857,7 @@ HSH_DerefObjCore(struct worker *wrk, struct objcore **ocp)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	assert(oc->refcnt > 0);
+	INIT_OBJ(&rush, RUSH_MAGIC);
 
 	oh = oc->objhead;
 	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
@@ -782,21 +866,23 @@ HSH_DerefObjCore(struct worker *wrk, struct objcore **ocp)
 	assert(oh->refcnt > 0);
 	r = --oc->refcnt;
 	if (!r)
-		VTAILQ_REMOVE(&oh->objcs, oc, list);
-	if (oh->waitinglist != NULL)
-		hsh_rush(wrk, oh);
+		VTAILQ_REMOVE(&oh->objcs, oc, hsh_list);
+	if (!VTAILQ_EMPTY(&oh->waitinglist))
+		hsh_rush1(wrk, oh, &rush, rushmax);
 	Lck_Unlock(&oh->mtx);
+	hsh_rush2(wrk, &rush);
 	if (r != 0)
 		return (r);
 
+	AZ(oc->exp_flags);
+
 	BAN_DestroyObj(oc);
 	AZ(oc->ban);
 
 	if (oc->stobj->stevedore != NULL)
 		ObjFreeObj(wrk, oc);
-	FREE_OBJ(oc);
+	ObjDestroy(wrk, &oc);
 
-	wrk->stats->n_objectcore--;
 	/* Drop our ref on the objhead */
 	assert(oh->refcnt > 0);
 	(void)HSH_DerefObjHead(wrk, &oh);
@@ -807,16 +893,15 @@ int
 HSH_DerefObjHead(struct worker *wrk, struct objhead **poh)
 {
 	struct objhead *oh;
+	struct rush rush;
 	int r;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
-	AN(poh);
-	oh = *poh;
-	*poh = NULL;
-	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
+	TAKE_OBJ_NOTNULL(oh, poh, OBJHEAD_MAGIC);
+	INIT_OBJ(&rush, RUSH_MAGIC);
 
 	if (oh == private_oh) {
-		AZ(oh->waitinglist);
+		assert(VTAILQ_EMPTY(&oh->waitinglist));
 		Lck_Lock(&oh->mtx);
 		assert(oh->refcnt > 1);
 		oh->refcnt--;
@@ -832,16 +917,14 @@ HSH_DerefObjHead(struct worker *wrk, struct objhead **poh)
 	 * just make the hold the same ref's as objcore, that would
 	 * confuse hashers.
 	 */
-	while (oh->waitinglist != NULL) {
-		Lck_Lock(&oh->mtx);
-		assert(oh->refcnt > 0);
-		r = oh->refcnt;
-		hsh_rush(wrk, oh);
+	Lck_Lock(&oh->mtx);
+	while (oh->refcnt == 1 && !VTAILQ_EMPTY(&oh->waitinglist)) {
+		hsh_rush1(wrk, oh, &rush, HSH_RUSH_ALL);
 		Lck_Unlock(&oh->mtx);
-		if (r > 1)
-			break;
-		usleep(100000);
+		hsh_rush2(wrk, &rush);
+		Lck_Lock(&oh->mtx);
 	}
+	Lck_Unlock(&oh->mtx);
 
 	assert(oh->refcnt > 0);
 	r = hash->deref(oh);
