@@ -114,23 +114,27 @@ http_fail(const struct http *hp)
 
 static struct http_msg {
 	unsigned	nbr;
+	const char	*status;
 	const char	*txt;
 } http_msg[] = {
-#define HTTP_RESP(n, t)	{ n, t},
+#define HTTP_RESP(n, t)	{ n, #n, t},
 #include "tbl/http_response.h"
-	{ 0, NULL }
+	{ 0, "0", NULL }
 };
 
 const char *
-http_Status2Reason(unsigned status)
+http_Status2Reason(unsigned status, const char **sstr)
 {
 	struct http_msg *mp;
 
 	status %= 1000;
 	assert(status >= 100);
 	for (mp = http_msg; mp->nbr != 0 && mp->nbr <= status; mp++)
-		if (mp->nbr == status)
+		if (mp->nbr == status) {
+			if (sstr)
+				*sstr = mp->status;
 			return (mp->txt);
+		}
 	return ("Unknown HTTP Status");
 }
 
@@ -720,6 +724,8 @@ void
 http_SetStatus(struct http *to, uint16_t status)
 {
 	char buf[4];
+	const char *reason;
+	const char *sstr = NULL;
 
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
 	/*
@@ -729,9 +735,15 @@ http_SetStatus(struct http *to, uint16_t status)
 	to->status = status;
 	status %= 1000;
 	assert(status >= 100);
-	bprintf(buf, "%03d", status);
-	http_PutField(to, HTTP_HDR_STATUS, buf);
-	http_SetH(to, HTTP_HDR_REASON, http_Status2Reason(status));
+
+	reason = http_Status2Reason(status, &sstr);
+	if (sstr) {
+		http_SetH(to, HTTP_HDR_STATUS, sstr);
+	} else {
+		bprintf(buf, "%03d", status);
+		http_PutField(to, HTTP_HDR_STATUS, buf);
+	}
+	http_SetH(to, HTTP_HDR_REASON, reason);
 }
 
 /*--------------------------------------------------------------------*/
@@ -752,11 +764,17 @@ http_GetMethod(const struct http *hp)
 void
 http_ForceField(const struct http *to, unsigned n, const char *t)
 {
+	int i;
+
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
 	assert(n < HTTP_HDR_FIRST);
 	AN(t);
-	if (to->hd[n].b == NULL || strcmp(to->hd[n].b, t))
+	if (to->hd[n].b == NULL || strcmp(to->hd[n].b, t)) {
+		i = (HTTP_HDR_UNSET - HTTP_HDR_METHOD);
+		i += to->logtag;
+		VSLbt(to->vsl, (enum VSL_tag_e)i, to->hd[n]);
 		http_SetH(to, n, t);
+	}
 }
 
 /*--------------------------------------------------------------------*/
@@ -770,9 +788,8 @@ http_PutResponse(struct http *to, const char *proto, uint16_t status,
 	if (proto != NULL)
 		http_SetH(to, HTTP_HDR_PROTO, proto);
 	http_SetStatus(to, status);
-	if (reason == NULL)
-		reason = http_Status2Reason(status);
-	http_SetH(to, HTTP_HDR_REASON, reason);
+	if (reason != NULL)
+		http_SetH(to, HTTP_HDR_REASON, reason);
 }
 
 /*--------------------------------------------------------------------
@@ -790,8 +807,7 @@ http_EstimateWS(const struct http *fm, unsigned how)
 	for (u = 0; u < fm->nhd; u++) {
 		if (u == HTTP_HDR_METHOD || u == HTTP_HDR_URL)
 			continue;
-		AN(fm->hd[u].b);
-		AN(fm->hd[u].e);
+		Tcheck(fm->hd[u]);
 		if (fm->hdf[u] & HDF_FILTER)
 			continue;
 #define HTTPH(a, b, c) \
@@ -833,8 +849,7 @@ HTTP_Encode(const struct http *fm, uint8_t *p0, unsigned l, unsigned how)
 	for (u = 0; u < fm->nhd; u++) {
 		if (u == HTTP_HDR_METHOD || u == HTTP_HDR_URL)
 			continue;
-		AN(fm->hd[u].b);
-		AN(fm->hd[u].e);
+		Tcheck(fm->hd[u]);
 		if (fm->hdf[u] & HDF_FILTER)
 			continue;
 #define HTTPH(a, b, c) \
@@ -896,7 +911,7 @@ uint16_t
 HTTP_GetStatusPack(struct worker *wrk, struct objcore *oc)
 {
 	const char *ptr;
-	ptr = ObjGetattr(wrk, oc, OA_HEADERS, NULL);
+	ptr = ObjGetAttr(wrk, oc, OA_HEADERS, NULL);
 	AN(ptr);
 
 	return(vbe16dec(ptr + 2));
@@ -915,7 +930,7 @@ HTTP_IterHdrPack(struct worker *wrk, struct objcore *oc, const char **p)
 	AN(p);
 
 	if (*p == NULL) {
-		ptr = ObjGetattr(wrk, oc, OA_HEADERS, NULL);
+		ptr = ObjGetAttr(wrk, oc, OA_HEADERS, NULL);
 		AN(ptr);
 		ptr += 4;	/* Skip nhd and status */
 		ptr = strchr(ptr, '\0') + 1;	/* Skip :proto: */
@@ -948,7 +963,7 @@ HTTP_GetHdrPack(struct worker *wrk, struct objcore *oc, const char *hdr)
 
 	if (hdr[0] == ':') {
 		/* Special cases */
-		ptr = ObjGetattr(wrk, oc, OA_HEADERS, NULL);
+		ptr = ObjGetAttr(wrk, oc, OA_HEADERS, NULL);
 		AN(ptr);
 		ptr += 4;	/* Skip nhd and status */
 
@@ -992,7 +1007,7 @@ HTTP_Merge(struct worker *wrk, struct objcore *oc, struct http *to)
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
 
-	ptr = ObjGetattr(wrk, oc, OA_HEADERS, NULL);
+	ptr = ObjGetAttr(wrk, oc, OA_HEADERS, NULL);
 	AN(ptr);
 
 	to->status = vbe16dec(ptr + 2);
@@ -1027,8 +1042,7 @@ http_filterfields(struct http *to, const struct http *fm, unsigned how)
 	to->nhd = HTTP_HDR_FIRST;
 	to->status = fm->status;
 	for (u = HTTP_HDR_FIRST; u < fm->nhd; u++) {
-		if (fm->hd[u].b == NULL)
-			continue;
+		Tcheck(fm->hd[u]);
 		if (fm->hdf[u] & HDF_FILTER)
 			continue;
 		Tcheck(fm->hd[u]);
@@ -1084,8 +1098,10 @@ http_CopyHome(const struct http *hp)
 	char *p;
 
 	for (u = 0; u < hp->nhd; u++) {
-		if (hp->hd[u].b == NULL)
+		if (hp->hd[u].b == NULL) {
+			assert(u < HTTP_HDR_FIRST);
 			continue;
+		}
 		if (hp->hd[u].b >= hp->ws->s && hp->hd[u].e <= hp->ws->e)
 			continue;
 
@@ -1142,7 +1158,8 @@ http_PrintfHeader(struct http *to, const char *fmt, ...)
 	va_end(ap);
 	if (n + 1 >= l || to->nhd >= to->shd) {
 		http_fail(to);
-		VSLb(to->vsl, SLT_LostHeader, "%s", to->ws->f);
+		VSLb(to->vsl, SLT_LostHeader, "%s",
+		    n + 1 >= l ? fmt : to->ws->f);
 		WS_Release(to->ws, 0);
 		return;
 	}
@@ -1183,8 +1200,7 @@ http_Unset(struct http *hp, const char *hdr)
 	uint16_t u, v;
 
 	for (v = u = HTTP_HDR_FIRST; u < hp->nhd; u++) {
-		if (hp->hd[u].b == NULL)
-			continue;
+		Tcheck(hp->hd[u]);
 		if (http_IsHdr(&hp->hd[u], hdr)) {
 			http_VSLH_del(hp, u);
 			continue;
