@@ -40,6 +40,7 @@
 #include <signal.h>
 
 #include "cache.h"
+#include "cache_transport.h"
 
 #include "cache_filter.h"
 #include "common/heritage.h"
@@ -47,7 +48,7 @@
 #include "vrt.h"
 #include "cache_director.h"
 #include "storage/storage.h"
-#include "vcli_priv.h"
+#include "vcli_serve.h"
 
 /*
  * The panic string is constructed in memory, then copied to the
@@ -93,6 +94,20 @@ reqbody_status_2str(enum req_body_state_e e)
 
 /*--------------------------------------------------------------------*/
 
+static const char *
+boc_state_2str(enum boc_state_e e)
+{
+	switch (e) {
+#define BOC_STATE(U,l) case BOS_##U: return(#l);
+#include "tbl/boc_state.h"
+#undef BOC_STATE
+	default:
+		return ("?");
+	}
+}
+
+/*--------------------------------------------------------------------*/
+
 const char *
 sess_close_2str(enum sess_close sc, int want_desc)
 {
@@ -107,6 +122,30 @@ sess_close_2str(enum sess_close sc, int want_desc)
 	}
 }
 
+/*--------------------------------------------------------------------*/
+
+#define N_ALREADY 64
+static const void *already_list[N_ALREADY];
+static int already_idx;
+
+static int
+pan_already(struct vsb *vsb, const void *ptr)
+{
+	int i;
+
+	for (i = 0; i < already_idx; i++) {
+		if (already_list[i] == ptr) {
+			VSB_printf(vsb, "  [Already dumped, see above]\n");
+			VSB_printf(vsb, "},\n");
+			return (1);
+		}
+	}
+	if (already_idx < N_ALREADY)
+		already_list[already_idx++] = ptr;
+	return (0);
+}
+
+
 /*--------------------------------------------------------------------*/
 
 static void
@@ -114,24 +153,26 @@ pan_ws(struct vsb *vsb, const struct ws *ws)
 {
 
 	VSB_printf(vsb, "ws = %p {\n", ws);
+	if (pan_already(vsb, ws))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, ws, WS_MAGIC);
-	if (!(ws->id[0] & 0x20))
+	if (ws->id[0] != '\0' && (!(ws->id[0] & 0x20)))
 		VSB_printf(vsb, "OVERFLOWED ");
-	VSB_printf(vsb, "id = \"%s\",\n",  ws->id);
-	VSB_printf(vsb, "{s,f,r,e} = {%p",  ws->s);
+	VSB_printf(vsb, "id = \"%s\",\n", ws->id);
+	VSB_printf(vsb, "{s, f, r, e} = {%p", ws->s);
 	if (ws->f > ws->s)
-		VSB_printf(vsb, ",+%ld", (long) (ws->f - ws->s));
+		VSB_printf(vsb, ", +%ld", (long) (ws->f - ws->s));
 	else
-		VSB_printf(vsb, ",%p", ws->f);
+		VSB_printf(vsb, ", %p", ws->f);
 	if (ws->r > ws->s)
-		VSB_printf(vsb, ",+%ld", (long) (ws->r - ws->s));
+		VSB_printf(vsb, ", +%ld", (long) (ws->r - ws->s));
 	else
-		VSB_printf(vsb, ",%p", ws->r);
+		VSB_printf(vsb, ", %p", ws->r);
 	if (ws->e > ws->s)
-		VSB_printf(vsb, ",+%ld", (long) (ws->e - ws->s));
+		VSB_printf(vsb, ", +%ld", (long) (ws->e - ws->s));
 	else
-		VSB_printf(vsb, ",%p", ws->e);
+		VSB_printf(vsb, ", %p", ws->e);
 	VSB_printf(vsb, "},\n");
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
@@ -144,6 +185,8 @@ pan_htc(struct vsb *vsb, const struct http_conn *htc)
 {
 
 	VSB_printf(vsb, "http_conn = %p {\n", htc);
+	if (pan_already(vsb, htc))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, htc, HTTP_CONN_MAGIC);
 	VSB_printf(vsb, "fd = %d,\n", htc->fd);
@@ -173,6 +216,8 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 	int i;
 
 	VSB_printf(vsb, "http[%s] = %p {\n", id, h);
+	if (pan_already(vsb, h))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, h, HTTP_MAGIC);
 	VSB_printf(vsb, "ws[%s] = %p,\n", h->ws ? h->ws->id : "", h->ws);
@@ -190,20 +235,54 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 	VSB_printf(vsb, "},\n");
 }
 
+/*--------------------------------------------------------------------*/
+static void
+pan_boc(struct vsb *vsb, const struct boc *boc)
+{
+	VSB_printf(vsb, "boc = %p {\n", boc);
+	if (pan_already(vsb, boc))
+		return;
+	VSB_indent(vsb, 2);
+	PAN_CheckMagic(vsb, boc, BOC_MAGIC);
+	VSB_printf(vsb, "refcnt = %u,\n", boc->refcount);
+	VSB_printf(vsb, "state = %s,\n", boc_state_2str(boc->state));
+	VSB_printf(vsb, "vary = %p,\n", boc->vary);
+	VSB_printf(vsb, "stevedore_priv = %p,\n", boc->stevedore_priv);
+	VSB_indent(vsb, -2);
+	VSB_printf(vsb, "},\n");
+}
+
 /*--------------------------------------------------------------------*/
 
 static void
 pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 {
+	const char *p;
 
 	VSB_printf(vsb, "objcore[%s] = %p {\n", typ, oc);
+	if (pan_already(vsb, oc))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, oc, OBJCORE_MAGIC);
 	VSB_printf(vsb, "refcnt = %d,\n", oc->refcnt);
-	VSB_printf(vsb, "flags = 0x%x,\n", oc->flags);
-	VSB_printf(vsb, "exp_flags = 0x%x,\n", oc->exp_flags);
-	VSB_printf(vsb, "exp = { %f, %f, %f, %f }\n",
-	    oc->exp.t_origin, oc->exp.ttl, oc->exp.grace, oc->exp.keep);
+	VSB_printf(vsb, "flags = {");
+	p = "";
+#define OC_FLAG(U, l, v) \
+	if (oc->flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
+#include "tbl/oc_flags.h"
+#undef OC_FLAG
+	VSB_printf(vsb, "},\n");
+	VSB_printf(vsb, "exp_flags = {");
+	p = "";
+#define OC_EXP_FLAG(U, l, v) \
+	if (oc->exp_flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
+#include "tbl/oc_exp_flags.h"
+#undef OC_EXP_FLAG
+	VSB_printf(vsb, "},\n");
+	if (oc->boc != NULL)
+		pan_boc(vsb, oc->boc);
+	VSB_printf(vsb, "exp = {%f, %f, %f, %f}\n",
+	    oc->t_origin, oc->ttl, oc->grace, oc->keep);
 	VSB_printf(vsb, "objhead = %p,\n", oc->objhead);
 	VSB_printf(vsb, "stevedore = %p", oc->stobj->stevedore);
 	if (oc->stobj->stevedore != NULL) {
@@ -211,6 +290,13 @@ pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 		if (strlen(oc->stobj->stevedore->ident))
 			VSB_printf(vsb, " %s", oc->stobj->stevedore->ident);
 		VSB_printf(vsb, ")");
+		if (oc->stobj->stevedore->panic) {
+			VSB_printf(vsb, " {\n");
+			VSB_indent(vsb, 2);
+			oc->stobj->stevedore->panic(vsb, oc);
+			VSB_indent(vsb, -2);
+			VSB_printf(vsb, "}");
+		}
 	}
 	VSB_printf(vsb, ",\n");
 	VSB_indent(vsb, -2);
@@ -227,6 +313,8 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 	const char *p;
 
 	VSB_printf(vsb, "worker = %p {\n", wrk);
+	if (pan_already(vsb, wrk))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, wrk, WORKER_MAGIC);
 	VSB_printf(vsb, "stack = {0x%jx -> 0x%jx},\n",
@@ -275,18 +363,18 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 	const char *p;
 
 	VSB_printf(vsb, "busyobj = %p {\n", bo);
+	if (pan_already(vsb, bo))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, bo, BUSYOBJ_MAGIC);
 	pan_ws(vsb, bo->ws);
-	VSB_printf(vsb, "refcnt = %u,\n", bo->refcount);
 	VSB_printf(vsb, "retries = %d, ", bo->retries);
 	VSB_printf(vsb, "failed = %d, ", bo->vfc->failed);
-	VSB_printf(vsb, "state = %d,\n", (int)bo->state);
 	VSB_printf(vsb, "flags = {");
 	p = "";
 	/*lint -save -esym(438,p) */
 #define BO_FLAG(l, r, w, d) \
-	if(bo->l) { VSB_printf(vsb,  "%s" #l, p); p = ", "; }
+	if(bo->l) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/bo_flags.h"
 #undef BO_FLAG
 	/*lint -restore */
@@ -327,12 +415,25 @@ static void
 pan_req(struct vsb *vsb, const struct req *req)
 {
 	const char *stp;
+	const struct transport *xp;
 
 	VSB_printf(vsb, "req = %p {\n", req);
+	if (pan_already(vsb, req))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, req, REQ_MAGIC);
-	VSB_printf(vsb, "vxid = %u, ", VXID(req->vsl->wid));
+	xp = req->transport;
+	VSB_printf(vsb, "vxid = %u, transport = %s", VXID(req->vsl->wid),
+	    xp == NULL ? "NULL" : xp->name);
 
+	if (xp != NULL && xp->req_panic != NULL) {
+		VSB_printf(vsb, " {\n");
+		VSB_indent(vsb, 2);
+		xp->req_panic(vsb, req);
+		VSB_indent(vsb, -2);
+		VSB_printf(vsb, "}");
+	}
+	VSB_printf(vsb, "\n");
 	switch (req->req_step) {
 #define REQ_STEP(l, u, arg) case R_STP_##u: stp = "R_STP_" #u; break;
 #include "tbl/steps.h"
@@ -370,11 +471,10 @@ pan_req(struct vsb *vsb, const struct req *req)
 
 	VCL_Panic(vsb, req->vcl);
 
-	if (req->objcore != NULL) {
+	if (req->body_oc != NULL)
+		pan_objcore(vsb, "BODY", req->body_oc);
+	if (req->objcore != NULL)
 		pan_objcore(vsb, "REQ", req->objcore);
-		if (req->objcore->busyobj != NULL)
-			pan_busyobj(vsb, req->objcore->busyobj);
-	}
 
 	VSB_printf(vsb, "flags = {\n");
 	VSB_indent(vsb, 2);
@@ -393,29 +493,32 @@ pan_req(struct vsb *vsb, const struct req *req)
 static void
 pan_sess(struct vsb *vsb, const struct sess *sp)
 {
-	const char *stp;
-	char *ci;
-	char *cp;
+	const char *ci;
+	const char *cp;
+	const struct transport *xp;
 
 	VSB_printf(vsb, "sp = %p {\n", sp);
+	if (pan_already(vsb, sp))
+		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, sp, SESS_MAGIC);
-	VSB_printf(vsb, "fd = %d, vxid = %u,\n", sp->fd, VXID(sp->vxid));
-	AZ(SES_Get_client_ip(sp, &ci));
-	AZ(SES_Get_client_port(sp, &cp));
-	VSB_printf(vsb, "client = %s %s,\n", ci, cp);
+	xp = XPORT_ByNumber(sp->sattr[SA_TRANSPORT]);
+	VSB_printf(vsb, "fd = %d, vxid = %u,\n",
+	    sp->fd, VXID(sp->vxid));
 	VSB_printf(vsb, "t_open = %f,\n", sp->t_open);
 	VSB_printf(vsb, "t_idle = %f,\n", sp->t_idle);
-	switch (sp->sess_step) {
-#define SESS_STEP(l, u) case S_STP_##u: stp = "S_STP_" #u; break;
-#include "tbl/steps.h"
-#undef SESS_STEP
-		default: stp = NULL;
+	VSB_printf(vsb, "transport = %s",
+	    xp == NULL ? "<none>" : xp->name);
+	if (xp != NULL && xp->sess_panic != NULL) {
+		VSB_printf(vsb, " {\n");
+		VSB_indent(vsb, 2);
+		xp->sess_panic(vsb, sp);
+		VSB_indent(vsb, -2);
+		VSB_printf(vsb, "}");
 	}
-	if (stp != NULL)
-		VSB_printf(vsb, "step = %s,\n", stp);
-	else
-		VSB_printf(vsb, "step = 0x%x,\n", sp->sess_step);
+	ci = SES_Get_String_Attr(sp, SA_CLIENT_IP);
+	cp = SES_Get_String_Attr(sp, SA_CLIENT_PORT);
+	VSB_printf(vsb, "\nclient = %s %s,\n", ci, cp);
 
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
@@ -565,9 +668,7 @@ ccf_panic(struct cli *cli, const char * const *av, void *priv)
 /*--------------------------------------------------------------------*/
 
 static struct cli_proto debug_cmds[] = {
-	{ "debug.panic.worker", "debug.panic.worker",
-		"\tPanic the worker process.",
-		0, 0, "d", ccf_panic },
+	{ CLICMD_DEBUG_PANIC_WORKER,		"d",	ccf_panic },
 	{ NULL }
 };
 
