@@ -39,6 +39,7 @@
 
 #include "cache.h"
 #include "cache_pool.h"
+#include "cache_transport.h"
 
 #include "vtim.h"
 
@@ -66,6 +67,9 @@ Req_New(const struct worker *wrk, struct sess *sp)
 	req->sp = sp;
 	req->top = req;	// esi overrides
 
+	INIT_OBJ(req->htc, HTTP_CONN_MAGIC);
+	req->htc->fd = -1;
+
 	e = (char*)req + sz;
 	p = (char*)(req + 1);
 	p = (void*)PRNDUP(p);
@@ -207,11 +211,11 @@ Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req)
 /*----------------------------------------------------------------------
  */
 
-void __match_proto__()
+void __match_proto__(vtr_req_fail_f)
 Req_Fail(struct req *req, enum sess_close reason)
 {
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 
-	if (req->sp->fd >= 0)
-		SES_Close(req->sp, reason);
+	AN(req->transport->req_fail);
+	req->transport->req_fail(req, reason);
 }
