@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2015 Varnish Software AS
+ * Copyright (c) 2006-2016 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,14 +26,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * Interface *to* compiled VCL code:  Loading, unloading, calling into etc.
- *
- * The interface *from* the compiled VCL code is in cache_vrt.c.
  */
 
 #include "config.h"
 
+#include <errno.h>
 #include <dlfcn.h>
+#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -44,14 +43,21 @@
 
 #include "cache_director.h"
 #include "cache_backend.h"
-#include "vcli.h"
-#include "vcli_priv.h"
+#include "vcli_serve.h"
 
 static const char * const VCL_TEMP_INIT = "init";
 static const char * const VCL_TEMP_COLD = "cold";
 static const char * const VCL_TEMP_WARM = "warm";
 static const char * const VCL_TEMP_BUSY = "busy";
 static const char * const VCL_TEMP_COOLING = "cooling";
+static const char * const VCL_TEMP_LABEL = "label";
+
+/*
+ * NB: The COOLING temperature is neither COLD nor WARM.
+ * And LABEL is not a temperature, it's a state.
+ */
+#define VCL_WARM(v) ((v)->temp == VCL_TEMP_WARM || (v)->temp == VCL_TEMP_BUSY)
+#define VCL_COLD(v) ((v)->temp == VCL_TEMP_INIT || (v)->temp == VCL_TEMP_COLD)
 
 struct vcl {
 	unsigned		magic;
@@ -64,8 +70,11 @@ struct vcl {
 	unsigned		busy;
 	unsigned		discard;
 	const char		*temp;
+	pthread_rwlock_t	temp_rwl;
 	VTAILQ_HEAD(,backend)	backend_list;
 	VTAILQ_HEAD(,vclref)	ref_list;
+	struct vcl		*label;
+	int			nlabels;
 };
 
 struct vclref {
@@ -86,6 +95,89 @@ static VTAILQ_HEAD(, vcl)	vcl_head =
 static struct lock		vcl_mtx;
 static struct vcl		*vcl_active; /* protected by vcl_mtx */
 
+static struct vrt_ctx ctx_cli;
+static unsigned handling_cli;
+static struct ws ws_cli;
+static char *ws_snapshot_cli;
+
+/*--------------------------------------------------------------------*/
+
+static struct vrt_ctx *
+vcl_get_ctx(unsigned method, int msg)
+{
+	AZ(ctx_cli.handling);
+	INIT_OBJ(&ctx_cli, VRT_CTX_MAGIC);
+	ctx_cli.handling = &handling_cli;
+	ctx_cli.method = method;
+	if (msg) {
+		ctx_cli.msg = VSB_new_auto();
+		AN(ctx_cli.msg);
+	}
+	ctx_cli.ws = &ws_cli;
+	WS_Assert(ctx_cli.ws);
+	return (&ctx_cli);
+}
+
+static void
+vcl_rel_ctx(struct vrt_ctx **ctx)
+{
+	assert(*ctx == &ctx_cli);
+	AN((*ctx)->handling);
+	if (ctx_cli.msg)
+		VSB_destroy(&ctx_cli.msg);
+	WS_Assert(ctx_cli.ws);
+	WS_Reset(&ws_cli, ws_snapshot_cli);
+	INIT_OBJ(*ctx, VRT_CTX_MAGIC);
+	*ctx = NULL;
+	VRTPRIV_dynamic_kill(NULL, 0);
+}
+
+/*--------------------------------------------------------------------*/
+
+static int
+vcl_send_event(VRT_CTX, enum vcl_event_e ev)
+{
+	int r;
+
+	ASSERT_CLI();
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
+	assert(ev == VCL_EVENT_LOAD ||
+	       ev == VCL_EVENT_WARM ||
+	       ev == VCL_EVENT_COLD ||
+	       ev == VCL_EVENT_DISCARD);
+	AN(ctx->handling);
+	AN(ctx->ws);
+
+	if (ev == VCL_EVENT_LOAD || ev == VCL_EVENT_WARM)
+		AN(ctx->msg);
+
+	r = ctx->vcl->conf->event_vcl(ctx, ev);
+
+	if (r && (ev == VCL_EVENT_COLD || ev == VCL_EVENT_DISCARD))
+		WRONG("A VMOD cannot fail COLD or DISCARD events");
+
+	return (r);
+}
+
+/*--------------------------------------------------------------------*/
+
+static struct vcl *
+vcl_find(const char *name)
+{
+	struct vcl *vcl;
+
+	ASSERT_CLI();
+	VTAILQ_FOREACH(vcl, &vcl_head, list) {
+		if (vcl->discard)
+			continue;
+		if (!strcmp(vcl->loaded_name, name))
+			return (vcl);
+	}
+	return (NULL);
+}
+
 /*--------------------------------------------------------------------*/
 
 void
@@ -99,10 +191,11 @@ VCL_Panic(struct vsb *vsb, const struct vcl *vcl)
 	VSB_printf(vsb, "vcl = {\n");
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, vcl, VCL_MAGIC);
+	VSB_printf(vsb, "name = \"%s\"\n", vcl->loaded_name);
 	VSB_printf(vsb, "busy = %u\n", vcl->busy);
 	VSB_printf(vsb, "discard = %u,\n", vcl->discard);
 	VSB_printf(vsb, "state = %s,\n", vcl->state);
-	VSB_printf(vsb, "temp = %s,\n", vcl->temp);
+	VSB_printf(vsb, "temp = %s,\n", (const volatile char *)vcl->temp);
 	VSB_printf(vsb, "conf = {\n");
 	VSB_indent(vsb, 2);
 	if (vcl->conf == NULL) {
@@ -129,7 +222,9 @@ VCL_Return_Name(unsigned r)
 {
 
 	switch (r) {
-#define VCL_RET_MAC(l, U, B) case VCL_RET_##U: return(#l);
+#define VCL_RET_MAC(l, U, B)	\
+	case VCL_RET_##U:	\
+		return(#l);
 #include "tbl/vcl_returns.h"
 #undef VCL_RET_MAC
 	default:
@@ -142,7 +237,9 @@ VCL_Method_Name(unsigned m)
 {
 
 	switch (m) {
-#define VCL_MET_MAC(func, upper, typ, bitmap) case VCL_MET_##upper: return (#upper);
+#define VCL_MET_MAC(func, upper, typ, bitmap)	\
+	case VCL_MET_##upper:			\
+		return (#upper);
 #include "tbl/vcl_returns.h"
 #undef VCL_MET_MAC
 	default:
@@ -153,16 +250,22 @@ VCL_Method_Name(unsigned m)
 /*--------------------------------------------------------------------*/
 
 static void
-VCL_Get(struct vcl **vcc)
+vcl_get(struct vcl **vcc, struct vcl *vcl)
 {
-	while (vcl_active == NULL)
-		(void)usleep(100000);
 
-	CHECK_OBJ_NOTNULL(vcl_active, VCL_MAGIC);
-	assert(vcl_active->temp == VCL_TEMP_WARM);
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	AZ(errno=pthread_rwlock_rdlock(&vcl->temp_rwl));
+	assert(VCL_WARM(vcl));
+	AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
 	Lck_Lock(&vcl_mtx);
-	AN(vcl_active);
-	*vcc = vcl_active;
+	AN(vcl);
+	if (vcl->label == NULL) {
+		AN(strcmp(vcl->state, VCL_TEMP_LABEL));
+		*vcc = vcl;
+	} else {
+		AZ(strcmp(vcl->state, VCL_TEMP_LABEL));
+		*vcc = vcl->label;
+	}
 	AN(*vcc);
 	AZ((*vcc)->discard);
 	(*vcc)->busy++;
@@ -173,12 +276,15 @@ void
 VCL_Refresh(struct vcl **vcc)
 {
 	CHECK_OBJ_NOTNULL(vcl_active, VCL_MAGIC);
-	assert(vcl_active->temp == VCL_TEMP_WARM);
 	if (*vcc == vcl_active)
 		return;
 	if (*vcc != NULL)
 		VCL_Rel(vcc);	/* XXX: optimize locking */
-	VCL_Get(vcc);
+
+	while (vcl_active == NULL)
+		(void)usleep(100000);
+
+	vcl_get(vcc, vcl_active);
 }
 
 void
@@ -186,7 +292,9 @@ VCL_Ref(struct vcl *vcl)
 {
 
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
-	assert(vcl->temp != VCL_TEMP_INIT && vcl->temp != VCL_TEMP_COLD);
+	AZ(errno=pthread_rwlock_rdlock(&vcl->temp_rwl));
+	assert(!VCL_COLD(vcl));
+	AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
 	Lck_Lock(&vcl_mtx);
 	assert(vcl->busy > 0);
 	vcl->busy++;
@@ -222,18 +330,22 @@ VCL_AddBackend(struct vcl *vcl, struct backend *be)
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
 	CHECK_OBJ_NOTNULL(be, BACKEND_MAGIC);
 
-	if (vcl->temp == VCL_TEMP_COOLING)
+	AZ(errno=pthread_rwlock_rdlock(&vcl->temp_rwl));
+	if (vcl->temp == VCL_TEMP_COOLING) {
+		AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
 		return (1);
+	}
 
 	Lck_Lock(&vcl_mtx);
 	VTAILQ_INSERT_TAIL(&vcl->backend_list, be, vcl_list);
 	Lck_Unlock(&vcl_mtx);
 
-	if (vcl->temp == VCL_TEMP_WARM || vcl->temp == VCL_TEMP_BUSY)
+	if (VCL_WARM(vcl))
 		/* Only when adding backend to already warm VCL */
 		VBE_Event(be, VCL_EVENT_WARM);
 	else if (vcl->temp != VCL_TEMP_INIT)
 		WRONG("Dynamic Backends can only be added to warm VCLs");
+	AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
 
 	return (0);
 }
@@ -249,8 +361,11 @@ VCL_DelBackend(struct backend *be)
 	Lck_Lock(&vcl_mtx);
 	VTAILQ_REMOVE(&vcl->backend_list, be, vcl_list);
 	Lck_Unlock(&vcl_mtx);
-	if (vcl->temp == VCL_TEMP_WARM)
+
+	AZ(errno=pthread_rwlock_rdlock(&vcl->temp_rwl));
+	if (VCL_WARM(vcl))
 		VBE_Event(be, VCL_EVENT_COLD);
+	AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
 }
 
 static void
@@ -303,7 +418,7 @@ VCL_Open(const char *fn, struct vsb *msg)
 	dlh = dlopen(fn, RTLD_NOW | RTLD_LOCAL);
 	if (dlh == NULL) {
 		VSB_printf(msg, "Could not load compiled VCL.\n");
-		VSB_printf(msg, "\tdlopen(%s) = %s\n", fn, dlerror());
+		VSB_printf(msg, "\tdlopen() = %s\n", dlerror());
 		return (NULL);
 	}
 	cnf = dlsym(dlh, "VCL_conf");
@@ -319,6 +434,7 @@ VCL_Open(const char *fn, struct vsb *msg)
 	}
 	ALLOC_OBJ(vcl, VCL_MAGIC);
 	AN(vcl);
+	AZ(errno=pthread_rwlock_init(&vcl->temp_rwl, NULL));
 	vcl->dlh = dlh;
 	vcl->conf = cnf;
 	return (vcl);
@@ -333,6 +449,7 @@ VCL_Close(struct vcl **vclp)
 	vcl = *vclp;
 	*vclp = NULL;
 	AZ(dlclose(vcl->dlh));
+	AZ(errno=pthread_rwlock_destroy(&vcl->temp_rwl));
 	FREE_OBJ(vcl);
 }
 
@@ -357,7 +474,7 @@ VCL_TestLoad(const char *fn)
 		retval = -1;
 	} else
 		VCL_Close(&vcl);
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 	return (retval);
 }
 
@@ -389,7 +506,9 @@ VCL_DefaultProbe(const struct vcl *vcl)
 	return (vcl->conf->default_probe);
 }
 
-/*--------------------------------------------------------------------*/
+/*--------------------------------------------------------------------
+ * VRT apis relating to VCL's as VCLS.
+ */
 
 void
 VRT_count(VRT_CTX, unsigned u)
@@ -404,6 +523,28 @@ VRT_count(VRT_CTX, unsigned u)
 		    ctx->vcl->conf->ref[u].line, ctx->vcl->conf->ref[u].pos);
 }
 
+VCL_VCL
+VRT_vcl_lookup(const char *name)
+{
+	VCL_VCL vcl;
+
+	vcl = vcl_find(name);
+	AN(vcl);
+	return (vcl);
+}
+
+void
+VRT_vcl_select(VRT_CTX, VCL_VCL vcl)
+{
+	struct req *req = ctx->req;
+
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	VCL_Rel(&req->vcl);
+	vcl_get(&req->vcl, vcl);
+	/* XXX: better logging */
+	VSLb(ctx->req->vsl, SLT_Debug, "Now using %s VCL", vcl->loaded_name);
+}
+
 struct vclref *
 VRT_ref_vcl(VRT_CTX, const char *desc)
 {
@@ -417,7 +558,7 @@ VRT_ref_vcl(VRT_CTX, const char *desc)
 
 	vcl = ctx->vcl;
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
-	xxxassert(vcl->temp == VCL_TEMP_WARM);
+	assert(VCL_WARM(vcl));
 
 	ALLOC_OBJ(ref, VCLREF_MAGIC);
 	AN(ref);
@@ -447,8 +588,12 @@ VRT_rel_vcl(VRT_CTX, struct vclref **refp)
 	vcl = ctx->vcl;
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
 	assert(vcl == ref->vcl);
-	assert(vcl->temp == VCL_TEMP_WARM || vcl->temp == VCL_TEMP_BUSY ||
-	    vcl->temp == VCL_TEMP_COOLING);
+
+	/* NB: A VCL may be released by a VMOD at any time, but it must happen
+	 * after a warmup and before the end of a cooldown. The release may or
+	 * may not happen while the same thread holds the temperature lock, so
+	 * instead we check that all references are gone in VCL_Nuke.
+	 */
 
 	Lck_Lock(&vcl_mtx);
 	assert(!VTAILQ_EMPTY(&vcl->ref_list));
@@ -461,52 +606,6 @@ VRT_rel_vcl(VRT_CTX, struct vclref **refp)
 
 /*--------------------------------------------------------------------*/
 
-static struct vcl *
-vcl_find(const char *name)
-{
-	struct vcl *vcl;
-
-	ASSERT_CLI();
-	VTAILQ_FOREACH(vcl, &vcl_head, list) {
-		if (vcl->discard)
-			continue;
-		if (!strcmp(vcl->loaded_name, name))
-			return (vcl);
-	}
-	return (NULL);
-}
-
-static int
-vcl_setup_event(VRT_CTX, enum vcl_event_e ev)
-{
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
-	AN(ctx->handling);
-	AN(ctx->vcl);
-	AN(ctx->msg);
-	assert(ev == VCL_EVENT_LOAD || ev == VCL_EVENT_WARM);
-
-	return (ctx->vcl->conf->event_vcl(ctx, ev));
-}
-
-static void
-vcl_failsafe_event(VRT_CTX, enum vcl_event_e ev)
-{
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
-	AN(ctx->handling);
-	AN(ctx->vcl);
-	assert(ev == VCL_EVENT_COLD || ev == VCL_EVENT_DISCARD ||
-	    ev == VCL_EVENT_USE);
-
-	if (ctx->vcl->conf->event_vcl(ctx, ev) != 0)
-		WRONG("A VMOD cannot fail USE, COLD or DISCARD events");
-}
-
 static void
 vcl_print_refs(VRT_CTX)
 {
@@ -540,17 +639,17 @@ vcl_set_state(VRT_CTX, const char *state)
 	assert(ctx->msg != NULL || *state == '0');
 
 	vcl = ctx->vcl;
+	AZ(errno=pthread_rwlock_wrlock(&vcl->temp_rwl));
 	AN(vcl->temp);
 
 	switch(state[0]) {
 	case '0':
 		assert(vcl->temp != VCL_TEMP_COLD);
-		if (vcl->busy == 0 && (vcl->temp == VCL_TEMP_WARM ||
-		    vcl->temp == VCL_TEMP_BUSY)) {
+		if (vcl->busy == 0 && VCL_WARM(vcl)) {
 
 			vcl->temp = VTAILQ_EMPTY(&vcl->ref_list) ?
 			    VCL_TEMP_COLD : VCL_TEMP_COOLING;
-			vcl_failsafe_event(ctx, VCL_EVENT_COLD);
+			AZ(vcl_send_event(ctx, VCL_EVENT_COLD));
 			vcl_BackendEvent(vcl, VCL_EVENT_COLD);
 		}
 		else if (vcl->busy)
@@ -572,7 +671,7 @@ vcl_set_state(VRT_CTX, const char *state)
 		}
 		else {
 			vcl->temp = VCL_TEMP_WARM;
-			i = vcl_setup_event(ctx, VCL_EVENT_WARM);
+			i = vcl_send_event(ctx, VCL_EVENT_WARM);
 			if (i == 0)
 				vcl_BackendEvent(vcl, VCL_EVENT_WARM);
 			else
@@ -582,6 +681,8 @@ vcl_set_state(VRT_CTX, const char *state)
 	default:
 		WRONG("Wrong enum state");
 	}
+	AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
+
 	return (i);
 }
 
@@ -601,36 +702,25 @@ vcl_cancel_load(VRT_CTX, struct cli *cli, const char *name, const char *step)
 	AZ(vcl->conf->event_vcl(ctx, VCL_EVENT_DISCARD));
 	vcl_KillBackends(vcl);
 	VCL_Close(&vcl);
-	VSB_delete(ctx->msg);
 }
 
 static void
-VCL_Load(struct cli *cli, const char *name, const char *fn, const char *state)
+vcl_load(struct cli *cli, struct vrt_ctx *ctx,
+    const char *name, const char *fn, const char *state)
 {
 	struct vcl *vcl;
-	struct vrt_ctx ctx;
-	unsigned hand = 0;
-	struct vsb *vsb;
 	int i;
 
 	ASSERT_CLI();
 
 	vcl = vcl_find(name);
-	if (vcl != NULL) {
-		VCLI_SetResult(cli, CLIS_PARAM);
-		VCLI_Out(cli, "Config '%s' already loaded", name);
-		return;
-	}
+	AZ(vcl);
 
-	vsb = VSB_new_auto();
-	AN(vsb);
-
-	vcl = VCL_Open(fn, vsb);
+	vcl = VCL_Open(fn, ctx->msg);
 	if (vcl == NULL) {
-		AZ(VSB_finish(vsb));
+		AZ(VSB_finish(ctx->msg));
 		VCLI_SetResult(cli, CLIS_PARAM);
-		VCLI_Out(cli, "%s", VSB_data(vsb));
-		VSB_delete(vsb);
+		VCLI_Out(cli, "%s", VSB_data(ctx->msg));
 		return;
 	}
 
@@ -641,28 +731,23 @@ VCL_Load(struct cli *cli, const char *name, const char *fn, const char *state)
 
 	vcl->temp = VCL_TEMP_INIT;
 
-	INIT_OBJ(&ctx, VRT_CTX_MAGIC);
-	ctx.method = VCL_MET_INIT;
-	ctx.handling = &hand;
-	ctx.vcl = vcl;
+	ctx->vcl = vcl;
 
-	VSB_clear(vsb);
-	ctx.msg = vsb;
-	i = vcl_setup_event(&ctx, VCL_EVENT_LOAD);
+	VSB_clear(ctx->msg);
+	i = vcl_send_event(ctx, VCL_EVENT_LOAD);
 	if (i) {
-		vcl_cancel_load(&ctx, cli, name, "initialization");
+		vcl_cancel_load(ctx, cli, name, "initialization");
 		return;
 	}
-	VSB_clear(vsb);
-	i = vcl_set_state(&ctx, state);
+	VSB_clear(ctx->msg);
+	i = vcl_set_state(ctx, state);
 	if (i) {
 		assert(*state == '1');
-		vcl_cancel_load(&ctx, cli, name, "warmup");
+		vcl_cancel_load(ctx, cli, name, "warmup");
 		return;
 	}
-	VSB_delete(vsb);
 	bprintf(vcl->state, "%s", state + 1);
-	assert(hand == VCL_RET_OK);
+	assert(*ctx->handling == VCL_RET_OK);
 	VCLI_Out(cli, "Loaded \"%s\" as \"%s\"", fn , name);
 	VTAILQ_INSERT_TAIL(&vcl_head, vcl, list);
 	Lck_Lock(&vcl_mtx);
@@ -673,66 +758,51 @@ VCL_Load(struct cli *cli, const char *name, const char *fn, const char *state)
 	VSC_C_main->n_vcl_avail++;
 }
 
-/*--------------------------------------------------------------------
- * This function is polled from the CLI thread to dispose of any non-busy
- * VCLs which have been discarded.
- */
-
-static void
-VCL_Nuke(struct vcl *vcl)
-{
-	struct vrt_ctx ctx;
-	unsigned hand = 0;
-
-	INIT_OBJ(&ctx, VRT_CTX_MAGIC);
-	ASSERT_CLI();
-	assert(vcl != vcl_active);
-	assert(vcl->discard);
-	AZ(vcl->busy);
-	assert(VTAILQ_EMPTY(&vcl->ref_list));
-	VTAILQ_REMOVE(&vcl_head, vcl, list);
-	ctx.method = VCL_MET_FINI;
-	ctx.handling = &hand;
-	ctx.vcl = vcl;
-	vcl_failsafe_event(&ctx, VCL_EVENT_DISCARD);
-	vcl_KillBackends(vcl);
-	free(vcl->loaded_name);
-	VCL_Close(&vcl);
-	VSC_C_main->n_vcl--;
-	VSC_C_main->n_vcl_discard--;
-}
-
 /*--------------------------------------------------------------------*/
 
 void
 VCL_Poll(void)
 {
-	struct vrt_ctx ctx;
+	struct vrt_ctx *ctx;
 	struct vcl *vcl, *vcl2;
-	unsigned hand;
 
 	ASSERT_CLI();
+	ctx = vcl_get_ctx(0, 0);
 	VTAILQ_FOREACH_SAFE(vcl, &vcl_head, list, vcl2) {
 		if (vcl->temp == VCL_TEMP_BUSY ||
 		    vcl->temp == VCL_TEMP_COOLING) {
-			INIT_OBJ(&ctx, VRT_CTX_MAGIC);
-			ctx.vcl = vcl;
-			ctx.handling = &hand;
-			(void)vcl_set_state(&ctx, "0");
+			ctx->vcl = vcl;
+			ctx->method = 0;
+			(void)vcl_set_state(ctx, "0");
+		}
+		if (vcl->discard && vcl->temp == VCL_TEMP_COLD) {
+			AZ(vcl->busy);
+			assert(vcl != vcl_active);
+			assert(VTAILQ_EMPTY(&vcl->ref_list));
+			VTAILQ_REMOVE(&vcl_head, vcl, list);
+			ctx->method = VCL_MET_FINI;
+			ctx->vcl = vcl;
+			AZ(vcl_send_event(ctx, VCL_EVENT_DISCARD));
+			vcl_KillBackends(vcl);
+			free(vcl->loaded_name);
+			VCL_Close(&vcl);
+			VSC_C_main->n_vcl--;
+			VSC_C_main->n_vcl_discard--;
 		}
-		if (vcl->discard && vcl->temp == VCL_TEMP_COLD)
-			VCL_Nuke(vcl);
 	}
+	vcl_rel_ctx(&ctx);
 }
 
 /*--------------------------------------------------------------------*/
 
 static void __match_proto__(cli_func_t)
-ccf_config_list(struct cli *cli, const char * const *av, void *priv)
+vcl_cli_list(struct cli *cli, const char * const *av, void *priv)
 {
 	struct vcl *vcl;
 	const char *flg;
 
+	/* NB: Shall generate same output as mcf_vcl_list() */
+
 	(void)av;
 	(void)priv;
 	ASSERT_CLI();
@@ -743,54 +813,57 @@ ccf_config_list(struct cli *cli, const char * const *av, void *priv)
 			flg = "discarded";
 		} else
 			flg = "available";
-		VCLI_Out(cli, "%-10s %4s/%-8s %6u %s\n",
+		VCLI_Out(cli, "%-10s %5s/%-8s %6u %s",
 		    flg, vcl->state, vcl->temp, vcl->busy, vcl->loaded_name);
+		if (vcl->label != NULL)
+			VCLI_Out(cli, " -> %s", vcl->label->loaded_name);
+		else if (vcl->nlabels > 1)
+			VCLI_Out(cli, " (%d labels)", vcl->nlabels);
+		else if (vcl->nlabels > 0)
+			VCLI_Out(cli, " (%d label)", vcl->nlabels);
+		VCLI_Out(cli, "\n");
 	}
 }
 
 static void __match_proto__(cli_func_t)
-ccf_config_load(struct cli *cli, const char * const *av, void *priv)
+vcl_cli_load(struct cli *cli, const char * const *av, void *priv)
 {
+	struct vrt_ctx *ctx;
 
 	AZ(priv);
 	ASSERT_CLI();
-	VCL_Load(cli, av[2], av[3], av[4]);
+	ctx = vcl_get_ctx(VCL_MET_INIT, 1);
+	vcl_load(cli, ctx, av[2], av[3], av[4]);
+	vcl_rel_ctx(&ctx);
 }
 
 static void __match_proto__(cli_func_t)
-ccf_config_state(struct cli *cli, const char * const *av, void *priv)
+vcl_cli_state(struct cli *cli, const char * const *av, void *priv)
 {
-	struct vrt_ctx ctx;
-	unsigned hand;
+	struct vrt_ctx *ctx;
 
-	INIT_OBJ(&ctx, VRT_CTX_MAGIC);
-	ctx.msg = VSB_new_auto();
-	AN(ctx.msg);
-	ctx.handling = &hand;
-
-	(void)cli;
 	AZ(priv);
 	ASSERT_CLI();
 	AN(av[2]);
 	AN(av[3]);
-	ctx.vcl = vcl_find(av[2]);
-	AN(ctx.vcl);			// MGT ensures this
-	if (vcl_set_state(&ctx, av[3]) == 0) {
-		bprintf(ctx.vcl->state, "%s", av[3] + 1);
-		VSB_delete(ctx.msg);
-		return;
+	ctx = vcl_get_ctx(0, 1);
+	ctx->vcl = vcl_find(av[2]);
+	AN(ctx->vcl);			// MGT ensures this
+	if (vcl_set_state(ctx, av[3]) == 0) {
+		bprintf(ctx->vcl->state, "%s", av[3] + 1);
+	} else {
+		AZ(VSB_finish(ctx->msg));
+		VCLI_SetResult(cli, CLIS_CANT);
+		VCLI_Out(cli, "Failed <vcl.state %s %s>", ctx->vcl->loaded_name,
+		    av[3] + 1);
+		if (VSB_len(ctx->msg))
+			VCLI_Out(cli, "\nMessage:\n\t%s", VSB_data(ctx->msg));
 	}
-	AZ(VSB_finish(ctx.msg));
-	VCLI_SetResult(cli, CLIS_CANT);
-	VCLI_Out(cli, "Failed <vcl.state %s %s>", ctx.vcl->loaded_name,
-	    av[3] + 1);
-	if (VSB_len(ctx.msg))
-		VCLI_Out(cli, "\nMessage:\n\t%s", VSB_data(ctx.msg));
-	VSB_delete(ctx.msg);
+	vcl_rel_ctx(&ctx);
 }
 
 static void __match_proto__(cli_func_t)
-ccf_config_discard(struct cli *cli, const char * const *av, void *priv)
+vcl_cli_discard(struct cli *cli, const char * const *av, void *priv)
 {
 	struct vcl *vcl;
 
@@ -801,21 +874,55 @@ ccf_config_discard(struct cli *cli, const char * const *av, void *priv)
 	AN(vcl);			// MGT ensures this
 	Lck_Lock(&vcl_mtx);
 	assert (vcl != vcl_active);	// MGT ensures this
+	AZ(vcl->nlabels);		// MGT ensures this
 	VSC_C_main->n_vcl_discard++;
 	VSC_C_main->n_vcl_avail--;
 	vcl->discard = 1;
+	if (vcl->label != NULL) {
+		AZ(strcmp(vcl->state, VCL_TEMP_LABEL));
+		vcl->label->nlabels--;
+		vcl->label= NULL;
+	}
 	Lck_Unlock(&vcl_mtx);
 
-	if (vcl->temp == VCL_TEMP_COLD)
-		VCL_Nuke(vcl);
+	if (!strcmp(vcl->state, VCL_TEMP_LABEL)) {
+		VTAILQ_REMOVE(&vcl_head, vcl, list);
+		free(vcl->loaded_name);
+	} else if (vcl->temp == VCL_TEMP_COLD)
+		VCL_Poll();
 }
 
 static void __match_proto__(cli_func_t)
-ccf_config_use(struct cli *cli, const char * const *av, void *priv)
+vcl_cli_label(struct cli *cli, const char * const *av, void *priv)
+{
+	struct vcl *lbl;
+	struct vcl *vcl;
+
+	ASSERT_CLI();
+	(void)cli;
+	(void)priv;
+	vcl = vcl_find(av[3]);
+	AN(vcl);				// MGT ensures this
+	lbl = vcl_find(av[2]);
+	if (lbl == NULL) {
+		ALLOC_OBJ(lbl, VCL_MAGIC);
+		AN(lbl);
+		bprintf(lbl->state, "%s", VCL_TEMP_LABEL);
+		lbl->temp = VCL_TEMP_WARM;
+		REPLACE(lbl->loaded_name, av[2]);
+		AZ(errno=pthread_rwlock_init(&lbl->temp_rwl, NULL));
+		VTAILQ_INSERT_TAIL(&vcl_head, lbl, list);
+	}
+	if (lbl->label != NULL)
+		lbl->label->nlabels--;
+	lbl->label = vcl;
+	vcl->nlabels++;
+}
+
+static void __match_proto__(cli_func_t)
+vcl_cli_use(struct cli *cli, const char * const *av, void *priv)
 {
 	struct vcl *vcl;
-	struct vrt_ctx ctx;
-	unsigned hand = 0;
 
 	ASSERT_CLI();
 	AN(cli);
@@ -823,17 +930,13 @@ ccf_config_use(struct cli *cli, const char * const *av, void *priv)
 	vcl = vcl_find(av[2]);
 	AN(vcl);				// MGT ensures this
 	assert(vcl->temp == VCL_TEMP_WARM);	// MGT ensures this
-	INIT_OBJ(&ctx, VRT_CTX_MAGIC);
-	ctx.handling = &hand;
-	ctx.vcl = vcl;
-	vcl_failsafe_event(&ctx, VCL_EVENT_USE);
 	Lck_Lock(&vcl_mtx);
 	vcl_active = vcl;
 	Lck_Unlock(&vcl_mtx);
 }
 
 static void __match_proto__(cli_func_t)
-ccf_config_show(struct cli *cli, const char * const *av, void *priv)
+vcl_cli_show(struct cli *cli, const char * const *av, void *priv)
 {
 	struct vcl *vcl;
 	int verbose = 0;
@@ -901,6 +1004,7 @@ vcl_call_method(struct worker *wrk, struct req *req, struct busyobj *bo,
 		ctx.http_req_top = req->top->http;
 		ctx.http_resp = req->resp;
 		ctx.req = req;
+		ctx.sp = req->sp;
 		ctx.now = req->t_prev;
 		ctx.ws = req->ws;
 	}
@@ -912,6 +1016,7 @@ vcl_call_method(struct worker *wrk, struct req *req, struct busyobj *bo,
 		ctx.http_bereq = bo->bereq;
 		ctx.http_beresp = bo->beresp;
 		ctx.bo = bo;
+		ctx.sp = bo->sp;
 		ctx.now = bo->t_prev;
 		ctx.ws = bo->ws;
 	}
@@ -957,12 +1062,13 @@ VCL_##func##_method(struct vcl *vcl, struct worker *wrk,		\
 /*--------------------------------------------------------------------*/
 
 static struct cli_proto vcl_cmds[] = {
-	{ CLI_VCL_LOAD,		"i", ccf_config_load },
-	{ CLI_VCL_LIST,		"i", ccf_config_list },
-	{ CLI_VCL_STATE,	"i", ccf_config_state },
-	{ CLI_VCL_DISCARD,	"i", ccf_config_discard },
-	{ CLI_VCL_USE,		"i", ccf_config_use },
-	{ CLI_VCL_SHOW,		"", ccf_config_show },
+	{ CLICMD_VCL_LOAD,		"", vcl_cli_load },
+	{ CLICMD_VCL_LIST,		"", vcl_cli_list },
+	{ CLICMD_VCL_STATE,		"", vcl_cli_state },
+	{ CLICMD_VCL_DISCARD,		"", vcl_cli_discard },
+	{ CLICMD_VCL_USE,		"", vcl_cli_use },
+	{ CLICMD_VCL_SHOW,		"", vcl_cli_show },
+	{ CLICMD_VCL_LABEL,		"", vcl_cli_label },
 	{ NULL }
 };
 
@@ -970,6 +1076,10 @@ void
 VCL_Init(void)
 {
 
+	assert(cache_param->workspace_client > 0);
+	WS_Init(&ws_cli, "cli", malloc(cache_param->workspace_client),
+	    cache_param->workspace_client);
+	ws_snapshot_cli = WS_Snapshot(&ws_cli);
 	CLI_AddFuncs(vcl_cmds);
 	Lck_New(&vcl_mtx, lck_vcl);
 }
