@@ -58,7 +58,7 @@ VRT_synth(VRT_CTX, unsigned code, const char *reason)
 		code = 503;
 	ctx->req->err_code = (uint16_t)code;
 	ctx->req->err_reason =
-	    reason ? reason : http_Status2Reason(ctx->req->err_code);
+	    reason ? reason : http_Status2Reason(ctx->req->err_code, NULL);
 }
 
 /*--------------------------------------------------------------------*/
@@ -290,9 +290,8 @@ VRT_hashdata(VRT_CTX, const char *str, ...)
 double
 VRT_r_now(VRT_CTX)
 {
-
-	(void)ctx;
-	return (VTIM_real());
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	return (ctx->now);
 }
 
 /*--------------------------------------------------------------------*/
@@ -475,14 +474,8 @@ VRT_ban_string(VRT_CTX, const char *str)
 	VAV_Free(av);
 }
 
-/*--------------------------------------------------------------------
- *
- * XXX this really should be ssize_t VRT_CacheReqBody(VRT_CTX, size_t)
- * - change with next VRT major bump
- */
-
-int
-VRT_CacheReqBody(VRT_CTX, long long maxsize)
+VCL_BYTES
+VRT_CacheReqBody(VRT_CTX, VCL_BYTES maxsize)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -506,10 +499,7 @@ VRT_purge(VRT_CTX, double ttl, double grace, double keep)
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->wrk, WORKER_MAGIC);
-	if (ctx->method == VCL_MET_HIT)
-		HSH_Purge(ctx->req->wrk, ctx->req->objcore->objhead,
-		    ttl, grace, keep);
-	else if (ctx->method == VCL_MET_MISS)
+	if (ctx->method == VCL_MET_HIT || ctx->method == VCL_MET_MISS)
 		HSH_Purge(ctx->req->wrk, ctx->req->objcore->objhead,
 		    ttl, grace, keep);
 }
