@@ -408,6 +408,21 @@ VRT_r_beresp_backend(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
+void
+VRT_l_bereq_body(VRT_CTX, const char *p, ...)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	assert(p == vrt_magic_string_unset);
+	if (ctx->bo->req != NULL) {
+		CHECK_OBJ_NOTNULL(ctx->bo->req, REQ_MAGIC);
+		ctx->bo->req = NULL;
+		http_Unset(ctx->bo->bereq, H_Content_Length);
+	}
+}
+
+/*--------------------------------------------------------------------*/
+
 void
 VRT_l_req_esi(VRT_CTX, unsigned process_esi)
 {
@@ -481,65 +496,65 @@ VRT_r_bereq_retries(VRT_CTX)
  *	grace&keep are relative to ttl
  */
 
-#define VRT_DO_EXP_L(which, sexp, fld, offset)			\
+#define VRT_DO_EXP_L(which, oc, fld, offset)			\
 								\
 void								\
-VRT_l_##which##_##fld(VRT_CTX, double a)	\
+VRT_l_##which##_##fld(VRT_CTX, double a)			\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
 	a += (offset);						\
 	if (a < 0.0)						\
 		a = 0.0;					\
-	sexp.fld = a;						\
+	oc->fld = a;						\
 	VSLb(ctx->vsl, SLT_TTL, "VCL %.0f %.0f %.0f %.0f",	\
-	    sexp.ttl, sexp.grace, sexp.keep, sexp.t_origin);	\
+	    oc->ttl, oc->grace, oc->keep, oc->t_origin);	\
 }
 
-#define VRT_DO_EXP_R(which, sexp, fld, offset)			\
+#define VRT_DO_EXP_R(which, oc, fld, offset)			\
 								\
 double								\
-VRT_r_##which##_##fld(VRT_CTX)		\
+VRT_r_##which##_##fld(VRT_CTX)					\
 {								\
 	double d;						\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
-	d = sexp.fld;						\
+	d = oc->fld;						\
 	if (d <= 0.0)						\
 		d = 0.0;					\
 	d -= (offset);						\
 	return(d);						\
 }
 
-VRT_DO_EXP_R(obj, ctx->req->objcore->exp, ttl,
-    ctx->now - ctx->req->objcore->exp.t_origin)
-VRT_DO_EXP_R(obj, ctx->req->objcore->exp, grace, 0)
-VRT_DO_EXP_R(obj, ctx->req->objcore->exp, keep, 0)
+VRT_DO_EXP_R(obj, ctx->req->objcore, ttl,
+    ctx->now - ctx->req->objcore->t_origin)
+VRT_DO_EXP_R(obj, ctx->req->objcore, grace, 0)
+VRT_DO_EXP_R(obj, ctx->req->objcore, keep, 0)
 
-VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore->exp, ttl,
-    ctx->now - ctx->bo->fetch_objcore->exp.t_origin)
-VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore->exp, ttl,
-    ctx->now - ctx->bo->fetch_objcore->exp.t_origin)
-VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore->exp, grace, 0)
-VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore->exp, grace, 0)
-VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore->exp, keep, 0)
-VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore->exp, keep, 0)
+VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore, ttl,
+    ctx->now - ctx->bo->fetch_objcore->t_origin)
+VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, ttl,
+    ctx->now - ctx->bo->fetch_objcore->t_origin)
+VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore, grace, 0)
+VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, grace, 0)
+VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore, keep, 0)
+VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, keep, 0)
 
 /*--------------------------------------------------------------------
  */
 
-#define VRT_DO_AGE_R(which, sexp)				\
+#define VRT_DO_AGE_R(which, oc)					\
 								\
 double								\
 VRT_r_##which##_##age(VRT_CTX)		\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
-	return(ctx->now - sexp.t_origin);			\
+	return(ctx->now - oc->t_origin);			\
 }
 
-VRT_DO_AGE_R(obj, ctx->req->objcore->exp)
-VRT_DO_AGE_R(beresp, ctx->bo->fetch_objcore->exp)
+VRT_DO_AGE_R(obj, ctx->req->objcore)
+VRT_DO_AGE_R(beresp, ctx->bo->fetch_objcore)
 
 /*--------------------------------------------------------------------
  * [be]req.xid
@@ -604,9 +619,8 @@ VRT_r_req_##field(VRT_CTX)				\
 		struct suckaddr *sa;				\
 								\
 		CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);		\
-		CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);		\
-		CHECK_OBJ_NOTNULL(ctx->req->sp, SESS_MAGIC);	\
-		AZ(SES_Get_##fld##_addr(ctx->req->sp, &sa));	\
+		CHECK_OBJ_NOTNULL(ctx->sp, SESS_MAGIC);		\
+		AZ(SES_Get_##fld##_addr(ctx->sp, &sa));		\
 		return (sa);					\
 	}
 
@@ -670,11 +684,36 @@ VRT_r_resp_is_streaming(VRT_CTX)
 	if (ctx->req->objcore == NULL)
 		return (0);	/* When called from vcl_synth */
 	CHECK_OBJ_NOTNULL(ctx->req->objcore, OBJCORE_MAGIC);
-	return (ctx->req->objcore->flags & OC_F_INCOMPLETE ? 1 : 0);
+	return (ctx->req->objcore->boc == NULL ? 0 : 1);
 }
 
 /*--------------------------------------------------------------------*/
 
+#define VRT_BODY_L(which)					\
+void								\
+VRT_l_##which##_body(VRT_CTX, const char *str, ...)		\
+{								\
+	va_list ap;						\
+	const char *p;						\
+	struct vsb *vsb;					\
+								\
+	CAST_OBJ_NOTNULL(vsb, ctx->specific, VSB_MAGIC);	\
+	va_start(ap, str);					\
+	p = str;						\
+	while (p != vrt_magic_string_end) {			\
+		if (p == NULL)					\
+			p = "(null)";				\
+		VSB_cat(vsb, p);				\
+		p = va_arg(ap, const char *);			\
+	}							\
+	va_end(ap);						\
+}
+
+VRT_BODY_L(beresp)
+VRT_BODY_L(resp)
+
+/*--------------------------------------------------------------------*/
+
 #define HTTP_VAR(x)						\
 struct http *							\
 VRT_r_##x(VRT_CTX)				\
