@@ -37,7 +37,7 @@
 
 #include "cache.h"
 
-#include "vcli_priv.h"
+#include "vcli_serve.h"
 #include "vrt.h"
 
 /*--------------------------------------------------------------------
@@ -149,10 +149,7 @@ VRT_Vmod_Fini(struct vmod **hdl)
 
 	ASSERT_CLI();
 
-	AN(hdl);
-	v = *hdl;
-	*hdl = NULL;
-	CHECK_OBJ_NOTNULL(v, VMOD_MAGIC);
+	TAKE_OBJ_NOTNULL(v, hdl, VMOD_MAGIC);
 
 #ifndef DONT_DLCLOSE_VMODS
 	/*
@@ -186,8 +183,7 @@ ccf_debug_vmod(struct cli *cli, const char * const *av, void *priv)
 }
 
 static struct cli_proto vcl_cmds[] = {
-	{ "debug.vmod", "debug.vmod", "\tShow loaded vmods.", 0, 0,
-		"d", ccf_debug_vmod },
+	{ CLICMD_DEBUG_VMOD,			"d", ccf_debug_vmod },
 	{ NULL }
 };
 
