@@ -38,14 +38,179 @@
 #include <stdlib.h>
 
 #include "cache/cache.h"
+#include "cache/cache_filter.h"
+#include "cache/cache_transport.h"
 #include "cache_http1.h"
 #include "hash/hash_slinger.h"
 
+#include "vsb.h"
 #include "vtcp.h"
 
-static const struct transport http1_transport = {
-	.magic = TRANSPORT_MAGIC,
-	.deliver = V1D_Deliver,
+static const char H1NEWREQ[] = "HTTP1::NewReq";
+static const char H1PROC[] = "HTTP1::Proc";
+static const char H1BUSY[] = "HTTP1::Busy";
+static const char H1CLEANUP[] = "HTTP1::Cleanup";
+
+static void
+http1_setstate(const struct sess *sp, const char *s)
+{
+	uintptr_t p;
+
+	p = (uintptr_t)s;
+	AZ(SES_Set_xport_priv(sp, &p));
+}
+
+static const char *
+http1_getstate(const struct sess *sp)
+{
+	uintptr_t *p;
+
+	AZ(SES_Get_xport_priv(sp, &p));
+	return (const char *)*p;
+}
+
+/*--------------------------------------------------------------------
+ * Call protocol for this request
+ */
+
+static void __match_proto__(task_func_t)
+http1_req(struct worker *wrk, void *arg)
+{
+	struct req *req;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CAST_OBJ_NOTNULL(req, arg, REQ_MAGIC);
+
+	THR_SetRequest(req);
+	req->transport = &HTTP1_transport;
+	AZ(wrk->aws->r);
+	HTTP1_Session(wrk, req);
+	AZ(wrk->v1l);
+	WS_Assert(wrk->aws);
+	THR_SetRequest(NULL);
+}
+
+/*--------------------------------------------------------------------
+ * Call protocol for this session (new or from waiter)
+ *
+ * When sessions are rescheduled from the waiter, a struct pool_task
+ * is put on the reserved session workspace (for reasons of memory
+ * conservation).  This reservation is released as the first thing.
+ * The acceptor and any other code which schedules this function
+ * must obey this calling convention with a dummy reservation.
+ */
+
+static void __match_proto__(task_func_t)
+http1_new_session(struct worker *wrk, void *arg)
+{
+	struct sess *sp;
+	struct req *req;
+	uintptr_t *u;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CAST_OBJ_NOTNULL(req, arg, REQ_MAGIC);
+	sp = req->sp;
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+
+	HTC_RxInit(req->htc, req->ws);
+	SES_Reserve_xport_priv(sp, &u);
+	http1_setstate(sp, H1NEWREQ);
+	wrk->task.func = http1_req;
+	wrk->task.priv = req;
+}
+
+static void __match_proto__(task_func_t)
+http1_unwait(struct worker *wrk, void *arg)
+{
+	struct sess *sp;
+	struct req *req;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CAST_OBJ_NOTNULL(sp, arg, SESS_MAGIC);
+	WS_Release(sp->ws, 0);
+	req = Req_New(wrk, sp);
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	req->htc->fd = sp->fd;
+	HTC_RxInit(req->htc, req->ws);
+	http1_setstate(sp, H1NEWREQ);
+	wrk->task.func = http1_req;
+	wrk->task.priv = req;
+}
+
+static void __match_proto__(vtr_req_body_t)
+http1_req_body(struct req *req)
+{
+
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	switch(req->htc->body_status) {
+	case BS_EOF:
+	case BS_LENGTH:
+	case BS_CHUNKED:
+		if (V1F_Setup_Fetch(req->htc->vfc, req->htc) != 0)
+			req->req_body_status = REQ_BODY_FAIL;
+		break;
+	default:
+		break;
+	}
+}
+
+static void
+http1_sess_panic(struct vsb *vsb, const struct sess *sp)
+{
+
+	VSB_printf(vsb, "state = %s\n", http1_getstate(sp));
+}
+
+static void
+http1_req_panic(struct vsb *vsb, const struct req *req)
+{
+
+	VSB_printf(vsb, "state = %s\n", http1_getstate(req->sp));
+}
+
+static void __match_proto__(vtr_req_fail_f)
+http1_req_fail(struct req *req, enum sess_close reason)
+{
+	if (req->sp->fd >= 0)
+		SES_Close(req->sp, reason);
+}
+
+static void __match_proto__(vtr_reembark_f)
+http1_reembark(struct worker *wrk, struct req *req)
+{
+	struct sess *sp;
+
+	sp = req->sp;
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+
+	http1_setstate(sp, H1BUSY);
+
+	if (!SES_Reschedule_Req(req))
+		return;
+
+	/* Couldn't schedule, ditch */
+	wrk->stats->busy_wakeup--;
+	wrk->stats->busy_killed++;
+	AN (req->vcl);
+	VCL_Rel(&req->vcl);
+	CNT_AcctLogCharge(wrk->stats, req);
+	Req_Release(req);
+	SES_Delete(sp, SC_OVERLOAD, NAN);
+	DSL(DBG_WAITINGLIST, req->vsl->wid, "kill from waiting list");
+	usleep(10000);
+}
+
+struct transport HTTP1_transport = {
+	.name =			"HTTP/1",
+	.magic =		TRANSPORT_MAGIC,
+	.deliver =		V1D_Deliver,
+	.unwait =		http1_unwait,
+	.req_body =		http1_req_body,
+	.req_fail =		http1_req_fail,
+	.new_session =		http1_new_session,
+	.sess_panic =		http1_sess_panic,
+	.req_panic =		http1_req_panic,
+	.reembark =		http1_reembark,
 };
 
 /*----------------------------------------------------------------------
@@ -159,6 +324,7 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 {
 	enum htc_status_e hs;
 	struct sess *sp;
+	const char *st;
 	int i;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
@@ -172,7 +338,7 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 	 * or waiter, but we'd rather do the syscall in the worker thread.
 	 * On systems which return errors for ioctl, we close early
 	 */
-	if (sp->sess_step == S_STP_H1NEWREQ && VTCP_blocking(sp->fd)) {
+	if (http1_getstate(sp) == H1NEWREQ && VTCP_blocking(sp->fd)) {
 		if (errno == ECONNRESET)
 			SES_Close(sp, SC_REM_CLOSE);
 		else
@@ -182,28 +348,18 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 	}
 
 	while (1) {
-		switch (sp->sess_step) {
-		case S_STP_H1NEWSESS:
-			if (VTCP_blocking(sp->fd)) {
-				if (errno == ECONNRESET)
-					SES_Close(sp, SC_REM_CLOSE);
-				else
-					SES_Close(sp, SC_TX_ERROR);
-				AN(Req_Cleanup(sp, wrk, req));
-				return;
-			}
-			sp->sess_step = S_STP_H1NEWREQ;
-			break;
-		case S_STP_H1NEWREQ:
+		st = http1_getstate(sp);
+		if (st == H1NEWREQ) {
 			assert(isnan(req->t_prev));
 			assert(isnan(req->t_req));
 			AZ(req->vcl);
 			AZ(req->esi_level);
 
-			hs = SES_RxStuff(req->htc, HTTP1_Complete,
+			hs = HTC_RxStuff(req->htc, HTTP1_Complete,
 			    &req->t_first, &req->t_req,
 			    sp->t_idle + cache_param->timeout_linger,
-			    sp->t_idle + cache_param->timeout_idle);
+			    sp->t_idle + cache_param->timeout_idle,
+			    cache_param->http_req_size);
 			XXXAZ(req->htc->ws->r);
 			if (hs < HTC_S_EMPTY) {
 				req->acct.req_hdrbytes +=
@@ -230,24 +386,55 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 			if (hs == HTC_S_IDLE) {
 				wrk->stats->sess_herd++;
 				Req_Release(req);
-				SES_Wait(sp);
+				SES_Wait(sp, &HTTP1_transport);
 				return;
 			}
 			if (hs != HTC_S_COMPLETE)
 				WRONG("htc_status (nonbad)");
 
+			if (H2_prism_complete(req->htc) == HTC_S_COMPLETE) {
+				if (!FEATURE(FEATURE_HTTP2)) {
+					VSLb(req->vsl, SLT_Debug,
+					    "H2 attempt");
+					SES_Close(req->sp, req->doclose);
+					http1_setstate(sp, H1CLEANUP);
+					continue;
+				}
+				VSLb(req->vsl, SLT_Debug,
+				    "H2 Prior Knowledge Upgrade");
+				http1_setstate(sp, NULL);
+				req->err_code = 1;
+				SES_SetTransport(wrk, sp, req, &H2_transport);
+				return;
+			}
+
 			i = http1_dissect(wrk, req);
 			req->acct.req_hdrbytes +=
 			    req->htc->rxbuf_e - req->htc->rxbuf_b;
 			if (i) {
 				SES_Close(req->sp, req->doclose);
-				sp->sess_step = S_STP_H1CLEANUP;
-				break;
+				http1_setstate(sp, H1CLEANUP);
+				continue;
+			}
+			if (http_HdrIs(req->http, H_Upgrade, "h2c")) {
+				if (!FEATURE(FEATURE_HTTP2)) {
+					VSLb(req->vsl, SLT_Debug,
+					    "H2 upgrade attempt");
+				} else if (req->htc->body_status != BS_NONE) {
+					VSLb(req->vsl, SLT_Debug,
+					    "H2 upgrade attempt has body");
+				} else {
+					VSLb(req->vsl, SLT_Debug, "H2 Upgrade");
+					http1_setstate(sp, NULL);
+					req->err_code = 2;
+					SES_SetTransport(wrk, sp, req,
+					    &H2_transport);
+					return;
+				}
 			}
 			req->req_step = R_STP_RECV;
-			sp->sess_step = S_STP_H1PROC;
-			break;
-		case S_STP_H1BUSY:
+			http1_setstate(sp, H1PROC);
+		} else if (st == H1BUSY) {
 			/*
 			 * Return from waitinglist.
 			 * Check to see if the remote has left.
@@ -260,32 +447,26 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 				AN(Req_Cleanup(sp, wrk, req));
 				return;
 			}
-			sp->sess_step = S_STP_H1PROC;
-			break;
-		case S_STP_H1PROC:
-			req->transport = &http1_transport;
-			req->task.func = SES_Proto_Req;
+			http1_setstate(sp, H1PROC);
+		} else if (st == H1PROC) {
+			req->transport = &HTTP1_transport;
+			req->task.func = http1_req;
 			req->task.priv = req;
-			if (CNT_Request(wrk, req) == REQ_FSM_DISEMBARK) {
-				sp->sess_step = S_STP_H1BUSY;
+			if (CNT_Request(wrk, req) == REQ_FSM_DISEMBARK)
 				return;
-			}
 			req->transport = NULL;
 			req->task.func = NULL;
 			req->task.priv = NULL;
-			sp->sess_step = S_STP_H1CLEANUP;
-			break;
-		case S_STP_H1CLEANUP:
+			http1_setstate(sp, H1CLEANUP);
+		} else if (st == H1CLEANUP) {
 			if (Req_Cleanup(sp, wrk, req))
 				return;
-			SES_RxReInit(req->htc);
+			HTC_RxInit(req->htc, req->ws);
 			if (req->htc->rxbuf_e != req->htc->rxbuf_b)
 				wrk->stats->sess_readahead++;
-			sp->sess_step = S_STP_H1NEWREQ;
-			break;
-		default:
+			http1_setstate(sp, H1NEWREQ);
+		} else {
 			WRONG("Wrong H1 session state");
 		}
-
 	}
 }
