@@ -68,8 +68,6 @@ HTTP1_Complete(struct http_conn *htc)
 	char *p;
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
-	AZ(htc->pipeline_b);
-	AZ(htc->pipeline_e);
 
 	assert(htc->rxbuf_e >= htc->rxbuf_b);
 	assert(*htc->rxbuf_e == '\0');
@@ -77,12 +75,9 @@ HTTP1_Complete(struct http_conn *htc)
 	/* Skip any leading white space */
 	for (p = htc->rxbuf_b ; vct_islws(*p); p++)
 		continue;
-	if (p == htc->rxbuf_e) {
-		/* All white space */
-		htc->rxbuf_e = htc->rxbuf_b;
-		*htc->rxbuf_e = '\0';
+	if (p == htc->rxbuf_e)
 		return (HTC_S_EMPTY);
-	}
+
 	/*
 	 * Here we just look for NL[CR]NL to see that reception
 	 * is completed.  More stringent validation happens later.
@@ -106,7 +101,8 @@ HTTP1_Complete(struct http_conn *htc)
  */
 
 static uint16_t
-http1_dissect_hdrs(struct http *hp, char *p, struct http_conn *htc)
+http1_dissect_hdrs(struct http *hp, char *p, struct http_conn *htc,
+    unsigned maxhdr)
 {
 	char *q, *r;
 
@@ -147,7 +143,7 @@ http1_dissect_hdrs(struct http *hp, char *p, struct http_conn *htc)
 				*q++ = ' ';
 		}
 
-		if (q - p > htc->maxhdr) {
+		if (q - p > maxhdr) {
 			VSLb(hp->vsl, SLT_BogoHeader, "Header too long: %.*s",
 			    (int)(q - p > 20 ? 20 : q - p), p);
 			return (400);
@@ -204,11 +200,8 @@ http1_dissect_hdrs(struct http *hp, char *p, struct http_conn *htc)
 	}
 	if (p < htc->rxbuf_e)
 		p += vct_skipcrlf(p);
-	if (p < htc->rxbuf_e) {
-		htc->pipeline_b = p;
-		htc->pipeline_e = htc->rxbuf_e;
-		htc->rxbuf_e = p;
-	}
+	HTC_RxPipeline(htc, p);
+	htc->rxbuf_e = p;
 	return (0);
 }
 
@@ -217,7 +210,8 @@ http1_dissect_hdrs(struct http *hp, char *p, struct http_conn *htc)
  */
 
 static uint16_t
-http1_splitline(struct http *hp, struct http_conn *htc, const int *hf)
+http1_splitline(struct http *hp, struct http_conn *htc, const int *hf,
+    unsigned maxhdr)
 {
 	char *p;
 	int i;
@@ -283,7 +277,7 @@ http1_splitline(struct http *hp, struct http_conn *htc, const int *hf)
 	*p = '\0';
 	p += i;
 
-	return (http1_dissect_hdrs(hp, p, htc));
+	return (http1_dissect_hdrs(hp, p, htc, maxhdr));
 }
 
 /*--------------------------------------------------------------------*/
@@ -361,7 +355,8 @@ HTTP1_DissectRequest(struct http_conn *htc, struct http *hp)
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 
-	retval = http1_splitline(hp, htc, HTTP1_Req);
+	retval = http1_splitline(hp, htc,
+	    HTTP1_Req, cache_param->http_req_hdr_len);
 	if (retval != 0)
 		return (retval);
 	hp->protover = http1_proto_ver(hp);
@@ -376,14 +371,13 @@ HTTP1_DissectRequest(struct http_conn *htc, struct http *hp)
 
 	/* RFC2616, section 5.2, point 1 */
 	if (!strncasecmp(hp->hd[HTTP_HDR_URL].b, "http://", 7))
-		b = e = hp->hd[HTTP_HDR_URL].b + 7;
+		b = hp->hd[HTTP_HDR_URL].b + 7;
 	else if (FEATURE(FEATURE_HTTPS_SCHEME) &&
-			!strncasecmp(hp->hd[HTTP_HDR_URL].b, "https://", 8))
-		b = e = hp->hd[HTTP_HDR_URL].b + 8;
+	    !strncasecmp(hp->hd[HTTP_HDR_URL].b, "https://", 8))
+		b = hp->hd[HTTP_HDR_URL].b + 8;
 	if (b) {
-		while (*e != '/' && *e != '\0')
-			e++;
-		if (*e == '/') {
+		e = strchr(b, '/');
+		if (e) {
 			http_Unset(hp, H_Host);
 			http_PrintfHeader(hp, "Host: %.*s", (int)(e - b), b);
 			hp->hd[HTTP_HDR_URL].b = e;
@@ -427,7 +421,8 @@ HTTP1_DissectResponse(struct http_conn *htc, struct http *hp,
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 	CHECK_OBJ_NOTNULL(req, HTTP_MAGIC);
 
-	if (http1_splitline(hp, htc, HTTP1_Resp))
+	if (http1_splitline(hp, htc,
+	    HTTP1_Resp, cache_param->http_resp_hdr_len))
 		retval = 503;
 
 	if (retval == 0) {
@@ -459,14 +454,14 @@ HTTP1_DissectResponse(struct http_conn *htc, struct http *hp,
 		    (int)(htc->rxbuf_e - htc->rxbuf_b), htc->rxbuf_b);
 		assert(retval >= 100 && retval <= 999);
 		assert(retval == 503);
-		hp->status = retval;
-		http_SetH(hp, HTTP_HDR_STATUS, "503");
-		http_SetH(hp, HTTP_HDR_REASON, http_Status2Reason(retval));
+		http_SetStatus(hp, 503);
 	}
 
 	if (hp->hd[HTTP_HDR_REASON].b == NULL ||
-	    !Tlen(hp->hd[HTTP_HDR_REASON]))
-		http_SetH(hp, HTTP_HDR_REASON, http_Status2Reason(hp->status));
+	    !Tlen(hp->hd[HTTP_HDR_REASON])) {
+		http_SetH(hp, HTTP_HDR_REASON,
+		    http_Status2Reason(hp->status, NULL));
+	}
 
 	htc->body_status = http1_body_status(hp, htc, 0);
 
