@@ -35,6 +35,8 @@
 
 struct cli;
 struct parspec;
+struct vcc;
+struct vclprog;
 
 extern struct vev_base	*mgt_evb;
 extern unsigned		d_flag;
@@ -67,6 +69,10 @@ void mgt_cli_telnet(const char *T_arg);
 void mgt_cli_master(const char *M_arg);
 void mgt_cli_secret(const char *S_arg);
 void mgt_cli_close_all(void);
+void mgt_DumpRstCli(void);
+void mgt_cli_init_cls(void);
+#define MCF_NOAUTH	0	/* NB: zero disables here-documents */
+#define MCF_AUTH	16
 
 /* mgt_jail.c */
 
@@ -133,9 +139,14 @@ void MGT_complain(const char *loud, const char *, ...) __v_printflike(2, 3);
 /* mgt_param.c */
 void MCF_InitParams(struct cli *);
 void MCF_CollectParams(void);
-void MCF_SetDefault(const char *param, const char *def);
-void MCF_SetMinimum(const char *param, const char *def);
-void MCF_SetMaximum(const char *param, const char *def);
+enum mcf_which_e {
+	MCF_DEFAULT = 32,
+	MCF_MINIMUM = 33,
+	MCF_MAXIMUM = 34,
+};
+void MCF_ParamConf(enum mcf_which_e, const char *param, const char *, ...)
+    __v_printflike(3, 4);
+
 void MCF_ParamSet(struct cli *, const char *param, const char *val);
 void MCF_ParamProtect(struct cli *, const char *arg);
 void MCF_DumpRstParam(void);
@@ -150,7 +161,7 @@ void mgt_SHM_Init(void);
 void mgt_SHM_static_alloc(const void *, ssize_t size,
     const char *class, const char *type, const char *ident);
 void mgt_SHM_Create(void);
-void mgt_SHM_Commit(void);
+int mgt_SHM_Commit(void);
 void mgt_SHM_Destroy(int keep);
 void mgt_SHM_Size_Adjust(void);
 
@@ -160,18 +171,19 @@ void STV_Config(const char *spec);
 void STV_Config_Transient(void);
 
 /* mgt_vcc.c */
-char *mgt_VccCompile(struct cli *, const char *vclname, const char *vclsrc,
-    const char *vclsrcfile, int C_flag);
-void mgt_vcc_init(void);
+char *mgt_VccCompile(struct cli *, struct vclprog *, const char *vclname,
+    const char *vclsrc, const char *vclsrcfile, int C_flag);
 
 void mgt_vcl_init(void);
 void mgt_vcc_startup(struct cli *, const char *b_arg, const char *f_arg,
     const char *vclsrc, int Cflag);
 int mgt_push_vcls_and_start(struct cli *, unsigned *status, char **p);
+void mgt_vcl_export_labels(struct vcc *);
 int mgt_has_vcl(void);
+void mgt_vcl_depends(struct vclprog *vp1, const char *name);
 extern char *mgt_cc_cmd;
-extern const char *mgt_vcl_dir;
-extern const char *mgt_vmod_dir;
+extern const char *mgt_vcl_path;
+extern const char *mgt_vmod_path;
 extern unsigned mgt_vcc_err_unref;
 extern unsigned mgt_vcc_allow_inline_c;
 extern unsigned mgt_vcc_unsafe_path;
