@@ -47,14 +47,11 @@
 #include "common/heritage.h"
 
 #include "vbm.h"
-#include "vcli.h"
-#include "vcli_priv.h"
+#include "vcli_serve.h"
 #include "vev.h"
 #include "vlu.h"
 #include "vtim.h"
 
-#include "mgt_cli.h"
-
 pid_t			child_pid = -1;
 
 static struct vbitmap	*fd_map;
@@ -111,11 +108,11 @@ mgt_panic_record(pid_t r)
 	char time_str[30];
 
 	if (child_panic != NULL)
-		VSB_delete(child_panic);
+		VSB_destroy(&child_panic);
 	child_panic = VSB_new_auto();
 	AN(child_panic);
 	VTIM_format(VTIM_real(), time_str);
-	VSB_printf(child_panic, "Last panic at: %s\n", time_str);
+	VSB_printf(child_panic, "Panic at: %s\n", time_str);
 	VSB_quote(child_panic, heritage.panic_str,
 	    strnlen(heritage.panic_str, heritage.panic_str_len),
 	    VSB_QUOTE_NONL);
@@ -127,11 +124,10 @@ mgt_panic_record(pid_t r)
 static void
 mgt_panic_clear(void)
 {
-	VSB_delete(child_panic);
-	child_panic = NULL;
+	VSB_destroy(&child_panic);
 }
 
-void __match_proto__(cli_func_t)
+static void __match_proto__(cli_func_t)
 mcf_panic_show(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)av;
@@ -147,7 +143,7 @@ mcf_panic_show(struct cli *cli, const char * const *av, void *priv)
 	VCLI_Out(cli, "%s\n", VSB_data(child_panic));
 }
 
-void __match_proto__(cli_func_t)
+static void __match_proto__(cli_func_t)
 mcf_panic_clear(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)priv;
@@ -207,7 +203,7 @@ mgt_child_inherit(int fd, const char *what)
 
 	assert(fd >= 0);
 	if (fd_map == NULL)
-		fd_map = vbit_init(128);
+		fd_map = vbit_new(128);
 	AN(fd_map);
 	if (what != NULL)
 		vbit_set(fd_map, fd);
@@ -252,6 +248,8 @@ child_listener(const struct vev *e, int what)
 static int __match_proto__(vev_cb_f)
 child_poker(const struct vev *e, int what)
 {
+	char *r = NULL;
+	unsigned status;
 
 	(void)e;
 	(void)what;
@@ -259,9 +257,14 @@ child_poker(const struct vev *e, int what)
 		return (1);
 	if (child_pid < 0)
 		return (0);
-	if (!mgt_cli_askchild(NULL, NULL, "ping\n"))
-		return (0);
-	return (0);
+	if (mgt_cli_askchild(&status, &r, "ping\n") || strncmp("PONG ", r, 5)) {
+		MGT_complain(C_ERR, "Unexpected reply from ping: %u %s",
+		    status, r);
+		if (status != CLIS_COMMS)
+			MGT_Child_Cli_Fail();
+	}
+	free(r);
+	return 0;
 }
 
 /*=====================================================================
@@ -534,7 +537,7 @@ mgt_reap_child(void)
 #endif
 	AZ(VSB_finish(vsb));
 	MGT_complain(status ? C_ERR : C_INFO, "%s", VSB_data(vsb));
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 
 	/* Dispose of shared memory but evacuate panic messages first */
 	if (heritage.panic_str[0] != '\0') {
@@ -614,17 +617,16 @@ mgt_stop_child(void)
 }
 
 /*=====================================================================
- * CLI command to start/stop child
+ * CLI commands to start/stop child
  */
 
-void __match_proto__(cli_func_t)
-mcf_server_startstop(struct cli *cli, const char * const *av, void *priv)
+static void __match_proto__(cli_func_t)
+mcf_server_start(struct cli *cli, const char * const *av, void *priv)
 {
 
 	(void)av;
-	if (priv != NULL && child_state == CH_RUNNING)
-		mgt_stop_child();
-	else if (priv == NULL && child_state == CH_STOPPED) {
+	(void)priv;
+	if (child_state == CH_STOPPED) {
 		if (mgt_has_vcl()) {
 			mgt_launch_child(cli);
 		} else {
@@ -637,9 +639,23 @@ mcf_server_startstop(struct cli *cli, const char * const *av, void *priv)
 	}
 }
 
+static void __match_proto__(cli_func_t)
+mcf_server_stop(struct cli *cli, const char * const *av, void *priv)
+{
+
+	(void)av;
+	(void)priv;
+	if (child_state == CH_RUNNING) {
+		mgt_stop_child();
+	} else {
+		VCLI_SetResult(cli, CLIS_CANT);
+		VCLI_Out(cli, "Child in state %s", ch_state[child_state]);
+	}
+}
+
 /*--------------------------------------------------------------------*/
 
-void
+static void
 mcf_server_status(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)av;
@@ -678,6 +694,17 @@ mgt_uptime(const struct vev *e, int what)
 	return (0);
 }
 
+/*--------------------------------------------------------------------*/
+
+static struct cli_proto cli_child[] = {
+	{ CLICMD_SERVER_STATUS,		"", mcf_server_status },
+	{ CLICMD_SERVER_START,		"", mcf_server_start },
+	{ CLICMD_SERVER_STOP,		"", mcf_server_stop },
+	{ CLICMD_PANIC_SHOW,		"", mcf_panic_show },
+	{ CLICMD_PANIC_CLEAR,		"", mcf_panic_clear },
+	{ NULL }
+};
+
 /*=====================================================================
  * This thread is the master thread in the management process.
  * The relatively simple task is to start and stop the child process
@@ -691,6 +718,8 @@ MGT_Run(void)
 	struct vev *e;
 	int i;
 
+	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_child);
+
 	mgt_uptime_t0 = VTIM_real();
 	e = vev_new();
 	XXXAN(e);
@@ -735,7 +764,11 @@ MGT_Run(void)
 		}
 	}
 
-	mgt_SHM_Commit();
+	i = mgt_SHM_Commit();
+	if (i != 0) {
+		MGT_complain(C_ERR, "Could not commit SHM file");
+		return;
+	}
 
 	i = vev_schedule(mgt_evb);
 	if (i != 0)
