@@ -45,9 +45,6 @@
 
 #include "mgt/mgt.h"
 
-#include "vcli.h"
-#include "vcli_common.h"
-#include "vcli_priv.h"
 #include "vcli_serve.h"
 #include "vev.h"
 #include "vrnd.h"
@@ -55,15 +52,23 @@
 #include "vss.h"
 #include "vtcp.h"
 
-#include "mgt_cli.h"
+#define CLI_CMD(U,l,s,h,d,m,M) \
+const struct cli_cmd_desc CLICMD_##U[1] = {{ l, s, h, d, m, M }};
+#include "tbl/cli_cmds.h"
+#undef CLI_CMD
+
+static const struct cli_cmd_desc *cmds[] = {
+#define CLI_CMD(U,l,s,h,d,m,M) CLICMD_##U,
+#include "tbl/cli_cmds.h"
+#undef CLI_CMD
+};
+
+static const int ncmds = sizeof cmds / sizeof cmds[0];
 
 static int		cli_i = -1, cli_o = -1;
-static struct VCLS	*cls;
+struct VCLS		*mgt_cls;
 static const char	*secret_file;
 
-#define	MCF_NOAUTH	0	/* NB: zero disables here-documents */
-#define MCF_AUTH	16
-
 /*--------------------------------------------------------------------*/
 
 static void
@@ -87,22 +92,8 @@ mcf_banner(struct cli *cli, const char *const *av, void *priv)
 
 /*--------------------------------------------------------------------*/
 
-/* XXX: what order should this list be in ? */
 static struct cli_proto cli_proto[] = {
-	{ CLI_BANNER,		"", mcf_banner, NULL },
-	{ CLI_SERVER_STATUS,	"", mcf_server_status, NULL },
-	{ CLI_SERVER_START,	"", mcf_server_startstop, NULL },
-	{ CLI_SERVER_STOP,	"", mcf_server_startstop, cli_proto },
-	{ CLI_VCL_LOAD,		"", mcf_vcl_load, NULL },
-	{ CLI_VCL_INLINE,	"", mcf_vcl_inline, NULL },
-	{ CLI_VCL_USE,		"", mcf_vcl_use, NULL },
-	{ CLI_VCL_STATE,	"", mcf_vcl_state, NULL },
-	{ CLI_VCL_DISCARD,	"", mcf_vcl_discard, NULL },
-	{ CLI_VCL_LIST,		"", mcf_vcl_list, NULL },
-	{ CLI_PARAM_SHOW,	"", mcf_param_show, NULL },
-	{ CLI_PARAM_SET,	"", mcf_param_set, NULL },
-	{ CLI_PANIC_SHOW,	"", mcf_panic_show, NULL },
-	{ CLI_PANIC_CLEAR,	"", mcf_panic_clear, NULL },
+	{ CLICMD_BANNER,		"", mcf_banner },
 	{ NULL }
 };
 
@@ -119,9 +110,7 @@ mcf_panic(struct cli *cli, const char * const *av, void *priv)
 }
 
 static struct cli_proto cli_debug[] = {
-	{ "debug.panic.master", "debug.panic.master",
-		"\tPanic the master process.",
-		0, 0, "d", mcf_panic, NULL},
+	{ CLICMD_DEBUG_PANIC_MASTER,		"d", mcf_panic },
 	{ NULL }
 };
 
@@ -141,10 +130,6 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 	 * running.
 	 */
 	if (cli_o <= 0) {
-		if (!strcmp(av[1], "help")) {
-			VCLI_Out(cli, "No help from child, (not running).\n");
-			return;
-		}
 		VCLI_SetResult(cli, CLIS_UNKNOWN);
 		VCLI_Out(cli,
 		    "Unknown request in manager process "
@@ -161,13 +146,13 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 	AZ(VSB_finish(vsb));
 	i = write(cli_o, VSB_data(vsb), VSB_len(vsb));
 	if (i != VSB_len(vsb)) {
-		VSB_delete(vsb);
+		VSB_destroy(&vsb);
 		VCLI_SetResult(cli, CLIS_COMMS);
 		VCLI_Out(cli, "CLI communication error");
 		MGT_Child_Cli_Fail();
 		return;
 	}
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 	if (VCLI_ReadResult(cli_i, &u, &q, mgt_param.cli_timeout))
 		MGT_Child_Cli_Fail();
 	VCLI_SetResult(cli, u);
@@ -175,9 +160,11 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 	free(q);
 }
 
+static const struct cli_cmd_desc CLICMD_WILDCARD[1] =
+    {{ "*", "<wild-card-entry>", "<fall through to cacher>", "", 0, -1 }};
+
 static struct cli_proto cli_askchild[] = {
-	{ "*", "<wild-card-entry>", "\t<fall through to cacher>\n",
-		0, 9999, "h*", mcf_askchild, NULL},
+	{ CLICMD_WILDCARD, "h*", mcf_askchild },
 	{ NULL }
 };
 
@@ -254,10 +241,13 @@ static void
 mgt_cli_challenge(struct cli *cli)
 {
 	int i;
+	uint8_t u;
 
-	VRND_Seed();
-	for (i = 0; i + 2L < sizeof cli->challenge; i++)
-		cli->challenge[i] = (random() % 26) + 'a';
+	AZ(VRND_RandomCrypto(cli->challenge, sizeof cli->challenge - 2));
+	for (i = 0; i < (sizeof cli->challenge) - 2; i++) {
+		AZ(VRND_RandomCrypto(&u, sizeof u));
+		cli->challenge[i] = (u % 26) + 'a';
+	}
 	cli->challenge[i++] = '\n';
 	cli->challenge[i] = '\0';
 	VCLI_Out(cli, "%s", cli->challenge);
@@ -307,11 +297,31 @@ mcf_auth(struct cli *cli, const char *const *av, void *priv)
 	mcf_banner(cli, av, priv);
 }
 
+/*--------------------------------------------------------------------*/
+
+static void __match_proto__(cli_func_t)
+mcf_help(struct cli *cli, const char * const *av, void *priv)
+{
+	if (cli_o <= 0)
+		VCLS_func_help(cli, av, priv);
+	else
+		mcf_askchild(cli, av, priv);
+}
+
+static void __match_proto__(cli_func_t)
+mcf_help_json(struct cli *cli, const char * const *av, void *priv)
+{
+	if (cli_o <= 0)
+		VCLS_func_help_json(cli, av, priv);
+	else
+		mcf_askchild(cli, av, priv);
+}
+
 static struct cli_proto cli_auth[] = {
-	{ CLI_HELP,		"", VCLS_func_help, NULL },
-	{ CLI_PING,		"", VCLS_func_ping },
-	{ CLI_AUTH,		"", mcf_auth, NULL },
-	{ CLI_QUIT,		"", VCLS_func_close, NULL},
+	{ CLICMD_HELP,		"", mcf_help, mcf_help_json },
+	{ CLICMD_PING,		"", VCLS_func_ping },
+	{ CLICMD_AUTH,		"", mcf_auth },
+	{ CLICMD_QUIT,		"", VCLS_func_close },
 	{ NULL }
 };
 
@@ -334,18 +344,17 @@ mgt_cli_cb_after(const struct cli *cli)
 
 /*--------------------------------------------------------------------*/
 
-static void
+void
 mgt_cli_init_cls(void)
 {
 
-	cls = VCLS_New(mgt_cli_cb_before, mgt_cli_cb_after,
+	mgt_cls = VCLS_New(mgt_cli_cb_before, mgt_cli_cb_after,
 	    &mgt_param.cli_buffer, &mgt_param.cli_limit);
-	AN(cls);
-	AZ(VCLS_AddFunc(cls, MCF_NOAUTH, cli_auth));
-	AZ(VCLS_AddFunc(cls, MCF_AUTH, cli_proto));
-	AZ(VCLS_AddFunc(cls, MCF_AUTH, cli_debug));
-	AZ(VCLS_AddFunc(cls, MCF_AUTH, cli_stv));
-	AZ(VCLS_AddFunc(cls, MCF_AUTH, cli_askchild));
+	AN(mgt_cls);
+	VCLS_AddFunc(mgt_cls, MCF_NOAUTH, cli_auth);
+	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_proto);
+	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_debug);
+	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_askchild);
 }
 
 /*--------------------------------------------------------------------
@@ -356,7 +365,7 @@ void
 mgt_cli_close_all(void)
 {
 
-	VCLS_Destroy(&cls);
+	VCLS_Destroy(&mgt_cls);
 }
 
 /*--------------------------------------------------------------------
@@ -370,7 +379,7 @@ mgt_cli_callback2(const struct vev *e, int what)
 
 	(void)e;
 	(void)what;
-	i = VCLS_PollFd(cls, e->fd, 0);
+	i = VCLS_PollFd(mgt_cls, e->fd, 0);
 	return (i);
 }
 
@@ -385,10 +394,8 @@ mgt_cli_setup(int fdi, int fdo, int verbose, const char *ident,
 
 	(void)ident;
 	(void)verbose;
-	if (cls == NULL)
-		mgt_cli_init_cls();
 
-	cli = VCLS_AddFd(cls, fdi, fdo, closefunc, priv);
+	cli = VCLS_AddFd(mgt_cls, fdi, fdo, closefunc, priv);
 
 	cli->ident = strdup(ident);
 
@@ -483,7 +490,7 @@ telnet_accept(const struct vev *ev, int what)
 	tn = telnet_new(i);
 	vsb = sock_id("telnet", i);
 	mgt_cli_setup(i, i, 0, VSB_data(vsb), telnet_close, tn);
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 	return (0);
 }
 
@@ -558,14 +565,14 @@ mgt_cli_telnet(const char *T_arg)
 	AN(vsb);
 	error = VSS_resolver(T_arg, NULL, mct_callback, vsb, &err);
 	if (err != NULL)
-		ARGV_ERR("Could resolve -T argument to address\n\t%s\n", err);
+		ARGV_ERR("Could not resolve -T argument to address\n\t%s\n", err);
 	AZ(error);
 	AZ(VSB_finish(vsb));
 	if (VSB_len(vsb) == 0)
 		ARGV_ERR("-T %s could not be listened on.", T_arg);
 	/* Save in shmem */
 	mgt_SHM_static_alloc(VSB_data(vsb), VSB_len(vsb) + 1, "Arg", "-T", "");
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 }
 
 /* Reverse CLI ("Master") connections --------------------------------*/
@@ -615,7 +622,7 @@ Marg_connect(const struct vev *e, int what)
 	}
 	vsb = sock_id("master", M_fd);
 	mgt_cli_setup(M_fd, M_fd, 0, VSB_data(vsb), Marg_closer, NULL);
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 	M_poll = 1;
 	return (1);
 }
@@ -677,7 +684,7 @@ mgt_cli_master(const char *M_arg)
 
 	error = VSS_resolver(M_arg, NULL, marg_cb, NULL, &err);
 	if (err != NULL)
-		ARGV_ERR("Could resolve -M argument to address\n\t%s\n", err);
+		ARGV_ERR("Could not resolve -M argument to address\n\t%s\n", err);
 	AZ(error);
 	if (VTAILQ_EMPTY(&m_addr_list))
 		ARGV_ERR("Could not resolve -M argument to address\n");
@@ -689,3 +696,36 @@ mgt_cli_master(const char *M_arg)
 	M_poker->name = "-M poker";
 	AZ(vev_add(mgt_evb, M_poker));
 }
+
+static int
+cli_cmp(const void *a, const void *b)
+{
+	struct cli_cmd_desc * const * const aa = a;
+	struct cli_cmd_desc * const * const bb = b;
+
+	return (strcmp((*aa)->request, (*bb)->request));
+}
+
+void
+mgt_DumpRstCli(void)
+{
+	const struct cli_cmd_desc *cp;
+	int i, j;
+
+	qsort(cmds, ncmds, sizeof cmds[0], cli_cmp);
+	for (i = 0; i < ncmds; i++, cp++) {
+		cp = cmds[i];
+		if (!strncmp(cp->request, "debug.", 6))
+			continue;
+		printf(".. _ref_cli_%s:\n\n", cp->syntax);
+		printf("%s\n", cp->syntax);
+		for (j = 0; j < strlen(cp->syntax); j++)
+			printf("~");
+		printf("\n");
+		printf("  %s\n", cp->help);
+		if (*cp->doc != '\0')
+			printf("\n%s\n", cp->doc);
+
+		printf("\n");
+	}
+}
