@@ -30,6 +30,7 @@
 #include "config.h"
 
 #include <ctype.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -39,11 +40,7 @@
 
 #include "mgt/mgt_param.h"
 #include "vav.h"
-#include "vcli.h"
-#include "vcli_common.h"
-#include "vcli_priv.h"
-
-#include "mgt_cli.h"
+#include "vcli_serve.h"
 
 struct plist {
 	unsigned			magic;
@@ -64,7 +61,7 @@ static const int tab0 = 3;
 
 static const char OBJ_STICKY_TEXT[] =
 	"\n\n"
-	"NB: This parameter is evaluated only when objects are created."
+	"NB: This parameter is evaluated only when objects are created. "
 	"To change it for all objects, restart or ban everything.";
 
 static const char DELAYED_EFFECT_TEXT[] =
@@ -84,7 +81,7 @@ static const char MUST_RELOAD_TEXT[] =
 static const char EXPERIMENTAL_TEXT[] =
 	"\n\n"
 	"NB: We do not know yet if it is a good idea to change "
-	"this parameter, or if the default value is even sensible.  "
+	"this parameter, or if the default value is even sensible. "
 	"Caution is advised, and feedback is most welcome.";
 
 static const char WIZARD_TEXT[] =
@@ -229,7 +226,7 @@ mcf_wrap(struct cli *cli, const char *text)
 
 /*--------------------------------------------------------------------*/
 
-void
+static void __match_proto__(cli_func_t)
 mcf_param_show(struct cli *cli, const char * const *av, void *priv)
 {
 	int n;
@@ -307,7 +304,7 @@ mcf_param_show(struct cli *cli, const char * const *av, void *priv)
 		VCLI_SetResult(cli, CLIS_PARAM);
 		VCLI_Out(cli, "Unknown parameter \"%s\".", av[2]);
 	}
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 }
 
 /*--------------------------------------------------------------------
@@ -380,7 +377,7 @@ MCF_ParamSet(struct cli *cli, const char *param, const char *val)
 
 /*--------------------------------------------------------------------*/
 
-void
+static void __match_proto__(cli_func_t)
 mcf_param_set(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -443,6 +440,14 @@ mcf_wash_param(struct cli *cli, const struct parspec *pp, const char **val,
 	}
 }
 
+/*--------------------------------------------------------------------*/
+
+static struct cli_proto cli_params[] = {
+	{ CLICMD_PARAM_SHOW,		"", mcf_param_show },
+	{ CLICMD_PARAM_SET,		"", mcf_param_set },
+	{ NULL }
+};
+
 /*--------------------------------------------------------------------
  * Wash the min/max/default values, and leave the default set.
  */
@@ -454,6 +459,8 @@ MCF_InitParams(struct cli *cli)
 	struct parspec *pp;
 	struct vsb *vsb;
 
+	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_params);
+
 	vsb = VSB_new_auto();
 	AN(vsb);
 	VTAILQ_FOREACH(pl, &phead, list) {
@@ -466,7 +473,7 @@ MCF_InitParams(struct cli *cli)
 		AN(pp->def);
 		mcf_wash_param(cli, pp, &pp->def, "default", vsb);
 	}
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 }
 
 /*--------------------------------------------------------------------*/
@@ -483,38 +490,35 @@ MCF_CollectParams(void)
 /*--------------------------------------------------------------------*/
 
 void
-MCF_SetDefault(const char *param, const char *new_def)
+MCF_ParamConf(enum mcf_which_e which, const char *param, const char *fmt, ...)
 {
 	struct parspec *pp;
+	struct vsb *vsb;
+	va_list ap;
 
 	pp = mcf_findpar(param);
 	AN(pp);
-	pp->def = new_def;
-	AN(pp->def);
-}
-
-void
-MCF_SetMinimum(const char *param, const char *new_min)
-{
-	struct parspec *pp;
-
-	AN(new_min);
-	pp = mcf_findpar(param);
-	AN(pp);
-	pp->min = new_min;
-	AN(pp->min);
-}
-
-void
-MCF_SetMaximum(const char *param, const char *new_max)
-{
-	struct parspec *pp;
-
-	AN(new_max);
-	pp = mcf_findpar(param);
-	AN(pp);
-	pp->max = new_max;
-	AN(pp->max);
+	vsb = VSB_new_auto();
+	AN(vsb);
+	va_start(ap, fmt);
+	VSB_vprintf(vsb, fmt, ap);
+	va_end(ap);
+	AZ(VSB_finish(vsb));
+	switch (which) {
+	case MCF_DEFAULT:
+		pp->def = strdup(VSB_data(vsb));
+		AN(pp->def);
+		break;
+	case MCF_MINIMUM:
+		pp->min = strdup(VSB_data(vsb));
+		AN(pp->min);
+		break;
+	case MCF_MAXIMUM:
+		pp->max = strdup(VSB_data(vsb));
+		AN(pp->max);
+		break;
+	}
+	VSB_delete(vsb);
 }
 
 /*--------------------------------------------------------------------*/
