@@ -56,15 +56,31 @@ struct parspec mgt_parspec[] = {
 		"and %o will be replaced with the output file name.",
 		MUST_RELOAD,
 		VCC_CC , NULL },
-	{ "vcl_dir", tweak_string, &mgt_vcl_dir,
+	{ "vcl_dir", tweak_string, &mgt_vcl_path,
 		NULL, NULL,
-		"Directory (or colon separated list of directories) "
-		"from which relative VCL filenames (vcl.load and "
-		"include) are to be found.",
+		"Old name for vcl_path, use that instead.",
 		0,
 		VARNISH_VCL_DIR,
 		NULL },
-	{ "vmod_dir", tweak_string, &mgt_vmod_dir,
+	{ "vcl_path", tweak_string, &mgt_vcl_path,
+		NULL, NULL,
+		"Directory (or colon separated list of directories) "
+		"from which relative VCL filenames (vcl.load and "
+		"include) are to be found.  By default Varnish searches "
+		"VCL files in both the system configuration and shared "
+		"data directories to allow packages to drop their VCL "
+		"files in a standard location where relative includes "
+		"would work.",
+		0,
+		VARNISH_VCL_DIR,
+		NULL },
+	{ "vmod_dir", tweak_string, &mgt_vmod_path,
+		NULL, NULL,
+		"Old name for vmod_path, use that instead.",
+		0,
+		VARNISH_VMOD_DIR,
+		NULL },
+	{ "vmod_path", tweak_string, &mgt_vmod_path,
 		NULL, NULL,
 		"Directory (or colon separated list of directories) "
 		"where VMODs are to be found.",
