@@ -329,16 +329,14 @@ tweak_vsl_buffer(struct vsb *vsb, const struct parspec *par, const char *arg)
 {
 	volatile unsigned *d1;
 	volatile ssize_t dest;
-	char buf[20];
 
 	d1 = par->priv;
 	dest = *d1;
 	if (tweak_generic_bytes(vsb, &dest, arg, par->min, par->max))
 		return (-1);
 	*d1 = dest;
-	bprintf(buf, "%u", *d1 - 12);
-	MCF_SetMaximum("vsl_reclen", strdup(buf));
-	MCF_SetMaximum("shm_reclen", strdup(buf));
+	MCF_ParamConf(MCF_MAXIMUM, "vsl_reclen", "%u", *d1 - 12);
+	MCF_ParamConf(MCF_MAXIMUM, "shm_reclen", "%u", *d1 - 12);
 	return (0);
 }
 
@@ -347,15 +345,13 @@ tweak_vsl_reclen(struct vsb *vsb, const struct parspec *par, const char *arg)
 {
 	volatile unsigned *d1;
 	volatile ssize_t dest;
-	char buf[20];
 
 	d1 = par->priv;
 	dest = *d1;
 	if (tweak_generic_bytes(vsb, &dest, arg, par->min, par->max))
 		return (-1);
 	*d1 = dest;
-	bprintf(buf, "%u", *d1 + 12);
-	MCF_SetMinimum("vsl_buffer", strdup(buf));
+	MCF_ParamConf(MCF_MINIMUM, "vsl_buffer", "%u", *d1 + 12);
 	return (0);
 }
 
