@@ -53,21 +53,15 @@
  * limit, so they don't end up crossing.
  */
 
-static char min_val[20];
-static char max_val[20];
-
 static int
 tweak_thread_pool_min(struct vsb *vsb, const struct parspec *par,
     const char *arg)
 {
-	struct vsb v2;
 
 	if (tweak_generic_uint(vsb, par->priv, arg, par->min, par->max))
 		return (-1);
-	AN(VSB_new(&v2, min_val, sizeof min_val, 0));
-	AZ(tweak_generic_uint(&v2, &mgt_param.wthread_min, NULL, NULL, NULL));
-	AZ(VSB_finish(&v2));
-	MCF_SetMinimum("thread_pool_max", min_val);
+	MCF_ParamConf(MCF_MINIMUM, "thread_pool_max",
+	    "%u", mgt_param.wthread_min);
 	return (0);
 }
 
@@ -75,14 +69,11 @@ static int
 tweak_thread_pool_max(struct vsb *vsb, const struct parspec *par,
     const char *arg)
 {
-	struct vsb v2;
 
 	if (tweak_generic_uint(vsb, par->priv, arg, par->min, par->max))
 		return (-1);
-	AN(VSB_new(&v2, max_val, sizeof max_val, 0));
-	AZ(tweak_generic_uint(&v2, &mgt_param.wthread_max, NULL, NULL, NULL));
-	AZ(VSB_finish(&v2));
-	MCF_SetMaximum("thread_pool_min", max_val);
+	MCF_ParamConf(MCF_MAXIMUM, "thread_pool_min",
+	    "%u", mgt_param.wthread_max);
 	return (0);
 }
 
@@ -93,11 +84,14 @@ struct parspec WRK_parspec[] = {
 		"1", NULL,
 		"Number of worker thread pools.\n"
 		"\n"
-		"Increasing number of worker pools decreases lock "
-		"contention.\n"
+		"Increasing the number of worker pools decreases lock "
+		"contention. Each worker pool also has a thread accepting "
+		"new connections, so for very high rates of incoming new "
+		"connections on systems with many cores, increasing the "
+		"worker pools may be required.\n"
 		"\n"
-		"Too many pools waste CPU and RAM resources, and more than "
-		"one pool for each CPU is probably detrimal to performance.\n"
+		"Too many pools waste CPU and RAM resources, and more than one "
+		"pool for each CPU is most likely detrimental to performance.\n"
 		"\n"
 		"Can be increased on the fly, but decreases require a "
 		"restart to take effect.",
@@ -149,7 +143,7 @@ struct parspec WRK_parspec[] = {
 		"Set this to a few milliseconds if you see the "
 		"'threads_failed' counter grow too much.\n"
 		"\n"
-		"Setting this too high results in insuffient worker threads.",
+		"Setting this too high results in insufficient worker threads.",
 		EXPERIMENTAL,
 		"0", "seconds" },
 	{ "thread_pool_fail_delay",
