@@ -44,7 +44,7 @@
 #include "mgt/mgt.h"
 #include "common/heritage.h"
 
-#include "flopen.h"
+#include "vfl.h"
 #include "vsm_priv.h"
 #include "vmb.h"
 #include "vfil.h"
@@ -112,7 +112,7 @@ vsm_n_check(void)
 		fprintf(stderr,
 		    "VSM (%s) not a regular file.\n", VSM_FILENAME);
 	} else {
-		i = fltest(fd, &pid);
+		i = VFL_Test(fd, &pid);
 		if (i < 0) {
 			fprintf(stderr,
 			    "Cannot determine locking status of VSM (%s)\n.",
@@ -150,7 +150,7 @@ vsm_zerofile(const char *fn, ssize_t size)
 	int fd;
 	int flags;
 
-	fd = flopen(fn, O_RDWR | O_CREAT | O_EXCL | O_NONBLOCK, 0640);
+	fd = VFL_Open(fn, O_RDWR | O_CREAT | O_EXCL | O_NONBLOCK, 0640);
 	if (fd < 0) {
 		fprintf(stderr, "Could not create %s: %s\n",
 		    fn, strerror(errno));
@@ -251,20 +251,22 @@ mgt_SHM_Create(void)
  * Commit the VSM
  */
 
-void
+int
 mgt_SHM_Commit(void)
 {
 	char fnbuf[64];
+	int retval = 0;
 
 	bprintf(fnbuf, "%s.%jd", VSM_FILENAME, (intmax_t)getpid());
 	VJ_master(JAIL_MASTER_FILE);
 	if (rename(fnbuf, VSM_FILENAME)) {
-		fprintf(stderr, "Rename failed %s -> %s: %s\n",
+		MGT_complain(C_ERR, "Rename failed %s -> %s: %s\n",
 		    fnbuf, VSM_FILENAME, strerror(errno));
 		(void)unlink(fnbuf);
-		exit(1);
+		retval = -1;
 	}
 	VJ_master(JAIL_MASTER_LOW);
+	return (retval);
 }
 
 /*--------------------------------------------------------------------
