@@ -29,6 +29,7 @@
  * Storage method based on umem_alloc(3MALLOC)
  */
 
+//lint -e{766}
 #include "config.h"
 
 #ifdef HAVE_LIBUMEM
@@ -40,7 +41,9 @@
 #include <umem.h>
 
 #include "cache/cache.h"
+#include "cache/cache_obj.h"
 #include "storage/storage.h"
+#include "storage/storage_simple.h"
 
 static size_t			smu_max = SIZE_MAX;
 static MTX			smu_mtx;
@@ -161,7 +164,9 @@ const struct stevedore smu_stevedore = {
 	.alloc		=	smu_alloc,
 	.free		=	smu_free,
 	.trim		=	smu_trim,
-	.methods	=	&default_oc_methods,
+	.allocobj	=	SML_allocobj,
+	.panic		=	SML_panic,
+	.methods	=	&SML_methods,
 };
 
 #endif /* HAVE_UMEM_H */
