@@ -54,9 +54,10 @@
 #include "vas.h"
 #include "vcs.h"
 #include "vut.h"
+#include "vtim.h"
 
-#define HIST_N 2000 /* how far back we remember */
-#define HIST_RES 100 /* bucket resolution */
+#define HIST_N 2000		/* how far back we remember */
+#define HIST_RES 100		/* bucket resolution */
 
 static const char progname[] = "varnishhist";
 
@@ -68,7 +69,7 @@ static int hist_buckets;
 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 
 static int end_of_file = 0;
-static int delay = 1;
+static double delay = 1;
 static unsigned rr_hist[HIST_N];
 static unsigned nhist;
 static unsigned next_hist;
@@ -76,7 +77,9 @@ static unsigned *bucket_miss;
 static unsigned *bucket_hit;
 static char *format;
 static int match_tag;
-
+double timebend = 0, t0;
+double vsl_t0 = 0, vsl_to, vsl_ts = 0;
+pthread_cond_t timebend_cv;
 static double log_ten;
 
 static int scales[] = {
@@ -105,37 +108,33 @@ static int scales[] = {
 
 struct profile {
 	const char *name;
+	char VSL_arg;
 	enum VSL_tag_e tag;
 	const char *prefix;
 	int field;
 	int hist_low;
 	int hist_high;
 }
+#define HIS_PROF(name,vsl_arg,tag,prefix,field,hist_low,high_high,doc)	\
+	{name,vsl_arg,tag,prefix,field,hist_low,high_high},
+#define HIS_NO_PREFIX	NULL
+#define HIS_CLIENT	'c'
+#define HIS_BACKEND	'b'
 profiles[] = {
-	{
-		.name = "responsetime",
-		.tag = SLT_Timestamp,
-		.prefix = "Process:",
-		.field = 3,
-		.hist_low = -6,
-		.hist_high = 3
-	}, {
-		.name = "size",
-		.tag = SLT_ReqAcct,
-		.prefix = NULL,
-		.field = 5,
-		.hist_low = 1,
-		.hist_high = 8
-	}, {
-		.name = 0,
-	}
+#include "varnishhist_profiles.h"
+	{ NULL }
 };
+#undef HIS_NO_PREFIX
+#undef HIS_BACKEND
+#undef HIS_CLIENT
+#undef HIS_PROF
 
 static struct profile *active_profile;
 
 static void
 update(void)
 {
+	char t[VTIM_FORMAT_SIZE];
 	unsigned w = COLS / hist_range;
 	unsigned n = w * hist_range;
 	unsigned bm[n], bh[n];
@@ -146,8 +145,6 @@ update(void)
 	erase();
 
 	/* Draw horizontal axis */
-	w = COLS / hist_range;
-	n = w * hist_range;
 	for (i = 0; i < n; ++i)
 		(void)mvaddch(LINES - 2, i, '-');
 	for (k = 0, l = hist_low; k < hist_range; ++k, ++l) {
@@ -171,13 +168,23 @@ update(void)
 			max = bm[l] + bh[l];
 	}
 
-	/* scale */
+	/* scale,time */
 	assert(LINES - 3 >= 0);
 	for (i = 0; max / scales[i] > (unsigned)(LINES - 3); ++i)
 		/* nothing */ ;
 	scale = scales[i];
 
-	mvprintw(0, 0, "1:%d, n = %d", scale, nhist);
+	if (vsl_t0 > 0) {
+		VTIM_format(vsl_ts, t);
+
+		mvprintw(0, 0, "1:%d, n = %d, d = %g @ %s x %g",
+		    scale, nhist, delay, t, timebend);
+	} else
+		mvprintw(0, 0, "1:%d, n = %d, d = %g",
+		    scale, nhist, delay);
+
+	for (j = 2; j < LINES - 3; j += 5)
+		mvprintw(j, 0, "%d_", (LINES - 3 - j) * scale);
 
 	/* show them */
 	for (i = 0; i < n; ++i) {
@@ -190,22 +197,40 @@ update(void)
 	refresh();
 }
 
+inline static void
+upd_vsl_ts(const char *p)
+{
+	double t;
+
+	if (timebend == 0)
+		return;
+
+	p = strchr(p, ' ');
+
+	if (p == NULL)
+		return;
+
+	t = strtod(p + 1, NULL);
+
+	if (t > vsl_ts)
+		vsl_ts = t;
+}
+
 static int /*__match_proto__ (VSLQ_dispatch_f)*/
 accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
-	void *priv)
+    void *priv)
 {
 	int i, tag, skip, match, hit;
 	unsigned u;
 	double value;
 	struct VSL_transaction *tr;
+	double t;
+	const char *tsp;
 
 	(void)vsl;
 	(void)priv;
 
 	for (tr = pt[0]; tr != NULL; tr = *++pt) {
-		if (tr->type != VSL_t_req)
-			/* Only look at client requests */
-			continue;
 		if (tr->reason == VSL_r_esi)
 			/* Skip ESI requests */
 			continue;
@@ -213,8 +238,21 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 		hit = 0;
 		skip = 0;
 		match = 0;
-		while (skip == 0 && 1 == VSL_Next(tr->c)) {
-			/* get the value we want, and register if it's a hit*/
+		tsp = NULL;
+		while (skip == 0) {
+			i = VSL_Next(tr->c);
+			if (i == -3) {
+				/* overrun - need to skip forward */
+				AZ(pthread_mutex_lock(&mtx));
+				vsl_to = vsl_t0 = vsl_ts = 0;
+				t0 = VTIM_mono();
+				AZ(pthread_mutex_unlock(&mtx));
+				break;
+			}
+			if (i != 1)
+				break;
+
+			/* get the value we want and register if it's a hit */
 			tag = VSL_TAG(tr->c->rec.ptr);
 
 			switch (tag) {
@@ -223,21 +261,26 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				break;
 			case SLT_VCL_return:
 				if (!strcasecmp(VSL_CDATA(tr->c->rec.ptr),
-					"restart"))
+				    "restart") ||
+				    !strcasecmp(VSL_CDATA(tr->c->rec.ptr),
+				    "retry"))
 					skip = 1;
 				break;
+			case SLT_Timestamp:
+				tsp = VSL_CDATA(tr->c->rec.ptr);
+				/* FALLTHROUGH */
 			default:
 				if (tag != match_tag)
 					break;
 
 				if (active_profile->prefix &&
 				    strncmp(VSL_CDATA(tr->c->rec.ptr),
-					active_profile->prefix,
-					strlen(active_profile->prefix)) != 0)
+				    active_profile->prefix,
+				    strlen(active_profile->prefix)) != 0)
 					break;
 
-				i = sscanf(VSL_CDATA(tr->c->rec.ptr), format,
-				    &value);
+				i = sscanf(VSL_CDATA(tr->c->rec.ptr),
+				    format, &value);
 				if (i != 1)
 					break;
 				match = 1;
@@ -257,7 +300,15 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 		i -= hist_low * HIST_RES;
 		assert(i >= 0);
 		assert(i < hist_buckets);
-		pthread_mutex_lock(&mtx);
+
+		AZ(pthread_mutex_lock(&mtx));
+
+		/*
+		 * only parse the last tsp seen in this transaction -
+		 * it should be the latest.
+		 */
+		if (tsp)
+			upd_vsl_ts(tsp);
 
 		/* phase out old data */
 		if (nhist == HIST_N) {
@@ -286,8 +337,31 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 		if (++next_hist == HIST_N) {
 			next_hist = 0;
 		}
-		pthread_mutex_unlock(&mtx);
+		AZ(pthread_mutex_unlock(&mtx));
 	}
+
+	if (vsl_ts < vsl_to)
+		return (0);
+
+	t = VTIM_mono();
+
+	AZ(pthread_mutex_lock(&mtx));
+	if (vsl_t0 == 0)
+		vsl_to = vsl_t0 = vsl_ts;
+
+	assert(t > t0);
+	vsl_to = vsl_t0 + (t - t0) * timebend;
+
+	if (vsl_ts > vsl_to) {
+		double when = VTIM_real() + vsl_ts - vsl_to;
+		struct timespec ts;
+		// Lck_CondWait
+		ts.tv_nsec = (long)(modf(when, &t) * 1e9);
+		ts.tv_sec = (long)t;
+		(void)pthread_cond_timedwait(&timebend_cv, &mtx, &ts);
+	}
+	AZ(pthread_mutex_unlock(&mtx));
+
 	return (0);
 }
 
@@ -311,9 +385,9 @@ do_curses(void *arg)
 	curs_set(0);
 	erase();
 	for (;;) {
-		pthread_mutex_lock(&mtx);
+		AZ(pthread_mutex_lock(&mtx));
 		update();
-		pthread_mutex_unlock(&mtx);
+		AZ(pthread_mutex_unlock(&mtx));
 
 		timeout(delay * 1000);
 		switch ((ch = getch())) {
@@ -324,17 +398,17 @@ do_curses(void *arg)
 			erase();
 			break;
 #endif
-		case '\014': /* Ctrl-L */
-		case '\024': /* Ctrl-T */
+		case '\014':	/* Ctrl-L */
+		case '\024':	/* Ctrl-T */
 			redrawwin(stdscr);
 			refresh();
 			break;
-		case '\032': /* Ctrl-Z */
+		case '\032':	/* Ctrl-Z */
 			endwin();
 			raise(SIGTSTP);
 			break;
-		case '\003': /* Ctrl-C */
-		case '\021': /* Ctrl-Q */
+		case '\003':	/* Ctrl-C */
+		case '\021':	/* Ctrl-Q */
 		case 'Q':
 		case 'q':
 			raise(SIGINT);
@@ -352,10 +426,36 @@ do_curses(void *arg)
 		case '9':
 			delay = 1 << (ch - '0');
 			break;
+		case '+':
+			delay /= 2;
+			if (delay < 1e-3)
+				delay = 1e-3;
+			break;
+		case '-':
+			delay *= 2;
+			break;
+		case '>':
+		case '<':
+			/* see below */
+			break;
 		default:
 			beep();
 			break;
 		}
+
+		if (ch == '<' || ch == '>') {
+			pthread_mutex_lock(&mtx);
+			vsl_to = vsl_t0 = vsl_ts;
+			t0 = VTIM_mono();
+			if (timebend == 0)
+				timebend = 1;
+			else if (ch == '<')
+				timebend /= 2;
+			else
+				timebend *= 2;
+			pthread_cond_broadcast(&timebend_cv);
+			pthread_mutex_unlock(&mtx);
+		}
 	}
 	pthread_exit(NULL);
 }
@@ -369,66 +469,101 @@ usage(int status)
 
 	fprintf(stderr, "Usage: %s <options>\n\n", progname);
 	fprintf(stderr, "Options:\n");
-	for (opt = vopt_usage; *opt != NULL; opt +=2)
+	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt += 2)
 		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
 	exit(status);
 }
 
+static void
+profile_error(const char *s)
+{
+	fprintf(stderr, "-P: '%s' is not a valid"
+	    " profile name or definition\n", s);
+	exit(1);
+}
+
 int
 main(int argc, char **argv)
 {
 	int i;
-	char *colon;
+	const char *colon, *ptag;
 	const char *profile = "responsetime";
 	pthread_t thr;
 	int fnum = -1;
 	struct profile cli_p = {0};
 	cli_p.name = 0;
 
-	VUT_Init(progname);
-	if (0)
-		(void)usage;
+	VUT_Init(progname, argc, argv, &vopt_spec);
+	AZ(pthread_cond_init(&timebend_cv, NULL));
 
-	/* only client requests */
-	assert(VUT_Arg('c', NULL));
-	while ((i = getopt(argc, argv, vopt_optstring)) != -1) {
+	while ((i = getopt(argc, argv, vopt_spec.vopt_optstring)) != -1) {
 		switch (i) {
 		case 'h':
 			/* Usage help */
 			usage(0);
+		case 'p':
+			delay = strtod(optarg, NULL);
+			if (delay <= 0)
+				VUT_Error(1, "-p: invalid '%s'", optarg);
+			break;
 		case 'P':
 			colon = strchr(optarg, ':');
-			/* no colon, take the profile as a name*/
+			/* no colon, take the profile as a name */
 			if (colon == NULL) {
 				profile = optarg;
 				break;
 			}
 			/* else it's a definition, we hope */
-			if (sscanf(colon+1, "%d:%d:%d",	&cli_p.field,
-				&cli_p.hist_low, &cli_p.hist_high) != 3) {
-				fprintf(stderr, "-P: '%s' is not a valid"
-				    " profile name or definition\n", optarg);
-				exit(1);
+			if (colon == optarg + 1 &&
+			    (*optarg == 'b' || *optarg == 'c')) {
+				cli_p.VSL_arg = *optarg;
+				ptag = colon + 1;
+				colon = strchr(colon + 1, ':');
+				if (colon == NULL)
+					profile_error(optarg);
+			} else {
+				ptag = optarg;
+				cli_p.VSL_arg = 'c';
 			}
 
-			match_tag = VSL_Name2Tag(optarg, colon - optarg);
-			if (match_tag < 0) {
-				fprintf(stderr,
-				    "-P: '%s' is not a valid tag name\n",
+			assert(colon);
+			if (sscanf(colon + 1, "%d:%d:%d", &cli_p.field,
+			    &cli_p.hist_low, &cli_p.hist_high) != 3)
+				profile_error(optarg);
+
+			match_tag = VSL_Name2Tag(ptag, colon - ptag);
+			if (match_tag < 0)
+				VUT_Error(1,
+				    "-P: '%s' is not a valid tag name",
 				    optarg);
-				exit(1);
-			}
 			cli_p.name = "custom";
 			cli_p.tag = match_tag;
 			profile = NULL;
 			active_profile = &cli_p;
 
 			break;
+		case 'B':
+			timebend = strtod(optarg, NULL);
+			if (timebend == 0)
+				VUT_Error(1,
+				    "-B: being able to bend time does not"
+				    " mean we can stop it"
+				    " (invalid factor '%s')", optarg);
+			if (timebend < 0)
+				VUT_Error(1,
+				    "-B: being able to bend time does not"
+				    " mean we can make it go backwards"
+				    " (invalid factor '%s')", optarg);
+			break;
 		default:
 			if (!VUT_Arg(i, optarg))
 				usage(1);
 		}
 	}
+
+	if (optind != argc)
+		usage(1);
+
 	/* Check for valid grouping mode */
 	assert(VUT.g_arg < VSL_g__MAX);
 	if (VUT.g_arg != VSL_g_vxid && VUT.g_arg != VSL_g_request)
@@ -438,16 +573,15 @@ main(int argc, char **argv)
 
 	if (profile) {
 		for (active_profile = profiles; active_profile->name;
-			active_profile++) {
-			if (strcmp(active_profile->name, profile) == 0) {
+		     active_profile++) {
+			if (strcmp(active_profile->name, profile) == 0)
 				break;
-			}
 		}
 	}
-	if (! active_profile->name) {
-		fprintf(stderr, "-P: No such profile '%s'\n", profile);
-		exit(1);
-	}
+	if (!active_profile->name)
+		VUT_Error(1, "-P: No such profile '%s'", profile);
+
+	assert(VUT_Arg(active_profile->VSL_arg, NULL));
 	match_tag = active_profile->tag;
 	fnum = active_profile->field;
 	hist_low = active_profile->hist_low;
@@ -458,20 +592,19 @@ main(int argc, char **argv)
 	bucket_hit = calloc(sizeof *bucket_hit, hist_buckets);
 	bucket_miss = calloc(sizeof *bucket_miss, hist_buckets);
 
+	if (timebend > 0)
+		t0 = VTIM_mono();
+
 	format = malloc(4 * fnum);
-	for (i = 0; i < fnum-1; i++) {
-		strcpy(format + 4*i, "%*s ");
-	}
-	strcpy(format + 4*(fnum-1), "%lf");
+	for (i = 0; i < fnum - 1; i++)
+		strcpy(format + 4 * i, "%*s ");
+	strcpy(format + 4 * (fnum - 1), "%lf");
 
 	log_ten = log(10.0);
 
 	VUT_Setup();
-	if (pthread_create(&thr, NULL, do_curses, NULL) != 0) {
-		fprintf(stderr, "pthread_create(): %s\n",
-				strerror(errno));
-		exit(1);
-	}
+	if (pthread_create(&thr, NULL, do_curses, NULL) != 0)
+		VUT_Error(1, "pthread_create(): %s", strerror(errno));
 	VUT.dispatch_f = &accumulate;
 	VUT.dispatch_priv = NULL;
 	VUT.sighup_f = sighup;
