@@ -68,8 +68,8 @@ usage(int status)
 	const char **opt;
 	fprintf(stderr, "Usage: %s <options>\n\n", progname);
 	fprintf(stderr, "Options:\n");
-	for (opt = vopt_usage; *opt != NULL; opt += 2)
-		fprintf(stderr, "  %-25s %s\n", *opt, *(opt + 1));
+	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt += 2)
+		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
 	exit(status);
 }
 
@@ -121,10 +121,10 @@ main(int argc, char * const *argv)
 {
 	int opt;
 
+	VUT_Init(progname, argc, argv, &vopt_spec);
 	memset(&LOG, 0, sizeof LOG);
-	VUT_Init(progname);
 
-	while ((opt = getopt(argc, argv, vopt_optstring)) != -1) {
+	while ((opt = getopt(argc, argv, vopt_spec.vopt_optstring)) != -1) {
 		switch (opt) {
 		case 'a':
 			/* Append to file */
