@@ -41,11 +41,16 @@
 #include <math.h>
 #include <stdint.h>
 
+#include "vapi/voptget.h"
+#include "vapi/vsl.h"
+#include "vdef.h"
 #include "vnum.h"
 #include "vtim.h"
+#include "vut.h"
 
 #include "varnishstat.h"
 
+static const char progname[] = "varnishstat";
 
 /*--------------------------------------------------------------------*/
 
@@ -152,7 +157,6 @@ do_json(struct VSM_data *vd)
 	printf("  \"timestamp\": \"%s\",\n", time_stamp);
 	(void)VSC_Iter(vd, NULL, do_json_cb, &jp);
 	printf("\n}\n");
-	fflush(stdout);
 }
 
 
@@ -224,22 +228,22 @@ do_list_cb(void *priv, const struct VSC_point * const pt)
 	sec = pt->section;
 	i = 0;
 	if (strcmp(sec->fantom->type, ""))
-		i += fprintf(stderr, "%s.", sec->fantom->type);
+		i += printf("%s.", sec->fantom->type);
 	if (strcmp(sec->fantom->ident, ""))
-		i += fprintf(stderr, "%s.", sec->fantom->ident);
-	i += fprintf(stderr, "%s", pt->desc->name);
+		i += printf("%s.", sec->fantom->ident);
+	i += printf("%s", pt->desc->name);
 	if (i < 30)
-		fprintf(stderr, "%*s", i - 30, "");
-	fprintf(stderr, " %s\n", pt->desc->sdesc);
+		printf("%*s", i - 30, "");
+	printf(" %s\n", pt->desc->sdesc);
 	return (0);
 }
 
 static void
 list_fields(struct VSM_data *vd)
 {
-	fprintf(stderr, "Varnishstat -f option fields:\n");
-	fprintf(stderr, "Field name                     Description\n");
-	fprintf(stderr, "----------                     -----------\n");
+	printf("Varnishstat -f option fields:\n");
+	printf("Field name                     Description\n");
+	printf("----------                     -----------\n");
 
 	(void)VSC_Iter(vd, NULL, do_list_cb, NULL);
 }
@@ -247,50 +251,38 @@ list_fields(struct VSM_data *vd)
 /*--------------------------------------------------------------------*/
 
 static void
-usage(void)
+usage(int status)
 {
-#define FMT "    %-28s # %s\n"
-	fprintf(stderr, "usage: varnishstat "
-	    "[-1lV] [-f field] [-t seconds|<off>] "
-	    VSC_n_USAGE "\n");
-	fprintf(stderr, FMT, "-1", "Print the statistics to stdout.");
-	fprintf(stderr, FMT, "-f field", "Field inclusion glob");
-	fprintf(stderr, FMT, "",
-	    "If it starts with '^' it is used as an exclusion list.");
-	fprintf(stderr, FMT, "-l",
-	    "Lists the available fields to use with the -f option.");
-	fprintf(stderr, FMT, "-n varnish_name",
-	    "The varnishd instance to get logs from.");
-	fprintf(stderr, FMT, "-N filename",
-	    "Filename of a stale VSM instance.");
-	fprintf(stderr, FMT, "-t seconds|<off>",
-	    "Timeout before returning error on initial VSM connection.");
-	fprintf(stderr, FMT, "-V", "Display the version number and exit.");
-	fprintf(stderr, FMT, "-x",
-	    "Print statistics to stdout as XML.");
-	fprintf(stderr, FMT, "-j",
-	    "Print statistics to stdout as JSON.");
-#undef FMT
-	exit(1);
+	const char **opt;
+
+	fprintf(stderr, "Usage: %s <options>\n\n", progname);
+	fprintf(stderr, "Options:\n");
+	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt +=2)
+		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
+	exit(status);
 }
 
 int
 main(int argc, char * const *argv)
 {
-	int c;
 	struct VSM_data *vd;
 	double t_arg = 5.0, t_start = NAN;
 	int once = 0, xml = 0, json = 0, f_list = 0, curses = 0;
+	signed char opt;
 	int i;
 
+	VUT_Init(progname, argc, argv, &vopt_spec);
 	vd = VSM_New();
 	AN(vd);
 
-	while ((c = getopt(argc, argv, VSC_ARGS "1f:lVxjt:")) != -1) {
-		switch (c) {
+	while ((opt = getopt(argc, argv, vopt_spec.vopt_optstring)) != -1) {
+		switch (opt) {
 		case '1':
 			once = 1;
 			break;
+		case 'h':
+			/* Usage help */
+			usage(0);
 		case 'l':
 			f_list = 1;
 			break;
@@ -299,14 +291,10 @@ main(int argc, char * const *argv)
 				t_arg = -1.;
 			else {
 				t_arg = VNUM(optarg);
-				if (isnan(t_arg)) {
-					fprintf(stderr, "-t: Syntax error");
-					exit(1);
-				}
-				if (t_arg < 0.) {
-					fprintf(stderr, "-t: Range error");
-					exit(1);
-				}
+				if (isnan(t_arg))
+					VUT_Error(1, "-t: Syntax error");
+				if (t_arg < 0.)
+					VUT_Error(1, "-t: Range error");
 			}
 			break;
 		case 'V':
@@ -319,13 +307,17 @@ main(int argc, char * const *argv)
 			json = 1;
 			break;
 		default:
-			if (VSC_Arg(vd, c, optarg) > 0)
-				break;
-			fprintf(stderr, "%s\n", VSM_Error(vd));
-			usage();
+			i = VSC_Arg(vd, opt, optarg);
+			if (i < 0)
+				VUT_Error(1, "%s", VSM_Error(vd));
+			if (!i)
+				usage(1);
 		}
 	}
 
+	if (optind != argc)
+		usage(1);
+
 	if (!(xml || json || once || f_list))
 		curses = 1;
 
@@ -334,8 +326,8 @@ main(int argc, char * const *argv)
 		if (!i)
 			break;
 		if (isnan(t_start) && t_arg > 0.) {
-			fprintf(stderr, "Can't open log -"
-			    " retrying for %.0f seconds\n", t_arg);
+			VUT_Error(0, "Can't open log -"
+			    " retrying for %.0f seconds", t_arg);
 			t_start = VTIM_real();
 		}
 		if (t_arg <= 0.)
@@ -347,18 +339,14 @@ main(int argc, char * const *argv)
 	}
 
 	if (curses) {
-		if (i && t_arg >= 0.) {
-			fprintf(stderr, "%s\n", VSM_Error(vd));
-			exit(1);
-		}
+		if (i && t_arg >= 0.)
+			VUT_Error(1, "%s", VSM_Error(vd));
 		do_curses(vd, 1.0);
 		exit(0);
 	}
 
-	if (i) {
-		fprintf(stderr, "%s\n", VSM_Error(vd));
-		exit(1);
-	}
+	if (i)
+		VUT_Error(1, "%s", VSM_Error(vd));
 
 	if (xml)
 		do_xml(vd);
@@ -371,8 +359,5 @@ main(int argc, char * const *argv)
 	else
 		assert(0);
 
-	/* end of output block marker. */
-	printf("\n");
-
 	exit(0);
 }
