@@ -71,11 +71,62 @@ struct macro {
 
 static VTAILQ_HEAD(,macro) macro_list = VTAILQ_HEAD_INITIALIZER(macro_list);
 
+/**********************************************************************/
+
+static struct macro *
+macro_def_int(const char *name, const char *fmt, va_list ap)
+{
+	struct macro *m;
+	char buf[256];
+
+	VTAILQ_FOREACH(m, &macro_list, list)
+		if (!strcmp(name, m->name))
+			break;
+	if (m == NULL) {
+		ALLOC_OBJ(m, MACRO_MAGIC);
+		AN(m);
+		REPLACE(m->name, name);
+		AN(m->name);
+		VTAILQ_INSERT_TAIL(&macro_list, m, list);
+	}
+	AN(m);
+	vbprintf(buf, fmt, ap);
+	REPLACE(m->val, buf);
+	AN(m->val);
+	return (m);
+}
+
+
+/**********************************************************************
+ * This is for defining macros before we fork the child process which
+ * runs the test-case.
+ */
+
+void
+extmacro_def(const char *name, const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	(void)macro_def_int(name, fmt, ap);
+	va_end(ap);
+}
+
+/**********************************************************************
+ * Below this point is run inside the testing child-process.
+ */
+
 static pthread_mutex_t		macro_mtx;
 
 static void
 init_macro(void)
 {
+	struct macro *m;
+
+	/* Dump the extmacros for completeness */
+	VTAILQ_FOREACH(m, &macro_list, list)
+		vtc_log(vltop, 4, "extmacro def %s=%s", m->name, m->val);
+
 	AZ(pthread_mutex_init(&macro_mtx, NULL));
 }
 
@@ -84,7 +135,6 @@ macro_def(struct vtclog *vl, const char *instance, const char *name,
     const char *fmt, ...)
 {
 	char buf1[256];
-	char buf2[256];
 	struct macro *m;
 	va_list ap;
 
@@ -96,21 +146,9 @@ macro_def(struct vtclog *vl, const char *instance, const char *name,
 	}
 
 	AZ(pthread_mutex_lock(&macro_mtx));
-	VTAILQ_FOREACH(m, &macro_list, list)
-		if (!strcmp(name, m->name))
-			break;
-	if (m == NULL) {
-		ALLOC_OBJ(m, MACRO_MAGIC);
-		AN(m);
-		REPLACE(m->name, name);
-		VTAILQ_INSERT_TAIL(&macro_list, m, list);
-	}
-	AN(m);
 	va_start(ap, fmt);
-	vbprintf(buf2, fmt, ap);
+	m = macro_def_int(name, fmt, ap);
 	va_end(ap);
-	REPLACE(m->val, buf2);
-	AN(m->val);
 	vtc_log(vl, 4, "macro def %s=%s", name, m->val);
 	AZ(pthread_mutex_unlock(&macro_mtx));
 }
@@ -147,7 +185,6 @@ macro_get(const char *b, const char *e)
 	int l;
 	char *retval = NULL;
 
-
 	l = e - b;
 
 	if (l == 4 && !memcmp(b, "date", l)) {
@@ -163,7 +200,7 @@ macro_get(const char *b, const char *e)
 		CHECK_OBJ_NOTNULL(m, MACRO_MAGIC);
 		if (!strncmp(b, m->name, l) && m->name[l] == '\0')
 			break;
-}
+	}
 	if (m != NULL)
 		retval = strdup(m->val);
 	AZ(pthread_mutex_unlock(&macro_mtx));
@@ -197,10 +234,10 @@ macro_expand(struct vtclog *vl, const char *text)
 		p += 2;
 		m = macro_get(p, q);
 		if (m == NULL) {
-			VSB_delete(vsb);
+			VSB_destroy(&vsb);
 			vtc_log(vl, 0, "Macro ${%.*s} not found", (int)(q - p),
 			    p);
-			return (NULL);
+			NEEDLESS_RETURN (NULL);
 		}
 		VSB_printf(vsb, "%s", m);
 		free(m);
@@ -210,56 +247,6 @@ macro_expand(struct vtclog *vl, const char *text)
 	return (vsb);
 }
 
-/*
- * extmacro is a list of macro's that are defined from the
- * command line and are applied to the macro list of each test
- * instance. No locking is required as they are set before any tests
- *  are started.
- */
-
-struct extmacro {
-	unsigned		magic;
-#define EXTMACRO_MAGIC		0x51019ded
-	VTAILQ_ENTRY(extmacro)	list;
-	char			*name;
-	char			*val;
-};
-
-static VTAILQ_HEAD(, extmacro) extmacro_list =
-	VTAILQ_HEAD_INITIALIZER(extmacro_list);
-
-void
-extmacro_def(const char *name, const char *fmt, ...)
-{
-	char buf[256];
-	struct extmacro *m;
-	va_list ap;
-
-	VTAILQ_FOREACH(m, &extmacro_list, list)
-		if (!strcmp(name, m->name))
-			break;
-	if (m == NULL && fmt != NULL) {
-		ALLOC_OBJ(m, EXTMACRO_MAGIC);
-		AN(m);
-		REPLACE(m->name, name);
-		AN(m->name);
-		VTAILQ_INSERT_TAIL(&extmacro_list, m, list);
-	}
-	if (fmt != NULL) {
-		AN(m);
-		va_start(ap, fmt);
-		vbprintf(buf, fmt, ap);
-		va_end(ap);
-		REPLACE(m->val, buf);
-		AN(m->val);
-	} else if (m != NULL) {
-		VTAILQ_REMOVE(&extmacro_list, m, list);
-		free(m->name);
-		free(m->val);
-		FREE_OBJ(m);
-	}
-}
-
 /**********************************************************************
  * Parse a string
  *
@@ -275,7 +262,7 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 {
 	char *token_s[MAX_TOKENS], *token_e[MAX_TOKENS];
 	struct vsb *token_exp[MAX_TOKENS];
-	char *p, *q, *f, *buf;
+	char *e, *p, *q, *f, *buf;
 	int nest_brace;
 	int tn;
 	const struct cmds *cp;
@@ -283,7 +270,9 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 	AN(spec);
 	buf = strdup(spec);
 	AN(buf);
-	for (p = buf; *p != '\0'; p++) {
+	e = strchr(buf, '\0');
+	AN(e);
+	for (p = buf; p < e; p++) {
 		if (vtc_error || vtc_stop)
 			break;
 		/* Start of line */
@@ -311,8 +300,9 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 		/* First content on line, collect tokens */
 		tn = 0;
 		f = p;
-		while (*p != '\0') {
+		while (p < e) {
 			assert(tn < MAX_TOKENS);
+			assert(p < e);
 			if (*p == '\n') { /* End on NL */
 				break;
 			}
@@ -328,6 +318,7 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 				token_s[tn] = ++p;
 				q = p;
 				for (; *p != '\0'; p++) {
+					assert(p < e);
 					if (*p == '"')
 						break;
 					if (*p == '\\') {
@@ -347,7 +338,7 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 			} else if (*p == '{') { /* Braces */
 				nest_brace = 0;
 				token_s[tn] = p + 1;
-				for (; *p != '\0'; p++) {
+				for (; p < e; p++) {
 					if (*p == '{')
 						nest_brace++;
 					else if (*p == '}') {
@@ -359,25 +350,26 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 				token_e[tn++] = p++;
 			} else { /* other tokens */
 				token_s[tn] = p;
-				for (; *p != '\0' && !isspace(*p); p++)
-					;
+				for (; p < e && !isspace(*p); p++)
+					continue;
 				token_e[tn++] = p;
 			}
 		}
+
+		assert(p <= e);
 		assert(tn < MAX_TOKENS);
 		token_s[tn] = NULL;
 		for (tn = 0; token_s[tn] != NULL; tn++) {
 			token_exp[tn] = NULL;
 			AN(token_e[tn]);	/*lint !e771 */
 			*token_e[tn] = '\0';	/*lint !e771 */
-			if (NULL == strstr(token_s[tn], "${"))
-				continue;
-			token_exp[tn] = macro_expand(vl, token_s[tn]);
-			if (vtc_error) {
-				return;
+			if (NULL != strstr(token_s[tn], "${")) {
+				token_exp[tn] = macro_expand(vl, token_s[tn]);
+				if (vtc_error)
+					return;
+				token_s[tn] = VSB_data(token_exp[tn]);
+				token_e[tn] = strchr(token_s[tn], '\0');
 			}
-			token_s[tn] = VSB_data(token_exp[tn]);
-			token_e[tn] = strchr(token_s[tn], '\0');
 		}
 
 		for (cp = cmd; cp->name != NULL; cp++)
@@ -385,7 +377,7 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 				break;
 		if (cp->name == NULL) {
 			vtc_log(vl, 0, "Unknown command: \"%s\"", token_s[0]);
-			return;
+			NEEDLESS_RETURN;
 		}
 
 		assert(cp->cmd != NULL);
@@ -405,8 +397,15 @@ reset_cmds(const struct cmds *cmd)
 		cmd->cmd(NULL, NULL, NULL, NULL);
 }
 
-/**********************************************************************
- * Output test description
+/* SECTION: varnishtest varnishtest
+ *
+ * This should be the first command in your vtc as it will identify the test
+ * case with a short yet descriptive sentence. It takes exactly one argument, a
+ * string, eg::
+ *
+ *         varnishtest "Check that varnishtest is actually a valid command"
+ *
+ * It will also print that string in the log.
  */
 
 static void
@@ -425,8 +424,18 @@ cmd_varnishtest(CMD_ARGS)
 	AZ(av[2]);
 }
 
-/**********************************************************************
- * Shell command execution
+/* SECTION: shell shell
+ *
+ * Pass the string given as argument to a shell. If you have multiple commands
+ * to run, you can use curly barces to describe a multi-lines script, eg::
+ *
+ *         shell {
+ *                 echo begin
+ *                 cat /etc/fstab
+ *                 echo end
+ *         }
+ *
+ * The vtc will fail if the return code of the shell is not 0.
  */
 
 static void
@@ -448,8 +457,15 @@ cmd_shell(CMD_ARGS)
 		    av[1], s, strerror(errno));
 }
 
-/**********************************************************************
- * Shell command execution
+/* SECTION: err_shell err_shell
+ *
+ * This is very similar to the the ``shell`` command, except it takes a first
+ * string as argument before the command::
+ *
+ *         err_shell "foo" "echo foo"
+ *
+ * err_shell expect the shell command to fail AND stdout to match the string,
+ * failing the test case otherwise.
  */
 
 static void
@@ -493,13 +509,17 @@ cmd_err_shell(CMD_ARGS)
 	else
 		vtc_log(vl, 4,
 		    "Found expected string: (\"%s\")", av[1]);
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 }
 
-/**********************************************************************
- * Dump command arguments
+/* SECTION: client-server.spec.delay delay
+ *
+ * Take a float as argument and sleep for that number of seconds.
+ */
+/* SECTION: stream.spec.delay delay
+ *
+ * Take a float as argument and sleep for that number of seconds.
  */
-
 void
 cmd_delay(CMD_ARGS)
 {
@@ -516,105 +536,104 @@ cmd_delay(CMD_ARGS)
 	VTIM_sleep(f);
 }
 
-/**********************************************************************
- * Check random generator
- */
-
-#define NRNDEXPECT	12
-static const unsigned long random_expect[NRNDEXPECT] = {
-	1804289383,	846930886,	1681692777,	1714636915,
-	1957747793,	424238335,	719885386,	1649760492,
-	 596516649,	1189641421,	1025202362,	1350490027
-};
-
-#define RND_NEXT_1K	0x3bdcbe30
-
-static void
-cmd_random(CMD_ARGS)
-{
-	uint32_t l;
-	int i;
-
-	(void)cmd;
-	(void)priv;
-	if (av == NULL)
-		return;
-	srandom(1);
-	for (i = 0; i < NRNDEXPECT; i++) {
-		l = random();
-		if (l == random_expect[i])
-			continue;
-		vtc_log(vl, 4, "random[%d] = 0x%x (expect 0x%lx)",
-		    i, l, random_expect[i]);
-		vtc_log(vl, 1, "SKIPPING test: unknown srandom(1) sequence.");
-		vtc_stop = 1;
-		break;
-	}
-	l = 0;
-	for (i = 0; i < 1000; i++)
-		l += random();
-	if (l != RND_NEXT_1K) {
-		vtc_log(vl, 4, "sum(random[%d...%d]) = 0x%x (expect 0x%x)",
-		    NRNDEXPECT, NRNDEXPECT + 1000,
-		    l, RND_NEXT_1K);
-		vtc_log(vl, 1, "SKIPPING test: unknown srandom(1) sequence.");
-		vtc_stop = 1;
-	}
-}
-
-/**********************************************************************
- * Check features.
+/* SECTION: feature feature
+ *
+ * Test that the required feature(s) for a test are available, and skip the test
+ * otherwise. feature takes any number of arguments from this list:
+ *
+ * SO_RCVTIMEO_WORKS
+ *        The SO_RCVTIMEO socket option is working
+ * 64bit
+ *        The environment is 64 bits
+ * !OSX
+ *        The environment is not OSX
+ * dns
+ *        DNS lookups are working
+ * topbuild
+ *        varnishtest has been started with '-i'
+ * root
+ *        varnishtest has been invoked by the root user
+ * user_varnish
+ *        The varnish user is present
+ * user_vcache
+ *        The vcache user is present
+ * group_varnish
+ *        The varnish group is present
+ * cmd <command-line>
+ *        A command line that should execute with a zero exit status
  */
 
 static void
 cmd_feature(CMD_ARGS)
 {
-	int i;
+	int r;
+	int good;
 
 	(void)priv;
 	(void)cmd;
+
 	if (av == NULL)
 		return;
 
-	for (i = 1; av[i] != NULL; i++) {
+#define FEATURE(nm, tst)				\
+	do {						\
+		if (!strcmp(*av, nm)) {			\
+			if (tst) {			\
+				good = 1;		\
+			} else {			\
+				vtc_stop = 1;		\
+			}				\
+		}					\
+	} while (0)
+
+	for (av++; *av != NULL; av++) {
+		good = 0;
+		if (!strcmp(*av, "SO_RCVTIMEO_WORKS")) {
 #ifdef SO_RCVTIMEO_WORKS
-		if (!strcmp(av[i], "SO_RCVTIMEO_WORKS"))
-			continue;
-#endif
-		if (sizeof(void*) == 8 && !strcmp(av[i], "64bit"))
-			continue;
-
-		if (!strcmp(av[i], "pcre_jit") && VRE_has_jit)
-			continue;
-
-		if (!strcmp(av[i], "!OSX")) {
-#if !defined(__APPLE__) || !defined(__MACH__)
-			continue;
+			good = 1;
+#else
+			vtc_stop = 1;
 #endif
 		}
-		if (!strcmp(av[i], "dns") && feature_dns)
+
+		if (!strcmp(*av, "!OSX")) {
+#if !defined(__APPLE__) || !defined(__MACH__)
+			good = 1;
+#else
+			vtc_stop = 1;
+#endif
+		}
+		FEATURE("pcre_jit", VRE_has_jit);
+		FEATURE("64bit", sizeof(void*) == 8);
+		FEATURE("dns", feature_dns);
+		FEATURE("topbuild", iflg);
+		FEATURE("root", !geteuid());
+		FEATURE("user_varnish", getpwnam("varnish") != NULL);
+		FEATURE("user_vcache", getpwnam("vcache") != NULL);
+		FEATURE("group_varnish", getgrnam("varnish") != NULL);
+
+		if (!strcmp(*av, "cmd")) {
+			av++;
+			if (*av == NULL) {
+				vtc_log(vl, 0, "Missing the command-line");
+				return;
+			}
+			r = system(*av);
+			if (WEXITSTATUS(r) == 0)
+				good = 1;
+			else
+				vtc_stop = 1;
+		}
+		if (good)
 			continue;
 
-		if (!strcmp(av[i], "topbuild") && iflg)
-			continue;
-
-		if (!strcmp(av[i], "root") && !geteuid())
-			continue;
-
-		if (!strcmp(av[i], "user_varnish") &&
-		    getpwnam("varnish") != NULL)
-			continue;
-
-		if (!strcmp(av[i], "user_vcache") &&
-		    getpwnam("vcache") != NULL)
-			continue;
-
-		if (!strcmp(av[i], "group_varnish") &&
-		    getgrnam("varnish") != NULL)
-			continue;
-
-		vtc_log(vl, 1, "SKIPPING test, missing feature: %s", av[i]);
-		vtc_stop = 1;
+		if (!vtc_stop) {
+			vtc_log(vl, 0,
+			    "FAIL test, unknown feature: %s", *av);
+		} else {
+			vtc_log(vl, 1,
+			    "SKIPPING test, lacking feature: %s", *av);
+		}
 		return;
 	}
 }
@@ -624,19 +643,19 @@ cmd_feature(CMD_ARGS)
  */
 
 static const struct cmds cmds[] = {
-	{ "server",	cmd_server },
-	{ "client",	cmd_client },
-	{ "varnish",	cmd_varnish },
-	{ "delay",	cmd_delay },
-	{ "varnishtest",cmd_varnishtest },
-	{ "shell",	cmd_shell },
-	{ "err_shell",	cmd_err_shell },
-	{ "sema",	cmd_sema },
-	{ "barrier",	cmd_barrier },
-	{ "random",	cmd_random },
-	{ "feature",	cmd_feature },
-	{ "logexpect",	cmd_logexp },
-	{ "process",	cmd_process },
+#define CMD(n) { #n, cmd_##n }
+	CMD(server),
+	CMD(client),
+	CMD(varnish),
+	CMD(delay),
+	CMD(varnishtest),
+	CMD(shell),
+	CMD(err_shell),
+	CMD(barrier),
+	CMD(feature),
+	CMD(logexpect),
+	CMD(process),
+#undef CMD
 	{ NULL,		NULL }
 };
 
@@ -646,7 +665,6 @@ exec_file(const char *fn, const char *script, const char *tmpdir,
 {
 	unsigned old_err;
 	FILE *f;
-	struct extmacro *m;
 
 	(void)signal(SIGPIPE, SIG_IGN);
 
@@ -655,25 +673,9 @@ exec_file(const char *fn, const char *script, const char *tmpdir,
 	AN(vltop);
 
 	init_macro();
-	init_sema();
 	init_barrier();
 	init_server();
 
-	/* Apply extmacro definitions */
-	VTAILQ_FOREACH(m, &extmacro_list, list) {
-		CHECK_OBJ_NOTNULL(m, EXTMACRO_MAGIC);
-		macro_def(vltop, NULL, m->name, "%s", m->val);
-	}
-
-	/*
-	 * We need an IP number which will not repond, ever, and that is a
-	 * lot harder than it sounds.  This IP# is from RFC5737 and a
-	 * C-class broadcast at that.
-	 * If tests involving ${bad_ip} fails and you run linux, you should
-	 * check your /proc/sys/net/ipv4/ip_nonlocal_bind setting.
-	 */
-	macro_def(vltop, NULL, "bad_ip", "192.0.2.255");
-
 	/* Move into our tmpdir */
 	AZ(chdir(tmpdir));
 	macro_def(vltop, NULL, "tmpdir", "%s", tmpdir);
