@@ -43,6 +43,7 @@
 
 struct vtclog;
 struct cmds;
+struct suckaddr;
 
 #define CMD_ARGS \
     char * const *av, void *priv, const struct cmds *cmd, struct vtclog *vl
@@ -57,14 +58,15 @@ struct cmds {
 void parse_string(const char *spec, const struct cmds *cmd, void *priv,
     struct vtclog *vl);
 
-cmd_f cmd_delay;
-cmd_f cmd_server;
-cmd_f cmd_client;
-cmd_f cmd_varnish;
-cmd_f cmd_sema;
-cmd_f cmd_barrier;
-cmd_f cmd_logexp;
-cmd_f cmd_process;
+#define CMD(n) cmd_f cmd_##n
+CMD(delay);
+CMD(server);
+CMD(client);
+CMD(varnish);
+CMD(barrier);
+CMD(logexpect);
+CMD(process);
+#undef CMD
 
 extern volatile sig_atomic_t vtc_error; /* Error, bail out */
 extern int vtc_stop;		/* Abandon current test, no error */
@@ -74,11 +76,14 @@ extern unsigned vtc_maxdur;
 extern int vtc_witness;
 extern int feature_dns;
 
-void init_sema(void);
 void init_barrier(void);
 void init_server(void);
 
 int http_process(struct vtclog *vl, const char *spec, int sock, int *sfd);
+int http2_process(struct vtclog *vl, const char *spec, int sock, int *sfd,
+		unsigned nosettings);
+
+char * synth_body(const char *len, int rnd);
 
 void cmd_server_genvcl(struct vsb *vsb);
 
@@ -92,6 +97,9 @@ void vtc_dump(struct vtclog *vl, int lvl, const char *pfx,
 void vtc_hexdump(struct vtclog *vl, int lvl, const char *pfx,
     const unsigned char *str, int len);
 
+int vtc_send_proxy(int fd, int version, const struct suckaddr *sac,
+    const struct suckaddr *sas);
+
 int exec_file(const char *fn, const char *script, const char *tmpdir,
     char *logbuf, unsigned loglen);
 
@@ -103,3 +111,9 @@ struct vsb *macro_expand(struct vtclog *vl, const char *text);
 
 void extmacro_def(const char *name, const char *fmt, ...)
     __v_printflike(2, 3);
+
+struct http;
+void cmd_stream(CMD_ARGS);
+void start_h2(struct http *hp);
+void stop_h2(struct http *hp);
+void b64_settings(const struct http *hp, const char *s);
