@@ -40,46 +40,108 @@
 #include <string.h>
 
 #include "vtc.h"
+#include "vtc_http.h"
 
 #include "vct.h"
 #include "vgz.h"
 #include "vnum.h"
 #include "vre.h"
 #include "vtcp.h"
+#include "hpack.h"
 
-#define MAX_HDR		50
+static const struct cmds http_cmds[];
 
-struct http {
-	unsigned		magic;
-#define HTTP_MAGIC		0x2f02169c
-	int			fd;
-	int			*sfd;
-	int			timeout;
-	struct vtclog		*vl;
-
-	struct vsb		*vsb;
-
-	int			nrxbuf;
-	char			*rxbuf;
-	char			*rem_ip;
-	char			*rem_port;
-	int			prxbuf;
-	char			*body;
-	unsigned		bodyl;
-	char			bodylen[20];
-	char			chunklen[20];
-
-	char			*req[MAX_HDR];
-	char			*resp[MAX_HDR];
-
-	int			gziplevel;
-	int			gzipresidual;
-
-	int			fatal;
-};
+/* SECTION: client-server client/server
+ *
+ * Client and server threads are fake HTTP entities used to test your Varnish
+ * and VCL. They take any number of arguments, and the one that are not
+ * recognized, assuming they don't start with '-', are treated as
+ * specifications, laying out the actions to undertake::
+ *
+ *         client cNAME [...]
+ *         server sNAME [...]
+ *
+ * Clients and server are identified by a string that's the first argument,
+ * clients' names start with 'c' and servers' names start with 's'.
+ *
+ * As the client and server commands share a good deal of arguments and
+ * specification actions, they are grouped in this single section, specific
+ * items will be explicitly marked as such.
+ *
+ * SECTION: client-server.macros Macros and automatic behaviour
+ *
+ * To make things easier in the general case, clients will connect by default
+ * to the first Varnish server declared and the -vcl+backend switch of the
+ * ``varnish`` command will add all the declared servers as backends.
+ *
+ * Be careful though, servers will by default listen to the 127.0.0.1 IP and
+ * will pick a random port, and publish 3 macros: sNAME_addr, sNAME_port and
+ * sNAME_sock, but only once they are started. For varnishtest to
+ * create the vcl with the correct values, the server must be started when you
+ * use -vcl+backend.
+ *
+ * SECTION: client-server.args Arguments
+ *
+ * \-start
+ *        Start the thread in background, processing the last given
+ *        specification.
+ *
+ * \-wait
+ *        Block until the thread finishes.
+ *
+ * \-run (client only)
+ *        Equivalent to "-start -wait".
+ *
+ * \repeat NUMBER
+ *        Instead of processing the specification only once, do it NUMBER times.
+ *
+ * \-break (server only)
+ *        Stop the server.
+ *
+ * \-listen STRING (server only)
+ *        Dictate the listening socket for the server. STRING is of the form
+ *        "IP PORT".
+ *
+ * \-connect STRING (client only)
+ *        Indicate the server to connect to. STRING is also of the form
+ *        "IP PORT".
+ *
+ * \-dispatch (server only, s0 only)
+ *        Normally, to keep things simple, server threads only handle one
+ *        connection at a time, but the -dispatch switch allows to accept
+ *        any number of connection and handle them following the given spec.
+ *
+ *        However, -dispatch is only allowed for the server name "s0".
+ *
+ * \-proxy1 STRING (client only)
+ *        Use the PROXY protocol version 1 for this connection. STRING
+ *        is of the form "CLIENTIP:PORT SERVERIP:PORT".
+ *
+ * \-proxy2 STRING (client only)
+ *        Use the PROXY protocol version 2 for this connection. STRING
+ *        is of the form "CLIENTIP:PORT SERVERIP:PORT".
+ *
+ * SECTION: client-server.spec Specification
+ *
+ * It's a string, either double-quoted "like this", but most of the time
+ * enclosed in curly brackets, allowing multilining. Write a command per line in
+ * it, empty line are ignored, and long line can be wrapped by using a
+ * backslash. For example::
+ *
+ *     client c1 {
+ *         txreq -url /foo \
+ *               -hdr "bar: baz"
+ *
+ *         rxresp
+ *     } -run
+ */
 
 #define ONLY_CLIENT(hp, av)						\
 	do {								\
+		if (hp->h2)						\
+			vtc_log(hp->vl, 0,				\
+			    "\"%s\" only possible before H/2 upgrade",	\
+					av[0]);				\
 		if (hp->sfd != NULL)					\
 			vtc_log(hp->vl, 0,				\
 			    "\"%s\" only possible in client", av[0]);	\
@@ -87,6 +149,10 @@ struct http {
 
 #define ONLY_SERVER(hp, av)						\
 	do {								\
+		if (hp->h2)						\
+			vtc_log(hp->vl, 0,				\
+			    "\"%s\" only possible before H/2 upgrade",	\
+					av[0]);				\
 		if (hp->sfd == NULL)					\
 			vtc_log(hp->vl, 0,				\
 			    "\"%s\" only possible in server", av[0]);	\
@@ -100,7 +166,7 @@ static const char * const nl = "\r\n";
  * Generate a synthetic body
  */
 
-static char *
+char *
 synth_body(const char *len, int rnd)
 {
 	int i, j, k, l;
@@ -191,8 +257,40 @@ http_count_header(char * const *hh, const char *hdr)
 	return (r);
 }
 
-/**********************************************************************
- * Expect
+/* SECTION: client-server.spec.expect
+ *
+ * expect STRING1 OP STRING2
+ *         Test if "STRING1 OP STRING2" is true, and if not, fails the test.
+ *         OP can be ==, <, <=, >, >= when STRING1 and STRING2 represent numbers
+ *         in which case it's an order operator. If STRING1 and STRING2 are
+ *         meant as strings OP is a matching operator, either == (exact match)
+ *         or ~ (regex match).
+ *
+ *         varnishtet will first try to resolve STRING1 and STRING2 by looking
+ *         if they have special meanings, in which case, the resolved value is
+ *         use for the test. Note that this value can be a string representing a
+ *         number, allowing for tests such as::
+ *
+ *                 expect req.http.x-num > 2
+ *
+ *         Here's the list of recognized strings, most should be obvious as they
+ *         either match VCL logic, or the txreq/txresp options:
+ *
+ *         - remote.ip
+ *         - remote.port
+ *         - req.method
+ *         - req.url
+ *         - req.proto
+ *         - resp.proto
+ *         - resp.status
+ *         - resp.reason
+ *         - resp.chunklen
+ *         - req.bodylen
+ *         - req.body
+ *         - resp.bodylen
+ *         - resp.body
+ *         - req.http.NAME
+ *         - resp.http.NAME
  */
 
 static const char *
@@ -213,12 +311,14 @@ cmd_var_resolve(struct http *hp, char *spec)
 		return(hp->resp[0]);
 	if (!strcmp(spec, "resp.status"))
 		return(hp->resp[1]);
-	if (!strcmp(spec, "resp.msg"))
+	if (!strcmp(spec, "resp.reason"))
 		return(hp->resp[2]);
 	if (!strcmp(spec, "resp.chunklen"))
 		return(hp->chunklen);
 	if (!strcmp(spec, "req.bodylen"))
 		return(hp->bodylen);
+	if (!strcmp(spec, "req.body"))
+		return(hp->body != NULL ? hp->body : spec);
 	if (!strcmp(spec, "resp.bodylen"))
 		return(hp->bodylen);
 	if (!strcmp(spec, "resp.body"))
@@ -229,6 +329,11 @@ cmd_var_resolve(struct http *hp, char *spec)
 	} else if (!strncmp(spec, "resp.http.", 10)) {
 		hh = hp->resp;
 		hdr = spec + 10;
+	} else if (!strcmp(spec, "h2.state")) {
+		if (hp->h2)
+			return ("true");
+		else
+			return ("false");
 	} else
 		return (spec);
 	hdr = http_find_header(hh, hdr);
@@ -437,8 +542,8 @@ http_rxchar(struct http *hp, int n, int eof)
 			return (i);
 		if (i == 0)
 			vtc_log(hp->vl, hp->fatal,
-			    "HTTP rx EOF (fd:%d read: %s)",
-			    hp->fd, strerror(errno));
+			    "HTTP rx EOF (fd:%d read: %s) %d",
+			    hp->fd, strerror(errno), n);
 		if (i < 0)
 			vtc_log(hp->vl, hp->fatal,
 			    "HTTP rx failed (fd:%d read: %s)",
@@ -501,7 +606,6 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 	char *p;
 	int i, l, ll;
 
-	hp->body = hp->rxbuf + hp->prxbuf;
 	ll = 0;
 	p = http_find_header(hh, "transfer-encoding");
 	if (p != NULL && !strcasecmp(p, "chunked")) {
@@ -563,13 +667,15 @@ http_rxhdr(struct http *hp)
 	}
 	vtc_dump(hp->vl, 4, "rxhdr", hp->rxbuf, -1);
 	vtc_log(hp->vl, 4, "rxhdrlen = %zd", strlen(hp->rxbuf));
+	hp->body = hp->rxbuf + hp->prxbuf;
 }
 
-
-/**********************************************************************
- * Receive a response
+/* SECTION: client-server.spec.rxresp
+ *
+ * rxresp [-no_obj] (client only)
+ *         Receive and parse a response's headers and body. If -no_obj is present, only get
+ *         the headers.
  */
-
 static void
 cmd_http_rxresp(CMD_ARGS)
 {
@@ -603,6 +709,11 @@ cmd_http_rxresp(CMD_ARGS)
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
+/* SECTION: client-server.spec.rxreqhdrs
+ *
+ * rxresp (client only)
+ *         Receive and parse a response's headers.
+ */
 static void
 cmd_http_rxresphdrs(CMD_ARGS)
 {
@@ -616,7 +727,7 @@ cmd_http_rxresphdrs(CMD_ARGS)
 	av++;
 
 	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
+		vtc_log(hp->vl, 0, "Unknown http rxresp spec: %s\n", *av);
 	http_rxhdr(hp);
 	http_splitheader(hp, 0);
 	if (http_count_header(hp->resp, "Content-Length") > 1)
@@ -629,11 +740,8 @@ cmd_http_rxresphdrs(CMD_ARGS)
  * Ungzip rx'ed body
  */
 
-#define TRUST_ME(ptr)   ((void*)(uintptr_t)(ptr))
-
 #define OVERHEAD 64L
 
-
 static void
 cmd_http_gunzip_body(CMD_ARGS)
 {
@@ -813,6 +921,66 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 	return (av);
 }
 
+/* SECTION: client-server.spec.txre
+ *
+ * txreq|txresp [...]
+ *         Send a minimal request or response, but overload it if necessary.
+ *
+ *         txreq is client-specific and txresp is server-specific.
+ *
+ *         The only thing different between a request and a response, apart
+ *         from who can send them is that the first line (request line vs
+ *         status line), so all the options are prety much the same.
+ *
+ *         \-req STRING (txreq only)
+ *                 What method to use (default: "GET").
+ *
+ *         \-url STRING (txreq only)
+ *                 What location to use (default "/").
+ *
+ *         \-proto STRING
+ *                 What protocol use in the status line.
+ *                 (default: "HTTP/1.1").
+ *
+ *         \-status NUMBER (txresp only)
+ *                 What status code to return (default 200).
+ *
+ *         \-msg STRING (txresp only)
+ *                 What message to put in the status line (default: "OK").
+ *
+ *         These three switches can appear in any order but must come before the
+ *         following ones.
+ *
+ *         \-nolen
+ *                 Don't include a Content-Length header in the response.
+ *
+ *         \-hdr STRING
+ *                 Add STRING as a header, it must follow this format:
+ *                 "name: value". It can be called multiple times.
+ *
+ *         You can then use the arguments related to the body:
+ *
+ *         \-body STRING
+ *                 Input STRING as body.
+ *
+ *         \-bodylen NUMBER
+ *                 Generate and input a body that is NUMBER bytes-long.
+ *
+ *         \-gziplevel NUMBER
+ *		   Set the gzip level (call it before any of the other gzip
+ *		   switches).
+ *
+ *         \-gzipresidual NUMBER
+ *                 Add extra gzip bits. You should never need it.
+ *
+ *         \-gzipbody STRING
+ *                 Zip STRING and send it as body.
+ *
+ *         \-gziplen NUMBER
+ *                 Combine -body and -gzipbody: create a body of length NUMBER,
+ *                 zip it and send as body.
+ */
+
 /**********************************************************************
  * Transmit a response
  */
@@ -862,33 +1030,83 @@ cmd_http_txresp(CMD_ARGS)
 	http_write(hp, 4, "txresp");
 }
 
+static void
+cmd_http_upgrade(CMD_ARGS)
+{
+	char *h;
+	struct http *hp;
+
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	ONLY_SERVER(hp, av);
+	AN(hp->sfd);
+
+	h = http_find_header(hp->req, "Upgrade");
+	if (!h || strcmp(h, "h2c"))
+		vtc_log(vl, 0, "Req misses \"Upgrade: h2c\" header");
+
+	h = http_find_header(hp->req, "Connection");
+	if (!h || strcmp(h, "Upgrade, HTTP2-Settings"))
+		vtc_log(vl, 0, "Req misses \"Connection: "
+			"Upgrade, HTTP2-Settings\" header");
+
+	h = http_find_header(hp->req, "HTTP2-Settings");
+	if (!h)
+		vtc_log(vl, 0, "Req misses \"HTTP2-Settings\" header");
+
+
+	parse_string("txresp -status 101 "
+				"-hdr \"Connection: Upgrade\" "
+				"-hdr \"Upgrade: h2c\"\n", cmd, hp, vl);
+
+	b64_settings(hp, h);
+
+	parse_string("rxpri\n"
+			"stream 0 {\n"
+			"txsettings\n"
+			"rxsettings\n"
+			"txsettings -ack\n"
+			"rxsettings\n"
+			"expect settings.ack == true\n"
+			"} -start\n", cmd, hp, vl);
+}
+
 /**********************************************************************
  * Receive a request
  */
 
+/* SECTION: client-server.spec.rxreq
+ *
+ * rxreq (server only)
+ *         Receive and parse a request's headers and body.
+ */
 static void
 cmd_http_rxreq(CMD_ARGS)
 {
 	struct http *hp;
 
 	(void)cmd;
-	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	ONLY_SERVER(hp, av);
 	AZ(strcmp(av[0], "rxreq"));
 	av++;
 
 	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
+		vtc_log(vl, 0, "Unknown http rxreq spec: %s\n", *av);
 	http_rxhdr(hp);
 	http_splitheader(hp, 1);
 	if (http_count_header(hp->req, "Content-Length") > 1)
-		vtc_log(hp->vl, 0,
+		vtc_log(vl, 0,
 		    "Multiple Content-Length headers.\n");
 	http_swallow_body(hp, hp->req, 0);
-	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
+	vtc_log(vl, 4, "bodylen = %s", hp->bodylen);
 }
 
+/* SECTION: client-server.spec.rxreqhdrs
+ *
+ * rxreqhdrs
+ *         Receive and parse a request's headers (but not the body).
+ */
+
 static void
 cmd_http_rxreqhdrs(CMD_ARGS)
 {
@@ -909,6 +1127,12 @@ cmd_http_rxreqhdrs(CMD_ARGS)
 		    "Multiple Content-Length headers.\n");
 }
 
+/* SECTION: client-server.spec.rxreqbody
+ *
+ * rxreqbody (server only)
+ *         Receive a request's body.
+ */
+
 static void
 cmd_http_rxreqbody(CMD_ARGS)
 {
@@ -927,6 +1151,12 @@ cmd_http_rxreqbody(CMD_ARGS)
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
+/* SECTION: client-server.spec.rxrespbody
+ *
+ * rxrespbody
+ *         Receive a response's body.
+ */
+
 static void
 cmd_http_rxrespbody(CMD_ARGS)
 {
@@ -940,11 +1170,17 @@ cmd_http_rxrespbody(CMD_ARGS)
 	av++;
 
 	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
+		vtc_log(hp->vl, 0, "Unknown http rxrespbody spec: %s\n", *av);
 	http_swallow_body(hp, hp->resp, 0);
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
+/* SECTION: client-server.spec.rxchunk
+ *
+ * rxchunk
+ *         Receive an HTTP chunk
+ */
+
 static void
 cmd_http_rxchunk(CMD_ARGS)
 {
@@ -976,6 +1212,7 @@ cmd_http_txreq(CMD_ARGS)
 	const char *req = "GET";
 	const char *url = "/";
 	const char *proto = "HTTP/1.1";
+	const char *up = NULL;
 
 	(void)cmd;
 	(void)vl;
@@ -996,19 +1233,45 @@ cmd_http_txreq(CMD_ARGS)
 		} else if (!strcmp(*av, "-req")) {
 			req = av[1];
 			av++;
+		} else if (!hp->sfd && !strcmp(*av, "-up")) {
+			up = av[1];
+			av++;
 		} else
 			break;
 	}
 	VSB_printf(hp->vsb, "%s %s %s%s", req, url, proto, nl);
 
+	if (up)
+		VSB_printf(hp->vsb, "Connection: Upgrade, HTTP2-Settings%s"
+				"Upgrade: h2c%s"
+				"HTTP2-Settings: %s%s", nl, nl, up, nl);
+
 	av = http_tx_parse_args(av, vl, hp, NULL);
 	if (*av != NULL)
 		vtc_log(hp->vl, 0, "Unknown http txreq spec: %s\n", *av);
 	http_write(hp, 4, "txreq");
+
+	if (up) {
+		parse_string("rxresp\n"
+				"expect resp.status == 101\n"
+				"expect resp.http.connection == Upgrade\n"
+				"expect resp.http.upgrade == h2c\n"
+				"txpri\n", http_cmds, hp, vl);
+		b64_settings(hp, up);
+		parse_string("stream 0 {\n"
+				"txsettings\n"
+				"rxsettings\n"
+				"txsettings -ack\n"
+				"rxsettings\n"
+				"expect settings.ack == true"
+			     "} -start\n", http_cmds, hp, vl);
+	}
 }
 
-/**********************************************************************
- * Receive N characters
+/* SECTION: client-server.spec.recv
+ *
+ * recv NUMBER
+ *         Read NUMBER bytes from the connection.
  */
 
 static void
@@ -1035,8 +1298,10 @@ cmd_http_recv(CMD_ARGS)
 	}
 }
 
-/**********************************************************************
- * Send a string
+/* SECTION: client-server.spec.send
+ *
+ * send STRING
+ *         Push STRING on the connection.
  */
 
 static void
@@ -1057,8 +1322,10 @@ cmd_http_send(CMD_ARGS)
 		    strerror(errno));
 }
 
-/**********************************************************************
- * Send a string many times
+/* SECTION: client-server.spec.send_n
+ *
+ * send_n NUMBER STRING
+ *         Write STRING on the socket NUMBER times.
  */
 
 static void
@@ -1085,8 +1352,10 @@ cmd_http_send_n(CMD_ARGS)
 	}
 }
 
-/**********************************************************************
- * Send an OOB urgent message
+/* SECTION: client-server.spec.send_urgent
+ *
+ * send_urgent STRING
+ *         Send string as TCP OOB urgent data. You will never need this.
  */
 
 static void
@@ -1107,8 +1376,12 @@ cmd_http_send_urgent(CMD_ARGS)
 		    "Write error in http_send_urgent(): %s", strerror(errno));
 }
 
-/**********************************************************************
- * Send a hex string
+/* SECTION: client-server.spec.sendhex
+ *
+ * sendhex STRING
+ *         Send bytes as described by STRING. STRING should consist of hex pairs
+ *         possibly separated by whitespace or newlines. For example:
+ *         "0F EE a5    3df2".
  */
 
 static void
@@ -1148,8 +1421,10 @@ cmd_http_sendhex(CMD_ARGS)
 
 }
 
-/**********************************************************************
- * Send a string as chunked encoding
+/* SECTION: client-server.spec.chunked
+ *
+ * chunked STRING
+ *         Send STRING as chunked encoding.
  */
 
 static void
@@ -1168,6 +1443,13 @@ cmd_http_chunked(CMD_ARGS)
 	http_write(hp, 4, "chunked");
 }
 
+/* SECTION: client-server.spec.chunkedlen
+ *
+ * chunkedlen NUMBER
+ *         Do as ``chunked`` except that varnishtest will generate the string
+ *         for you, with a length of NUMBER characters.
+ */
+
 static void
 cmd_http_chunkedlen(CMD_ARGS)
 {
@@ -1203,8 +1485,11 @@ cmd_http_chunkedlen(CMD_ARGS)
 	http_write(hp, 4, "chunked");
 }
 
-/**********************************************************************
- * set the timeout
+
+/* SECTION: client-server.spec.timeout
+ *
+ * timeout NUMBER
+ *         Set the TCP timeout for this entity.
  */
 
 static void
@@ -1224,10 +1509,17 @@ cmd_http_timeout(CMD_ARGS)
 	hp->timeout = (int)(d * 1000.0);
 }
 
-/**********************************************************************
- * expect other end to close (server only)
+/* SECTION: client-server.spec.expect_close
+ *
+ * expect_close
+ *         Wait for the connected client to close the connection.
  */
 
+/* SECTION: client-server.spec.expect_close expect_close (server)
+ *
+ * Reads from the connection, expecting nothing to read but an EOF.
+ *
+ */
 static void
 cmd_http_expect_close(CMD_ARGS)
 {
@@ -1242,6 +1534,8 @@ cmd_http_expect_close(CMD_ARGS)
 	AZ(av[1]);
 
 	vtc_log(vl, 4, "Expecting close (fd = %d)", hp->fd);
+	if (hp->h2)
+		stop_h2(hp);
 	while (1) {
 		fds[0].fd = hp->fd;
 		fds[0].events = POLLIN | POLLERR;
@@ -1266,10 +1560,18 @@ cmd_http_expect_close(CMD_ARGS)
 	vtc_log(vl, 4, "fd=%d EOF, as expected", hp->fd);
 }
 
-/**********************************************************************
- * close a new connection  (server only)
+/* SECTION: client-server.spec.close
+ *
+ * close (server only)
+ *         Close the active TCP connection
  */
 
+/* SECTION: client-server.spec.close close (server)
+ *
+ * Close the connection. Not that if operating in H/2 mode, no extra (GOAWAY)
+ * frame is sent, it's simply a TCP close.
+ *
+ */
 static void
 cmd_http_close(CMD_ARGS)
 {
@@ -1281,14 +1583,23 @@ cmd_http_close(CMD_ARGS)
 	AZ(av[1]);
 	assert(hp->sfd != NULL);
 	assert(*hp->sfd >= 0);
+	if (hp->h2)
+		stop_h2(hp);
 	VTCP_close(&hp->fd);
 	vtc_log(vl, 4, "Closed");
 }
 
-/**********************************************************************
- * close and accept a new connection  (server only)
+/* SECTION: client-server.spec.accept_close
+ *
+ * accept (server only)
+ *         Close the active connection (if any) and accept a new one.
  */
 
+/* SECTION: client-server.spec.accept accept (server)
+ *
+ * Close the potential current connection, and accept a new one. Note that this
+ * new connection is H/1.
+ */
 static void
 cmd_http_accept(CMD_ARGS)
 {
@@ -1300,6 +1611,8 @@ cmd_http_accept(CMD_ARGS)
 	AZ(av[1]);
 	assert(hp->sfd != NULL);
 	assert(*hp->sfd >= 0);
+	if (hp->h2)
+		stop_h2(hp);
 	if (hp->fd >= 0)
 		VTCP_close(&hp->fd);
 	vtc_log(vl, 4, "Accepting");
@@ -1309,8 +1622,10 @@ cmd_http_accept(CMD_ARGS)
 	vtc_log(vl, 3, "Accepted socket fd is %d", hp->fd);
 }
 
-/**********************************************************************
- * loop operator
+/* SECTION: client-server.spec.loop
+ *
+ * loop NUMBER STRING
+ *         Process STRING as a specification, NUMBER times.
  */
 
 static void
@@ -1330,8 +1645,10 @@ cmd_http_loop(CMD_ARGS)
 	}
 }
 
-/**********************************************************************
- * Control fatality
+/* SECTION: client-server.spec.fatal
+ *
+ * fatal|non-fatal
+ *         Control whether a failure of this entity should stop the test.
  */
 
 static void
@@ -1350,6 +1667,119 @@ cmd_http_fatal(CMD_ARGS)
 	}
 }
 
+/* SECTION: client-server.spec.delay
+ *
+ * delay
+ *	Same as for the top-level delay.
+ *
+ * SECTION: client-server.spec.barrier
+ *
+ * barrier
+ *	Same as for the top-level barrier
+ */
+
+const char PREFACE[] = "PRI * HTTP/2.0\r\n\r\nSM\r\n\r\n";
+
+/* SECTION: client-server.spec.txpri txpri (client)
+ *
+ * Send an H/2 preface ("PRI * HTTP/2.0\\r\\n\\r\\nSM\\r\\n\\r\\n") and set
+ * client to H/2.
+ */
+static void
+cmd_http_txpri(CMD_ARGS)
+{
+	size_t l;
+	struct http *hp;
+	(void)cmd;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	ONLY_CLIENT(hp, av);
+
+	vtc_dump(hp->vl, 4, "txpri", PREFACE, sizeof(PREFACE) - 1);
+	l = write(hp->fd, PREFACE, sizeof(PREFACE) - 1);
+	if (l != sizeof(PREFACE) - 1)
+		vtc_log(vl, hp->fatal, "Write failed: (%zd vs %zd) %s",
+		    l, sizeof(PREFACE) - 1, strerror(errno));
+	start_h2(hp);
+	AN(hp->h2);
+}
+
+/* SECTION: client-server.spec.rxpri rxpri (server)
+ *
+ * Receive a preface, and if it matches, sets the server to H/2, aborts
+ * otherwise.
+ */
+static void
+cmd_http_rxpri(CMD_ARGS)
+{
+	struct http *hp;
+	(void)cmd;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	ONLY_SERVER(hp, av);
+
+	hp->prxbuf = 0;
+	if (!http_rxchar(hp, sizeof(PREFACE) - 1, 0))
+		vtc_log(vl, 0, "Couldn't retrieve connection preface");
+	if (strncmp(hp->rxbuf, PREFACE, sizeof(PREFACE) - 1))
+		vtc_log(vl, 0, "Received invalid preface\n");
+	start_h2(hp);
+	AN(hp->h2);
+}
+
+/* SECTION: client-server.spec.settings
+ *
+ * settings -dectbl INT
+ *         Force internal H/2 settings to certain values. Currently only
+ *         support setting the decoding table size.
+ */
+static void
+cmd_http_settings(CMD_ARGS)
+{
+	uint32_t n;
+	char *p;
+	struct http *hp;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	(void)cmd;
+
+	if (!hp->h2)
+		vtc_log(hp->vl, 0, "Only possible in H/2 mode");
+
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+
+	for(; *av != NULL; av++) {
+		if (!strcmp(*av, "-dectbl")) {
+			n = strtoul(av[1], &p, 0);
+			if (*p != '\0')
+				vtc_log(hp->vl, 0, "-dectbl takes an integer as"
+						" argument (found %s)", av[1]);
+			HPK_ResizeTbl(hp->decctx, n);
+			av++;
+		} else
+			vtc_log(vl, 0, "Unknown settings spec: %s\n", *av);
+	}
+}
+
+static void
+cmd_http_stream(CMD_ARGS)
+{
+	struct http *hp = (struct http *)priv;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	if (!hp->h2) {
+		vtc_log(hp->vl, 4, "Not in H/2 mode, do what's needed");
+		if (hp->sfd)
+			parse_string("rxpri", http_cmds, hp, vl);
+		else
+			parse_string("txpri", http_cmds, hp, vl);
+		parse_string("stream 0 {\n"
+				"txsettings\n"
+				"rxsettings\n"
+				"txsettings -ack\n"
+				"rxsettings\n"
+				"expect settings.ack == true"
+			     "} -run\n", http_cmds, hp, vl);
+	}
+	cmd_stream(av, hp, cmd, vl);
+}
+
 /**********************************************************************
  * Execute HTTP specifications
  */
@@ -1379,13 +1809,18 @@ static const struct cmds http_cmds[] = {
 	{ "chunkedlen",		cmd_http_chunkedlen },
 	{ "delay",		cmd_delay },
 	{ "barrier",		cmd_barrier },
-	{ "sema",		cmd_sema },
 	{ "expect_close",	cmd_http_expect_close },
 	{ "close",		cmd_http_close },
 	{ "accept",		cmd_http_accept },
 	{ "loop",		cmd_http_loop },
 	{ "fatal",		cmd_http_fatal },
 	{ "non-fatal",		cmd_http_fatal },
+
+	{ "rxpri",		cmd_http_rxpri },
+	{ "txpri",		cmd_http_txpri },
+	{ "stream",		cmd_http_stream },
+	{ "settings",		cmd_http_settings },
+	{ "upgrade",		cmd_http_upgrade },
 	{ NULL,			NULL }
 };
 
@@ -1420,10 +1855,13 @@ http_process(struct vtclog *vl, const char *spec, int sock, int *sfd)
 	hp->gziplevel = 0;
 	hp->gzipresidual = -1;
 
-	VTCP_hisname(sock, hp->rem_ip, VTCP_ADDRBUFSIZE, hp->rem_port, VTCP_PORTBUFSIZE);
+	VTCP_hisname(sock,
+	    hp->rem_ip, VTCP_ADDRBUFSIZE, hp->rem_port, VTCP_PORTBUFSIZE);
 	parse_string(spec, http_cmds, hp, vl);
+	if (hp->h2)
+		stop_h2(hp);
 	retval = hp->fd;
-	VSB_delete(hp->vsb);
+	VSB_destroy(&hp->vsb);
 	free(hp->rxbuf);
 	free(hp->rem_ip);
 	free(hp->rem_port);
