@@ -55,19 +55,6 @@ static double t0;
 
 /**********************************************************************/
 
-void
-vtc_loginit(char *buf, unsigned buflen)
-{
-
-	t0 = VTIM_mono();
-	vtclog_buf = buf;
-	vtclog_left = buflen;
-	AZ(pthread_mutex_init(&vtclog_mtx, NULL));
-	AZ(pthread_key_create(&log_key, NULL));
-}
-
-/**********************************************************************/
-
 
 struct vtclog *
 vtc_logopen(const char *id)
@@ -90,7 +77,7 @@ vtc_logclose(struct vtclog *vl)
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
 	if (pthread_getspecific(log_key) == vl)
 		AZ(pthread_setspecific(log_key, NULL));
-	VSB_delete(vl->vsb);
+	VSB_destroy(&vl->vsb);
 	AZ(pthread_mutex_destroy(&vl->mtx));
 	FREE_OBJ(vl);
 }
@@ -305,4 +292,17 @@ vtc_log_VAS_Fail(const char *func, const char *file, int line,
 	abort();
 }
 
-vas_f *VAS_Fail __attribute__((__noreturn__)) = vtc_log_VAS_Fail;
+/**********************************************************************/
+
+void
+vtc_loginit(char *buf, unsigned buflen)
+{
+
+	VAS_Fail = vtc_log_VAS_Fail;
+	t0 = VTIM_mono();
+	vtclog_buf = buf;
+	vtclog_left = buflen;
+	AZ(pthread_mutex_init(&vtclog_mtx, NULL));
+	AZ(pthread_key_create(&log_key, NULL));
+}
+
