@@ -26,33 +26,83 @@
  * SUCH DAMAGE.
  */
 
-/*
- * Synopsis:
- *   -v <varnish-instance>
- *   -d <0|1> (head/tail mode)
- *   -g <grouping-mode>
- *   -q <query>
+/* SECTION: logexpect logexpect
  *
- *   vsl arguments (vsl_arg.c)
- *   -b                   Only display backend records
- *   -c                   Only display client records
- *   -C                   Caseless regular expressions
- *   -i <taglist>         Include tags
- *   -I <[taglist:]regex> Include by regex
- *   -L <limit>           Incomplete transaction limit
- *   -T <seconds>         Transaction end timeout
+ * Reads the VSL and looks for records matching a given specification. It will
+ * process records trying to match the first pattern, and when done, will
+ * continue processing, trying to match the following pattern. If a pattern
+ * isn't matched, the test will fail.
  *
+ * logexpect threads are declared this way::
  *
- * logexpect lN -v <id> [-g <grouping>] [-d 0|1] [-q query] [vsl arguments] {
- *    expect <skip> <vxid> <tag> <regex>
- * }
+ *         logexpect lNAME -v <id> [-g <grouping>] [-d 0|1] [-q query] \
+ *                 [vsl arguments] {
+ *                         expect <skip> <vxid> <tag> <regex>
+ *                         expect <skip> <vxid> <tag> <regex>
+ *                         ...
+ *                 } [-start|-wait]
  *
- * skip: [uint|*]		Max number of record to skip
- * vxid: [uint|*|=]		vxid to match
- * tag:  [tagname|*|=]		Tag to match against
- * regex:			regular expression to match against (optional)
- * *:				Match anything
- * =:				Match value of last successfully matched record
+ * And once declared, you can start them, or wait on them::
+ *
+ *         logexpect lNAME <-start|-wait>
+ *
+ * With:
+ *
+ * lNAME
+ *         Name the logexpect thread, it must start with 'l'.
+ *
+ * \-v id
+ *         Specify the varnish instance to use (most of the time, id=v1).
+ *
+ * \-g <session|request|vxid|raw
+ *         Decide how records are grouped, see -g in ``man varnishlog`` for more
+ *         information.
+ *
+ * \-d <0|1>
+ *         Start processing log records at the head of the log instead of the
+ *         tail.
+ *
+ * \-q query
+ *         Filter records using a query expression, see ``man vsl-query`` for
+ *         more information.
+ *
+ * \-start
+ *         Start the logexpect thread in the background.
+ *
+ * \-wait
+ *         Wait for the logexpect thread to finish
+ *
+ * VSL arguments (similar to the varnishlog options):
+ *
+ * \-b|-c
+ *         Process only backend/client records.
+ *
+ * \-C
+ *         Use caseless regex
+ *
+ * \-i <taglist>
+ *         Include tags
+ *
+ * \-I <[taglist:]regex>
+ *         Include by regex
+ *
+ * \-T <seconds>
+ *         Transaction end timeout
+ *
+ * And the arguments of the specifications lines are:
+ *
+ * skip: [uint|*]
+ *         Max number of record to skip
+ *
+ * vxid: [uint|*|=]
+ *         vxid to match
+ *
+ * tag:  [tagname|*|=]
+ *         Tag to match against
+ *
+ * regex:
+ *         regular expression to match against (optional) ('*' is anything, '='
+ *         is the value of the last matched record)
  */
 
 #include "config.h"
@@ -122,7 +172,7 @@ logexp_delete_tests(struct logexp *le)
 	while ((test = VTAILQ_FIRST(&le->tests))) {
 		CHECK_OBJ_NOTNULL(test, LOGEXP_TEST_MAGIC);
 		VTAILQ_REMOVE(&le->tests, test, list);
-		VSB_delete(test->str);
+		VSB_destroy(&test->str);
 		if (test->vre)
 			VRE_free(&test->vre);
 		FREE_OBJ(test);
@@ -142,7 +192,7 @@ logexp_delete(struct logexp *le)
 	free(le->query);
 	VSM_Delete(le->vsm);
 	if (le->n_arg)
-		VSB_delete(le->n_arg);
+		VSB_destroy(&le->n_arg);
 	FREE_OBJ(le);
 }
 
@@ -469,7 +519,7 @@ logexp_spec(struct logexp *le, const char *spec)
 }
 
 void
-cmd_logexp(CMD_ARGS)
+cmd_logexpect(CMD_ARGS)
 {
 	struct logexp *le, *le2;
 	const char tmpdir[] = "${tmpdir}";
@@ -530,16 +580,14 @@ cmd_logexp(CMD_ARGS)
 				vtc_log(le->vl, 0, "Missing -v argument");
 				return;
 			}
-			if (le->n_arg != NULL) {
-				VSB_delete(le->n_arg);
-				le->n_arg = NULL;
-			}
+			if (le->n_arg != NULL)
+				VSB_destroy(&le->n_arg);
 			vsb = VSB_new_auto();
 			AN(vsb);
 			AZ(VSB_printf(vsb, "%s/%s", tmpdir, av[1]));
 			AZ(VSB_finish(vsb));
 			le->n_arg = macro_expand(le->vl, VSB_data(vsb));
-			VSB_delete(vsb);
+			VSB_destroy(&vsb);
 			if (le->n_arg == NULL)
 				return;
 			av++;
