@@ -38,7 +38,6 @@
 
 #include "vtc.h"
 
-#include "vss.h"
 #include "vtcp.h"
 
 struct server {
@@ -167,7 +166,7 @@ server_thread(void *priv)
 		l = sizeof addr_s;
 		fd = accept(s->sock, addr, &l);
 		if (fd < 0)
-			vtc_log(vl, 0, "Accepted failed: %s", strerror(errno));
+			vtc_log(vl, 0, "Accept failed: %s", strerror(errno));
 		vtc_log(vl, 3, "accepted fd %d", fd);
 		fd = http_process(vl, s->spec, fd, &s->sock);
 		vtc_log(vl, 3, "shutting fd %d", fd);
