@@ -35,6 +35,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <fnmatch.h>
 #include <inttypes.h>
 #include <poll.h>
 #include <stdio.h>
@@ -54,6 +55,8 @@
 #include "vtim.h"
 
 extern int leave_temp;
+extern char *vmod_path;
+extern struct vsb *params_vsb;
 
 struct varnish {
 	unsigned		magic;
@@ -66,6 +69,8 @@ struct varnish {
 	int			fds[4];
 	pid_t			pid;
 
+	float			syntax;
+
 	pthread_t		tp;
 	pthread_t		tp_vsl;
 
@@ -132,7 +137,7 @@ varnish_ask_cli(const struct varnish *v, const char *cmd, char **repl)
 static void
 wait_stopped(const struct varnish *v)
 {
-	char *r;
+	char *r = NULL;
 	enum VCLI_status_e st;
 
 	while (1) {
@@ -146,6 +151,7 @@ wait_stopped(const struct varnish *v)
 			break;
 		}
 		free(r);
+		r = NULL;
 		(void)usleep(200000);
 	}
 }
@@ -156,7 +162,7 @@ wait_stopped(const struct varnish *v)
 static void
 wait_running(const struct varnish *v)
 {
-	char *r;
+	char *r = NULL;
 	enum VCLI_status_e st;
 
 	while (1) {
@@ -176,6 +182,7 @@ wait_running(const struct varnish *v)
 			break;
 		}
 		free(r);
+		r = NULL;
 		(void)usleep(200000);
 	}
 }
@@ -286,6 +293,8 @@ varnish_new(const char *name)
 
 	REPLACE(v->jail, "");
 
+	v->syntax = 4.0;
+
 	v->vl = vtc_logopen(name);
 	AN(v->vl);
 
@@ -294,7 +303,7 @@ varnish_new(const char *name)
 	AN(vsb);
 	v->workdir = strdup(VSB_data(vsb));
 	AN(v->workdir);
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 
 	bprintf(buf, "rm -rf %s ; mkdir -p %s", v->workdir, v->workdir);
 	AZ(system(buf));
@@ -386,7 +395,7 @@ varnish_launch(struct varnish *v)
 	struct pollfd fd[2];
 	enum VCLI_status_e u;
 	const char *err;
-	char *r;
+	char *r = NULL;
 
 	v->vd = VSM_New();
 
@@ -402,8 +411,9 @@ varnish_launch(struct varnish *v)
 	vsb = VSB_new_auto();
 	AN(vsb);
 	VSB_printf(vsb, "cd ${pwd} &&");
-	VSB_printf(vsb, " exec ${varnishd} %s -d -n %s",
+	VSB_printf(vsb, " exec varnishd %s -d -n %s",
 	    v->jail, v->workdir);
+	VSB_cat(vsb, VSB_data(params_vsb));
 	if (vtc_witness)
 		VSB_cat(vsb, " -p debug=+witness");
 	if (leave_temp)
@@ -419,12 +429,14 @@ varnish_launch(struct varnish *v)
 		VSB_printf(vsb, ",%s", v->proto);
 	VSB_printf(vsb, " -M '%s %s'", abuf, pbuf);
 	VSB_printf(vsb, " -P %s/varnishd.pid", v->workdir);
+	if (vmod_path != NULL)
+		VSB_printf(vsb, " -p vmod_path=%s", vmod_path);
 	VSB_printf(vsb, " %s", VSB_data(v->args));
 	AZ(VSB_finish(vsb));
 	vtc_log(v->vl, 3, "CMD: %s", VSB_data(vsb));
 	vsb1 = macro_expand(v->vl, VSB_data(vsb));
 	AN(vsb1);
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 	vsb = vsb1;
 	vtc_log(v->vl, 3, "CMD: %s", VSB_data(vsb));
 	AZ(pipe(&v->fds[0]));
@@ -451,7 +463,7 @@ varnish_launch(struct varnish *v)
 	AZ(close(v->fds[3]));
 	v->fds[0] = v->fds[2];
 	v->fds[2] = v->fds[3] = -1;
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 	AZ(pthread_create(&v->tp, NULL, varnish_thread, v));
 	AZ(pthread_create(&v->tp_vsl, NULL, varnishlog_thread, v));
 
@@ -512,6 +524,7 @@ varnish_launch(struct varnish *v)
 	VCLI_AuthResponse(nfd, r, abuf + 5);
 	AZ(close(nfd));
 	free(r);
+	r = NULL;
 	strcat(abuf, "\n");
 
 	u = varnish_ask_cli(v, abuf, &r);
@@ -533,7 +546,7 @@ static void
 varnish_start(struct varnish *v)
 {
 	enum VCLI_status_e u;
-	char *resp, *h, *p;
+	char *resp = NULL, *h, *p;
 
 	if (v->cli_fd < 0)
 		varnish_launch(v);
@@ -547,6 +560,7 @@ varnish_start(struct varnish *v)
 		vtc_log(v->vl, 0, "CLI start command failed: %u %s", u, resp);
 	wait_running(v);
 	free(resp);
+	resp = NULL;
 	u = varnish_ask_cli(v, "debug.xid 999", &resp);
 	if (vtc_error)
 		return;
@@ -554,6 +568,7 @@ varnish_start(struct varnish *v)
 		vtc_log(v->vl, 0, "CLI debug.xid command failed: %u %s",
 		    u, resp);
 	free(resp);
+	resp = NULL;
 	u = varnish_ask_cli(v, "debug.listen_address", &resp);
 	if (vtc_error)
 		return;
@@ -696,8 +711,7 @@ varnish_cli(struct varnish *v, const char *cli, unsigned exp)
  */
 
 static void
-varnish_vcl(struct varnish *v, const char *vcl, enum VCLI_status_e expect,
-    char **resp)
+varnish_vcl(struct varnish *v, const char *vcl, int fail, char **resp)
 {
 	struct vsb *vsb;
 	enum VCLI_status_e u;
@@ -709,28 +723,31 @@ varnish_vcl(struct varnish *v, const char *vcl, enum VCLI_status_e expect,
 	vsb = VSB_new_auto();
 	AN(vsb);
 
-	VSB_printf(vsb, "vcl.inline vcl%d << %s\nvcl 4.0;\n%s\n%s\n",
-	    ++v->vcl_nbr, NONSENSE, vcl, NONSENSE);
+	VSB_printf(vsb, "vcl.inline vcl%d << %s\nvcl %.1f;\n%s\n%s\n",
+	    ++v->vcl_nbr, NONSENSE, v->syntax, vcl, NONSENSE);
 	AZ(VSB_finish(vsb));
 
 	u = varnish_ask_cli(v, VSB_data(vsb), resp);
-	if (u != expect) {
-		VSB_delete(vsb);
+	if (u == CLIS_OK && fail) {
+		VSB_destroy(&vsb);
 		vtc_log(v->vl, 0,
-		    "VCL compilation got %u expected %u",
-		    u, expect);
+		    "VCL compilation succeeded expected failure");
 		return;
-	}
-	if (u == CLIS_OK) {
+	} else if (u == CLIS_OK) {
 		VSB_clear(vsb);
 		VSB_printf(vsb, "vcl.use vcl%d", v->vcl_nbr);
 		AZ(VSB_finish(vsb));
 		u = varnish_ask_cli(v, VSB_data(vsb), NULL);
 		assert(u == CLIS_OK);
+	} else if (!fail) {
+		VSB_destroy(&vsb);
+		vtc_log(v->vl, 0,
+		    "VCL compilation failed expected success");
+		return;
 	} else {
 		vtc_log(v->vl, 2, "VCL compilation failed (as expected)");
 	}
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 }
 
 /**********************************************************************
@@ -753,7 +770,7 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 	vsb2 = VSB_new_auto();
 	AN(vsb2);
 
-	VSB_printf(vsb2, "vcl 4.0;\n");
+	VSB_printf(vsb2, "vcl %.1f;\n", v->syntax);
 
 	cmd_server_genvcl(vsb2);
 
@@ -765,8 +782,8 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 
 	u = varnish_ask_cli(v, VSB_data(vsb), NULL);
 	if (u != CLIS_OK) {
-		VSB_delete(vsb);
-		VSB_delete(vsb2);
+		VSB_destroy(&vsb);
+		VSB_destroy(&vsb2);
 		vtc_log(v->vl, 0, "FAIL VCL does not compile");
 		return;
 	}
@@ -775,8 +792,66 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 	AZ(VSB_finish(vsb));
 	u = varnish_ask_cli(v, VSB_data(vsb), NULL);
 	assert(u == CLIS_OK);
-	VSB_delete(vsb);
-	VSB_delete(vsb2);
+	VSB_destroy(&vsb);
+	VSB_destroy(&vsb2);
+}
+
+/**********************************************************************
+ */
+
+struct dump_priv {
+	const char *arg;
+	const struct varnish *v;
+};
+
+static int
+do_stat_dump_cb(void *priv, const struct VSC_point * const pt)
+{
+	const struct varnish *v;
+	struct dump_priv *dp;
+	uint64_t u;
+	char buf[1024];
+
+	if (pt == NULL)
+		return (0);
+	dp = priv;
+	v = dp->v;
+
+	if(strcmp(pt->desc->ctype, "uint64_t"))
+		return (0);
+	u = *(const volatile uint64_t*)pt->ptr;
+
+	strcpy(buf, pt->section->type);
+	if (pt->section->ident[0] != '\0') {
+		strcat(buf, ".");
+		strcat(buf, pt->section->ident);
+	}
+	strcat(buf, ".");
+	strcat(buf, pt->desc->name);
+
+	if (strcmp(dp->arg, "*")) {
+		if (fnmatch(dp->arg, buf, 0))
+			return (0);
+	}
+
+	vtc_log(v->vl, 4, "VSC %s %ju",  buf, (uintmax_t)u);
+	return (0);
+}
+
+static void
+varnish_vsc(const struct varnish *v, const char *arg)
+{
+	struct dump_priv dp;
+
+	memset(&dp, 0, sizeof dp);
+	dp.v = v;
+	dp.arg = arg;
+	if (VSM_Abandoned(v->vd)) {
+		VSM_Close(v->vd);
+		VSM_Open(v->vd);
+	}
+
+	(void)VSC_Iter(v->vd, NULL, do_stat_dump_cb, &dp);
 }
 
 /**********************************************************************
@@ -811,6 +886,9 @@ do_stat_cb(void *priv, const struct VSC_point * const pt)
 	return (1);
 }
 
+/**********************************************************************
+ */
+
 static void
 varnish_expect(const struct varnish *v, char * const *av)
 {
@@ -908,9 +986,94 @@ varnish_expect(const struct varnish *v, char * const *av)
 		    av[0], (uintmax_t)sp.val, av[1], av[2], (uintmax_t)ref);
 	}
 }
-
-/**********************************************************************
- * Varnish server cmd dispatch
+/* SECTION: varnish varnish
+ *
+ * Define and interact with varnish instances.
+ *
+ * To define a Varnish server, you'll use this syntax::
+ *
+ *	varnish vNAME [-arg STRING] [-vcl STRING] [-vcl+backend STRING]
+ *		[-errvcl STRING STRING] [-jail STRING] [-proto PROXY]
+ *
+ * The first ``varnish vNAME`` invocation will start the varnishd master
+ * process in the background, waiting for the ``-start`` switch to actually
+ * start the child.
+ *
+ * With:
+ *
+ * vNAME
+ *         Identify the Varnish server with a string, it must starts with 'v'.
+ *
+ * \-arg STRING
+ *         Pass an argument to varnishd, for example "-h simple_list".
+ *
+ * \-vcl STRING
+ *         Specify the VCL to load on this Varnish instance. You'll probably
+ *         want to use multi-lines strings for this ({...}).
+ *
+ * \-vcl+backend STRING
+ *         Do the exact same thing as -vcl, but adds the definition block of
+ *         known backends (ie. already defined).
+ *
+ * \-errvcl STRING1 STRING2
+ *         Load STRING2 as VCL, expecting it to fail, and Varnish to send an
+ *         error string matching STRING2
+ *
+ * \-jail STRING
+ *         Look at ``man varnishd`` (-j) for more information.
+ *
+ * \-proto PROXY
+ *         Have Varnish use the proxy protocol. Note that PROXY here is the
+ *         actual string.
+ *
+ * You can decide to start the Varnish instance and/or wait for several events::
+ *
+ *         varnish vNAME [-start] [-wait] [-wait-running] [-wait-stopped]
+ *
+ * \-start
+ *         Start the child process.
+ *
+ * \-stop
+ *         Stop the child process.
+ *
+ * \-syntax
+ *         Set the VCL syntax level (default: 4.0)
+ *
+ * \-wait
+ *         Wait for that instance to terminate.
+ *
+ * \-wait-running
+ *         Wait for the Varnish child process to be started.
+ *
+ * \-wait-stopped
+ *         Wait for the Varnish child process to stop.
+ *
+ * \-cleanup
+ *         Once Varnish is stopped, clean everything after it. This is only used
+ *         in one test and you should never need it.
+ *
+ * Once Varnish is started, you can talk to it (as you would through
+ * ``varnishadm``) with these additional switches::
+ *
+ *         varnish vNAME [-cli STRING] [-cliok STRING] [-clierr STRING]
+ *                       [-expect STRING OP NUMBER]
+ *
+ * \-cli STRING|-cliok STRING|-clierr STATUS STRING
+ *         All three of these will send STRING to the CLI, the only difference
+ *         is what they expect the return code to be. -cli doesn't expect
+ *         anything, -cliok expects 200 and -clierr expects STATUS
+ *
+ * \-expect STRING OP NUMBER
+ *         Look into the VSM and make sure the counter identified by STRING has
+ *         a correct value. OP can be ==, >, >=, <, <=. For example::
+ *
+ *                 varnish v1 -expect SMA.s1.g_space > 1000000
+ *
+ * \-vsc PATTERN
+ *         Dump VSC counters matching PATTERN.  The PATTERN is a 'glob'
+ *         style pattern (ie: fnmatch(3)) as used in shell filename expansion.
+ *         To see all counters use pattern "*", to see all counters about
+ *         requests use "*req*".
  */
 
 void
@@ -982,7 +1145,7 @@ cmd_varnish(CMD_ARGS)
 			char *r = NULL;
 			AN(av[1]);
 			AN(av[2]);
-			varnish_vcl(v, av[2], CLIS_PARAM, &r);
+			varnish_vcl(v, av[2], 1, &r);
 			if (strstr(r, av[1]) == NULL)
 				vtc_log(v->vl, 0,
 				    "Did not find expected string: (\"%s\")",
@@ -1023,9 +1186,15 @@ cmd_varnish(CMD_ARGS)
 			varnish_stop(v);
 			continue;
 		}
+		if (!strcmp(*av, "-syntax")) {
+			AN(av[1]);
+			v->syntax = strtod(av[1], NULL);
+			av++;
+			continue;
+		}
 		if (!strcmp(*av, "-vcl")) {
 			AN(av[1]);
-			varnish_vcl(v, av[1], CLIS_OK, NULL);
+			varnish_vcl(v, av[1], 0, NULL);
 			av++;
 			continue;
 		}
@@ -1035,6 +1204,12 @@ cmd_varnish(CMD_ARGS)
 			av++;
 			continue;
 		}
+		if (!strcmp(*av, "-vsc")) {
+			AN(av[1]);
+			varnish_vsc(v, av[1]);
+			av++;
+			continue;
+		}
 		if (!strcmp(*av, "-wait-stopped")) {
 			wait_stopped(v);
 			continue;
