@@ -290,10 +290,6 @@ VSC_F(n_objecthead,		uint64_t, 1, 'g', 'i', info,
     "objecthead structs made",
 	"Number of objecthead structs made"
 )
-VSC_F(n_waitinglist,		uint64_t, 1, 'g', 'i', debug,
-    "waitinglist structs made",
-	"Number of waitinglist structs made"
-)
 
 VSC_F(n_backend,		uint64_t, 0, 'g', 'i', info,
     "Number of backends",
@@ -390,7 +386,7 @@ VSC_F(sess_readahead,		uint64_t, 1, 'c', 'i', info,
 )
 VSC_F(sess_herd,		uint64_t, 1, 'c', 'i', diag,
     "Session herd",
-	""
+	"Number of times the timeout_linger triggered"
 )
 
 #define SESS_CLOSE_ERR0 "OK  "
