@@ -29,19 +29,23 @@
 
 #include "config.h"
 
-#include <stdio.h>
 #include <errno.h>
 #include <poll.h>
-#include <time.h>
-#include <signal.h>
-#include <string.h>
-#include <stdlib.h>
 #include <pthread.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
 
-#include "libvarnish.h"
-#include "vev.h"
+#include "vdef.h"
 #include "miniobj.h"
+#include "vas.h"
+
 #include "binary_heap.h"
+#include "vqueue.h"
+#include "vev.h"
+#include "vtim.h"
 
 #undef DEBUG_EVENTS
 
@@ -90,7 +94,7 @@ struct vev_base {
 
 /*--------------------------------------------------------------------*/
 
-static void
+static void __match_proto__(binheap_update_t)
 vev_bh_update(void *priv, void *a, unsigned u)
 {
 	struct vev_base *evb;
@@ -101,11 +105,11 @@ vev_bh_update(void *priv, void *a, unsigned u)
 	e->__binheap_idx = u;
 }
 
-static int
-vev_bh_cmp(void *priv, void *a, void *b)
+static int __match_proto__(binheap_cmp_t)
+vev_bh_cmp(void *priv, const void *a, const void *b)
 {
 	struct vev_base *evb;
-	struct vev *ea, *eb;
+	const struct vev *ea, *eb;
 
 	CAST_OBJ_NOTNULL(evb, priv, VEV_BASE_MAGIC);
 	CAST_OBJ_NOTNULL(ea, a, VEV_MAGIC);
@@ -254,7 +258,7 @@ vev_add(struct vev_base *evb, struct vev *e)
 		es = &vev_sigs[e->sig];
 		if (es->vev != NULL)
 			return (EBUSY);
-		assert(es->happened == 0);
+		AZ(es->happened);
 		es->vev = e;
 		es->vevb = evb;
 		es->sigact.sa_flags = e->sig_flags;
@@ -278,7 +282,7 @@ vev_add(struct vev_base *evb, struct vev *e)
 	e->magic = VEV_MAGIC;	/* before binheap_insert() */
 
 	if (e->timeout != 0.0) {
-		e->__when += TIM_mono() + e->timeout;
+		e->__when += VTIM_mono() + e->timeout;
 		binheap_insert(evb->binheap, e);
 		assert(e->__binheap_idx > 0);
 		DBG(evb, "... bidx = %d\n", e->__binheap_idx);
@@ -296,7 +300,7 @@ vev_add(struct vev_base *evb, struct vev *e)
 
 	if (e->sig > 0) {
 		assert(es != NULL);
-		assert(sigaction(e->sig, &es->sigact, NULL) == 0);
+		AZ(sigaction(e->sig, &es->sigact, NULL));
 	}
 
 	return (0);
@@ -317,7 +321,7 @@ vev_del(struct vev_base *evb, struct vev *e)
 
 	if (e->__binheap_idx != 0)
 		binheap_delete(evb->binheap, e->__binheap_idx);
-	assert(e->__binheap_idx == 0);
+	AZ(e->__binheap_idx);
 
 	if (e->fd >= 0) {
 		DBG(evb, "... pidx = %d\n", e->__poll_idx);
@@ -338,7 +342,7 @@ vev_del(struct vev_base *evb, struct vev *e)
 		es->vevb = NULL;
 		es->sigact.sa_flags = e->sig_flags;
 		es->sigact.sa_handler = SIG_DFL;
-		assert(sigaction(e->sig, &es->sigact, NULL) == 0);
+		AZ(sigaction(e->sig, &es->sigact, NULL));
 		es->happened = 0;
 	}
 
@@ -455,7 +459,7 @@ vev_schedule_one(struct vev_base *evb)
 	if (e != NULL) {
 		CHECK_OBJ_NOTNULL(e, VEV_MAGIC);
 		assert(e->__binheap_idx == 1);
-		t = TIM_mono();
+		t = VTIM_mono();
 		if (e->__when <= t)
 			return (vev_sched_timeout(evb, e, t));
 		tmo = (int)((e->__when - t) * 1e3);
@@ -478,7 +482,7 @@ vev_schedule_one(struct vev_base *evb)
 		return (vev_sched_signal(evb));
 	if (i == 0) {
 		assert(e != NULL);
-		t = TIM_mono();
+		t = VTIM_mono();
 		if (e->__when <= t)
 			return (vev_sched_timeout(evb, e, t));
 	}
@@ -514,6 +518,6 @@ vev_schedule_one(struct vev_base *evb)
 			free(e);
 		}
 	}
-	assert(i == 0);
+	AZ(i);
 	return (1);
 }
