@@ -75,7 +75,7 @@ vxp_expr_lhs(struct vxp *vxp, struct vex_lhs **plhs)
 	AZ(*plhs);
 	ALLOC_OBJ(*plhs, VEX_LHS_MAGIC);
 	AN(*plhs);
-	(*plhs)->tags = vbit_init(SLT__MAX);
+	(*plhs)->tags = vbit_new(SLT__MAX);
 	(*plhs)->level = -1;
 
 	if (vxp->t->tok == '{') {
