@@ -74,26 +74,14 @@ struct method method_tab[] = {
 
 /*--------------------------------------------------------------------*/
 
-static void
-TlDoFree(struct vcc *tl, void *p)
-{
-	struct membit *mb;
-
-	mb = calloc(sizeof *mb, 1);
-	assert(mb != NULL);
-	mb->ptr = p;
-	VTAILQ_INSERT_TAIL(&tl->membits, mb, list);
-}
-
-
-void *
+void * __match_proto__(TlAlloc)
 TlAlloc(struct vcc *tl, unsigned len)
 {
 	void *p;
 
+	(void)tl;
 	p = calloc(len, 1);
 	assert(p != NULL);
-	TlDoFree(tl, p);
 	return (p);
 }
 
@@ -145,12 +133,12 @@ New_IniFin(struct vcc *tl)
 int
 IsMethod(const struct token *t)
 {
-	struct method *m;
+	int i;
 
 	assert(t->tok == ID);
-	for(m = method_tab; m->name != NULL; m++) {
-		if (vcc_IdIs(t, m->name))
-			return (m - method_tab);
+	for (i = 1; method_tab[i].name != NULL; i++) {
+		if (vcc_IdIs(t, method_tab[i].name))
+			return (i);
 	}
 	if ((t->b[0] == 'v'|| t->b[0] == 'V') &&
 	    (t->b[1] == 'c'|| t->b[1] == 'C') &&
@@ -202,46 +190,12 @@ Fc(const struct vcc *tl, int indent, const char *fmt, ...)
 
 /*--------------------------------------------------------------------*/
 
-void
-EncString(struct vsb *sb, const char *b, const char *e, int mode)
-{
-
-	if (e == NULL)
-		e = strchr(b, '\0');
-
-	VSB_cat(sb, "\"");
-	for (; b < e; b++) {
-		switch (*b) {
-		case '?':	// Trigraphs
-		case '\\':
-		case '"':
-			VSB_printf(sb, "\\%c", *b);
-			break;
-		case '\n':
-			VSB_printf(sb, "\\n");
-			if (mode)
-				VSB_printf(sb, "\"\n\t\"");
-			break;
-		case '\t': VSB_printf(sb, "\\t"); break;
-		case '\r': VSB_printf(sb, "\\r"); break;
-		case ' ': VSB_printf(sb, " "); break;
-		default:
-			if (isgraph(*b))
-				VSB_printf(sb, "%c", *b);
-			else
-				VSB_printf(sb, "\\%03o", (uint8_t)*b);
-			break;
-		}
-	}
-	VSB_cat(sb, "\"");
-}
-
 void
 EncToken(struct vsb *sb, const struct token *t)
 {
 
 	assert(t->tok == CSTR);
-	EncString(sb, t->dec, NULL, 1);
+	VSB_quote(sb, t->dec, -1, VSB_QUOTE_CSTR);
 }
 
 /*--------------------------------------------------------------------
@@ -264,7 +218,7 @@ EmitCoordinates(const struct vcc *tl, struct vsb *vsb)
 	VSB_printf(vsb, "\nstatic const char *srcname[VGC_NSRCS] = {\n");
 	VTAILQ_FOREACH(sp, &tl->sources, list) {
 		VSB_printf(vsb, "\t");
-		EncString(vsb, sp->name, NULL, 0);
+		VSB_quote(vsb, sp->name, -1, VSB_QUOTE_CSTR);
 		VSB_printf(vsb, ",\n");
 	}
 	VSB_printf(vsb, "};\n");
@@ -272,10 +226,10 @@ EmitCoordinates(const struct vcc *tl, struct vsb *vsb)
 	VSB_printf(vsb, "\nstatic const char *srcbody[%u] = {\n", tl->nsources);
 	VTAILQ_FOREACH(sp, &tl->sources, list) {
 		VSB_printf(vsb, "    /* ");
-		EncString(vsb, sp->name, NULL, 0);
+		VSB_quote(vsb, sp->name, -1, VSB_QUOTE_CSTR);
 		VSB_printf(vsb, "*/\n");
 		VSB_printf(vsb, "\t");
-		EncString(vsb, sp->b, sp->e, 1);
+		VSB_quote(vsb, sp->b, sp->e - sp->b, VSB_QUOTE_CSTR);
 		VSB_printf(vsb, ",\n");
 	}
 	VSB_printf(vsb, "};\n\n");
@@ -326,14 +280,13 @@ EmitCoordinates(const struct vcc *tl, struct vsb *vsb)
  * Init/Fini/Event
  *
  * We call DISCARD and COLD events in the opposite order of LOAD and
- * WARM. The child will panic if a USE event fails, since a WARM event
- * leads to a usable state.
+ * WARM.
  */
 
 static void
 EmitInitFini(const struct vcc *tl)
 {
-	struct inifin *p;
+	struct inifin *p, *q = NULL;
 	unsigned has_event = 0;
 
 	Fh(tl, 0, "\n");
@@ -351,7 +304,7 @@ EmitInitFini(const struct vcc *tl)
 		if (VSB_len(p->ini))
 			Fc(tl, 0, "\t/* %u */\n%s\n", p->n, VSB_data(p->ini));
 		Fc(tl, 0, "\tvgc_inistep = %u;\n\n", p->n);
-		VSB_delete(p->ini);
+		VSB_destroy(&p->ini);
 
 		AZ(VSB_finish(p->event));
 		if (VSB_len(p->event))
@@ -359,7 +312,7 @@ EmitInitFini(const struct vcc *tl)
 	}
 
 	Fc(tl, 0, "\t(void)VGC_function_vcl_init(ctx);\n");
-	Fc(tl, 0, "\treturn(*ctx->handling == VCL_RET_OK ? 0: -1);\n");
+	Fc(tl, 0, "\treturn (*ctx->handling == VCL_RET_OK ? 0: -1);\n");
 	Fc(tl, 0, "}\n");
 
 	/*
@@ -368,18 +321,23 @@ EmitInitFini(const struct vcc *tl)
 	Fc(tl, 0, "\nstatic int\nVGC_Discard(VRT_CTX)\n{\n\n");
 
 	Fc(tl, 0, "\t(void)VGC_function_vcl_fini(ctx);\n\n");
+
+	Fc(tl, 0, "\tswitch (vgc_inistep) {\n\n");
 	VTAILQ_FOREACH_REVERSE(p, &tl->inifin, inifinhead, list) {
 		AZ(VSB_finish(p->fin));
+		if (q)
+			assert(q->n > p->n);
+		q = p;
 		if (VSB_len(p->fin)) {
-			Fc(tl, 0, "\t/* %u */\n", p->n);
-			Fc(tl, 0, "\tif (vgc_inistep >= %u) {\n", p->n);
-			Fc(tl, 0, "%s\n", VSB_data(p->fin));
-			Fc(tl, 0, "\t}\n\n");
+			Fc(tl, 0, "\t\tcase %u :\n", p->n);
+			Fc(tl, 0, "\t%s\n", VSB_data(p->fin));
+			Fc(tl, 0, "\t\t\t/* FALLTHROUGH */\n");
 		}
-		VSB_delete(p->fin);
+		VSB_destroy(&p->fin);
 	}
+	Fc(tl, 0, "\t}\n\n");
 
-	Fc(tl, 0, "\treturn(0);\n");
+	Fc(tl, 0, "\treturn (0);\n");
 	Fc(tl, 0, "}\n");
 
 	if (has_event) {
@@ -403,24 +361,6 @@ EmitInitFini(const struct vcc *tl)
 		Fc(tl, 0, "\treturn (0);\n");
 		Fc(tl, 0, "}\n");
 
-		/*
-		 * USE (deprecated)
-		 */
-		Fc(tl, 0, "\nstatic int\n");
-		Fc(tl, 0, "VGC_Use(VRT_CTX, enum vcl_event_e ev)\n{\n\n");
-
-		VTAILQ_FOREACH(p, &tl->inifin, list) {
-			assert(p->n > 0);
-			if (VSB_len(p->event)) {
-				Fc(tl, 0, "\t/* %u */\n", p->n);
-				Fc(tl, 0, "\tif (%s)\n", VSB_data(p->event));
-				Fc(tl, 0, "\t\treturn (1);\n\n");
-			}
-		}
-
-		Fc(tl, 0, "\treturn (0);\n");
-		Fc(tl, 0, "}\n");
-
 		/*
 		 * COLD
 		 */
@@ -435,7 +375,7 @@ EmitInitFini(const struct vcc *tl)
 				Fc(tl, 0, "\t    %s != 0)\n", VSB_data(p->event));
 				Fc(tl, 0, "\t\tretval = 1;\n\n");
 			}
-			VSB_delete(p->event);
+			VSB_destroy(&p->event);
 		}
 
 		Fc(tl, 0, "\treturn (retval);\n");
@@ -449,17 +389,15 @@ EmitInitFini(const struct vcc *tl)
 	Fc(tl, 0, "VGC_Event(VRT_CTX, enum vcl_event_e ev)\n");
 	Fc(tl, 0, "{\n");
 	Fc(tl, 0, "\tif (ev == VCL_EVENT_LOAD)\n");
-	Fc(tl, 0, "\t\treturn(VGC_Load(ctx));\n");
+	Fc(tl, 0, "\t\treturn (VGC_Load(ctx));\n");
 	if (has_event) {
 		Fc(tl, 0, "\tif (ev == VCL_EVENT_WARM)\n");
-		Fc(tl, 0, "\t\treturn(VGC_Warmup(ctx, ev));\n");
-		Fc(tl, 0, "\tif (ev == VCL_EVENT_USE)\n");
-		Fc(tl, 0, "\t\treturn(VGC_Use(ctx, ev));\n");
+		Fc(tl, 0, "\t\treturn (VGC_Warmup(ctx, ev));\n");
 		Fc(tl, 0, "\tif (ev == VCL_EVENT_COLD)\n");
-		Fc(tl, 0, "\t\treturn(VGC_Cooldown(ctx, ev));\n");
+		Fc(tl, 0, "\t\treturn (VGC_Cooldown(ctx, ev));\n");
 	}
 	Fc(tl, 0, "\tif (ev == VCL_EVENT_DISCARD)\n");
-	Fc(tl, 0, "\t\treturn(VGC_Discard(ctx));\n");
+	Fc(tl, 0, "\t\treturn (VGC_Discard(ctx));\n");
 	Fc(tl, 0, "\n");
 	if (!has_event)
 		Fc(tl, 0, "\t(void)vgc_warmupstep;\n");
@@ -508,31 +446,21 @@ vcc_new_source(const char *b, const char *e, const char *name)
 	return (sp);
 }
 
-static void
-vcc_destroy_source(struct source *sp)
-{
-
-	if (sp->freeit != NULL)
-		free(sp->freeit);
-	free(sp->name);
-	free(sp);
-}
-
 /*--------------------------------------------------------------------*/
 
 static struct source *
-vcc_file_source(const struct vcp * const vcp, struct vsb *sb, const char *fn)
+vcc_file_source(const struct vcc *tl, const char *fn)
 {
 	char *f, *fnp;
 	struct source *sp;
 
-	if (!vcp->unsafe_path && strchr(fn, '/') != NULL) {
-		VSB_printf(sb, "VCL filename '%s' is unsafe.\n", fn);
+	if (!tl->unsafe_path && strchr(fn, '/') != NULL) {
+		VSB_printf(tl->sb, "VCL filename '%s' is unsafe.\n", fn);
 		return (NULL);
 	}
 	f = NULL;
-	if (VFIL_searchpath(vcp->vcl_path, NULL, &f, fn, &fnp) || f == NULL) {
-		VSB_printf(sb, "Cannot read file '%s' (%s)\n",
+	if (VFIL_searchpath(tl->vcl_path, NULL, &f, fn, &fnp) || f == NULL) {
+		VSB_printf(tl->sb, "Cannot read file '%s' (%s)\n",
 		    fnp != NULL ? fnp : fn, strerror(errno));
 		free(fnp);
 		return (NULL);
@@ -558,7 +486,7 @@ vcc_resolve_includes(struct vcc *tl)
 			continue;
 
 		t1 = VTAILQ_NEXT(t, list);
-		assert(t1 != NULL);	/* There's always an EOI */
+		AN(t1);			/* There's always an EOI */
 		if (t1->tok != CSTR) {
 			VSB_printf(tl->sb,
 			    "include not followed by string constant.\n");
@@ -566,7 +494,7 @@ vcc_resolve_includes(struct vcc *tl)
 			return;
 		}
 		t2 = VTAILQ_NEXT(t1, list);
-		assert(t2 != NULL);	/* There's always an EOI */
+		AN(t2);			/* There's always an EOI */
 
 		if (t2->tok != ';') {
 			VSB_printf(tl->sb,
@@ -595,10 +523,10 @@ vcc_resolve_includes(struct vcc *tl)
 			VSB_bcat(vsb, t1->src->name, p - t1->src->name);
 			VSB_cat(vsb, t1->dec + 1);
 			AZ(VSB_finish(vsb));
-			sp = vcc_file_source(tl->param, tl->sb, VSB_data(vsb));
-			VSB_delete(vsb);
+			sp = vcc_file_source(tl, VSB_data(vsb));
+			VSB_destroy(&vsb);
 		} else {
-			sp = vcc_file_source(tl->param, tl->sb, t1->dec);
+			sp = vcc_file_source(tl, t1->dec);
 		}
 		if (sp == NULL) {
 			vcc_ErrWhere(tl, t1);
@@ -618,117 +546,37 @@ vcc_resolve_includes(struct vcc *tl)
 	}
 }
 
-/*--------------------------------------------------------------------*/
-
-static struct vcc *
-vcc_NewVcc(const struct vcp *vcp)
-{
-	struct vcc *tl;
-	int i;
-
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
-	ALLOC_OBJ(tl, VCC_MAGIC);
-	AN(tl);
-	tl->param = vcp;
-	tl->vars = vcc_vars;
-	VTAILQ_INIT(&tl->symbols);
-	VTAILQ_INIT(&tl->inifin);
-	VTAILQ_INIT(&tl->membits);
-	VTAILQ_INIT(&tl->tokens);
-	VTAILQ_INIT(&tl->sources);
-
-	tl->nsources = 0;
-
-	/* General C code */
-	tl->fc = VSB_new_auto();
-	assert(tl->fc != NULL);
-
-	/* Forward decls (.h like) */
-	tl->fh = VSB_new_auto();
-	assert(tl->fh != NULL);
-
-	/* body code of methods */
-	for (i = 0; i < VCL_MET_MAX; i++) {
-		tl->fm[i] = VSB_new_auto();
-		assert(tl->fm[i] != NULL);
-	}
-	return (tl);
-}
-
-/*--------------------------------------------------------------------*/
-
-static char *
-vcc_DestroyTokenList(struct vcc *tl, char *ret)
-{
-	struct membit *mb;
-	struct source *sp;
-	struct symbol *sym;
-	int i;
-
-	while (!VTAILQ_EMPTY(&tl->membits)) {
-		mb = VTAILQ_FIRST(&tl->membits);
-		VTAILQ_REMOVE(&tl->membits, mb, list);
-		free(mb->ptr);
-		free(mb);
-	}
-	while (!VTAILQ_EMPTY(&tl->sources)) {
-		sp = VTAILQ_FIRST(&tl->sources);
-		VTAILQ_REMOVE(&tl->sources, sp, list);
-		vcc_destroy_source(sp);
-	}
-
-	while (!VTAILQ_EMPTY(&tl->symbols)) {
-		sym = VTAILQ_FIRST(&tl->symbols);
-		VTAILQ_REMOVE(&tl->symbols, sym, list);
-		FREE_OBJ(sym);
-	}
-
-	VSB_delete(tl->fh);
-	VSB_delete(tl->fc);
-	for (i = 0; i < VCL_MET_MAX; i++)
-		VSB_delete(tl->fm[i]);
-
-	free(tl);
-	return (ret);
-}
-
 /*--------------------------------------------------------------------
  * Compile the VCL code from the given source and return the C-source
  */
 
-static char *
-vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
-    struct source *sp)
+static struct vsb *
+vcc_CompileSource(struct vcc *tl, struct source *sp)
 {
-	struct vcc *tl;
 	struct symbol *sym;
 	const struct var *v;
 	struct vsb *vsb;
-
-	char *of;
 	int i;
 
-	tl = vcc_NewVcc(vcp);
-	tl->sb = sb;
-
 	vcc_Expr_Init(tl);
 
-	for (v = tl->vars; v->name != NULL; v++) {
+	for (v = vcc_vars; v->name != NULL; v++) {
 		if (v->fmt == HEADER) {
-			sym = VCC_AddSymbolStr(tl, v->name, SYM_WILDCARD);
+			sym = VCC_Symbol(tl, NULL, v->name, NULL,
+			    SYM_NONE, 1);
 			sym->wildcard = vcc_Var_Wildcard;
+			sym->wildcard_priv = v;
 		} else {
-			sym = VCC_AddSymbolStr(tl, v->name, SYM_VAR);
+			sym = VCC_Symbol(tl, NULL, v->name, NULL, SYM_VAR, 1);
 		}
-		sym->var = v;
 		sym->fmt = v->fmt;
 		sym->eval = vcc_Eval_Var;
 		sym->r_methods = v->r_methods;
+		sym->rname = v->rname;
+		sym->w_methods = v->w_methods;
+		sym->lname = v->lname;
 	}
 
-	sym = VCC_AddSymbolStr(tl, "storage.", SYM_WILDCARD);
-	sym->wildcard = vcc_Stv_Wildcard;
-
 	Fh(tl, 0, "/* ---===### VCC generated .h code ###===---*/\n");
 	Fc(tl, 0, "\n/* ---===### VCC generated .c code ###===---*/\n");
 	Fh(tl, 0, "\nextern const struct VCL_conf VCL_conf;\n");
@@ -738,32 +586,32 @@ vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
 	sp->idx = tl->nsources++;
 	vcc_Lexer(tl, sp);
 	if (tl->err)
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Register and lex the builtin VCL */
-	sp = vcc_new_source(tl->param->builtin_vcl, NULL, "Builtin");
+	sp = vcc_new_source(tl->builtin_vcl, NULL, "Builtin");
 	assert(sp != NULL);
 	VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
 	sp->idx = tl->nsources++;
 	vcc_Lexer(tl, sp);
 	if (tl->err)
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Add "END OF INPUT" token */
 	vcc_AddToken(tl, EOI, sp->e, sp->e);
 	if (tl->err)
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Expand and lex any includes in the token string */
 	vcc_resolve_includes(tl);
 	if (tl->err)
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Parse the token string */
 	tl->t = VTAILQ_FIRST(&tl->tokens);
 	vcc_Parse(tl);
 	if (tl->err)
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Check if we have any backends at all */
 	if (tl->default_director == NULL) {
@@ -771,7 +619,7 @@ vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
 		    "No backends or directors found in VCL program, "
 		    "at least one is necessary.\n");
 		tl->err = 1;
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 	}
 
 	/* Configure the default director */
@@ -779,15 +627,15 @@ vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
 
 	/* Check for orphans */
 	if (vcc_CheckReferences(tl))
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Check that all action returns are legal */
 	if (vcc_CheckAction(tl) || tl->err)
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Check that all variable uses are legal */
 	if (vcc_CheckUses(tl) || tl->err)
-		return (vcc_DestroyTokenList(tl, NULL));
+		return (NULL);
 
 	/* Emit method functions */
 	Fh(tl, 1, "\n");
@@ -811,7 +659,7 @@ vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
 			Fc(tl, 1, "  VRT_handling(ctx, VCL_RET_OK);\n");
 		Fc(tl, 1, "%s", VSB_data(tl->fm[i]));
 		if (method_tab[i].bitval == VCL_MET_INIT)
-			Fc(tl, 1, "  return(1);\n");
+			Fc(tl, 1, "  return (1);\n");
 		Fc(tl, 1, "}\n");
 	}
 
@@ -824,6 +672,9 @@ vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
 	vsb = VSB_new_auto();
 	AN(vsb);
 
+	AZ(VSB_finish(tl->fi));
+	VSB_cat(vsb, VSB_data(tl->fi));
+
 	vcl_output_lang_h(vsb);
 
 	EmitCoordinates(tl, vsb);
@@ -835,14 +686,7 @@ vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
 	VSB_cat(vsb, VSB_data(tl->fc));
 
 	AZ(VSB_finish(vsb));
-
-	of = strdup(VSB_data(vsb));
-	AN(of);
-
-	VSB_delete(vsb);
-
-	/* done */
-	return (vcc_DestroyTokenList(tl, of));
+	return (vsb);
 }
 
 /*--------------------------------------------------------------------
@@ -850,23 +694,24 @@ vcc_CompileSource(const struct vcp * const vcp, struct vsb *sb,
  * formatted into the vsb.
  */
 
-char *
-VCC_Compile(const struct vcp *vcp, struct vsb *sb,
+struct vsb *
+VCC_Compile(struct vcc *tl, struct vsb **sb,
     const char *vclsrc, const char *vclsrcfile)
 {
 	struct source *sp;
-	char *r;
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
+	struct vsb *r = NULL;
+
+	CHECK_OBJ_NOTNULL(tl, VCC_MAGIC);
 	AN(sb);
 	AN(vclsrcfile);
 
 	if (vclsrc != NULL)
 		sp = vcc_new_source(vclsrc, NULL, vclsrcfile);
 	else
-		sp = vcc_file_source(vcp, sb, vclsrcfile);
-	if (sp == NULL)
-		return (NULL);
-	r = vcc_CompileSource(vcp, sb, sp);
+		sp = vcc_file_source(tl, vclsrcfile);
+	if (sp != NULL)
+		r = vcc_CompileSource(tl, sp);
+	*sb = tl->sb;
 	return (r);
 }
 
@@ -874,15 +719,36 @@ VCC_Compile(const struct vcp *vcp, struct vsb *sb,
  * Allocate a compiler instance
  */
 
-struct vcp *
-VCP_New(void)
+struct vcc *
+VCC_New(void)
 {
-	struct vcp *vcp;
+	struct vcc *tl;
+	int i;
 
-	ALLOC_OBJ(vcp, VCP_MAGIC);
-	AN(vcp);
+	ALLOC_OBJ(tl, VCC_MAGIC);
+	AN(tl);
+	VTAILQ_INIT(&tl->inifin);
+	VTAILQ_INIT(&tl->tokens);
+	VTAILQ_INIT(&tl->sources);
 
-	return (vcp);
+	tl->nsources = 0;
+
+	tl->fi = VSB_new_auto();
+	assert(tl->fi != NULL);
+
+	tl->fc = VSB_new_auto();
+	assert(tl->fc != NULL);
+
+	tl->fh = VSB_new_auto();
+	assert(tl->fh != NULL);
+
+	for (i = 0; i < VCL_MET_MAX; i++) {
+		tl->fm[i] = VSB_new_auto();
+		assert(tl->fm[i] != NULL);
+	}
+	tl->sb = VSB_new_auto();
+	AN(tl->sb);
+	return (tl);
 }
 
 /*--------------------------------------------------------------------
@@ -890,37 +756,35 @@ VCP_New(void)
  */
 
 void
-VCP_Builtin_VCL(struct vcp *vcp, const char *str)
+VCC_Builtin_VCL(struct vcc *vcc, const char *str)
 {
 
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
-	REPLACE(vcp->builtin_vcl, str);
+	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
+	REPLACE(vcc->builtin_vcl, str);
 }
 
 /*--------------------------------------------------------------------
- * Configure default VCL source directory
+ * Configure default VCL source path
  */
 
 void
-VCP_VCL_dir(struct vcp *vcp, const char *str)
+VCC_VCL_path(struct vcc *vcc, const char *str)
 {
 
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
-	REPLACE(vcp->vcl_dir, str);
-	VFIL_setpath(&vcp->vcl_path, str);
+	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
+	VFIL_setpath(&vcc->vcl_path, str);
 }
 
 /*--------------------------------------------------------------------
- * Configure default VMOD directory
+ * Configure default VMOD path
  */
 
 void
-VCP_VMOD_dir(struct vcp *vcp, const char *str)
+VCC_VMOD_path(struct vcc *vcc, const char *str)
 {
 
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
-	REPLACE(vcp->vmod_dir, str);
-	VFIL_setpath(&vcp->vmod_path, str);
+	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
+	VFIL_setpath(&vcc->vmod_path, str);
 }
 
 /*--------------------------------------------------------------------
@@ -928,25 +792,53 @@ VCP_VMOD_dir(struct vcp *vcp, const char *str)
  */
 
 void
-VCP_Err_Unref(struct vcp *vcp, unsigned u)
+VCC_Err_Unref(struct vcc *vcc, unsigned u)
 {
 
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
-	vcp->err_unref = u;
+	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
+	vcc->err_unref = u;
 }
 
 void
-VCP_Allow_InlineC(struct vcp *vcp, unsigned u)
+VCC_Allow_InlineC(struct vcc *vcc, unsigned u)
 {
 
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
-	vcp->allow_inline_c = u;
+	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
+	vcc->allow_inline_c = u;
 }
 
 void
-VCP_Unsafe_Path(struct vcp *vcp, unsigned u)
+VCC_Unsafe_Path(struct vcc *vcc, unsigned u)
 {
 
-	CHECK_OBJ_NOTNULL(vcp, VCP_MAGIC);
-	vcp->unsafe_path = u;
+	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
+	vcc->unsafe_path = u;
+}
+
+/*--------------------------------------------------------------------
+ * Configure settings
+ */
+
+static void
+vcc_predef_vcl(struct vcc *vcc, const char *name)
+{
+	struct symbol *sym;
+
+	sym = VCC_Symbol(vcc, NULL, name, NULL, SYM_VCL, 1);
+	AN(sym);
+	sym->fmt = VCL;
+	sym->r_methods = VCL_MET_RECV;
+}
+
+void
+VCC_Predef(struct vcc *vcc, const char *type, const char *name)
+{
+
+	CHECK_OBJ_NOTNULL(vcc, VCC_MAGIC);
+	if (!strcmp(type, "VCL_STEVEDORE"))
+		vcc_stevedore(vcc, name);
+	else if (!strcmp(type, "VCL_VCL"))
+		vcc_predef_vcl(vcc, name);
+	else
+		WRONG("Unknown VCC predef type");
 }
