@@ -160,7 +160,7 @@ vcc_Compound(struct vcc *tl)
 			Fb(tl, 1, "}\n");
 			return;
 		case CSRC:
-			if (tl->param->allow_inline_c) {
+			if (tl->allow_inline_c) {
 				Fb(tl, 1, "%.*s\n",
 				    (int) (tl->t->e - (tl->t->b + 2)),
 				    tl->t->b + 1);
@@ -211,13 +211,8 @@ vcc_ParseFunction(struct vcc *tl)
 	int m, i;
 
 	vcc_NextToken(tl);
-	ExpectErr(tl, ID);
-	if (!vcc_isCid(tl->t)) {
-		VSB_printf(tl->sb,
-		    "Names of VCL sub's cannot contain '-'\n");
-		vcc_ErrWhere(tl, tl->t);
-		return;
-	}
+	vcc_ExpectCid(tl, "function");
+	ERRCHK(tl);
 
 	m = IsMethod(tl->t);
 	if (m == -2) {
@@ -225,7 +220,7 @@ vcc_ParseFunction(struct vcc *tl)
 		    "VCL sub's named 'vcl*' are reserved names.\n");
 		vcc_ErrWhere(tl, tl->t);
 		VSB_printf(tl->sb, "Valid vcl_* methods are:\n");
-		for (i = 0; method_tab[i].name != NULL; i++)
+		for (i = 1; method_tab[i].name != NULL; i++)
 			VSB_printf(tl->sb, "\t%s\n", method_tab[i].name);
 		return;
 	} else if (m != -1) {
@@ -245,7 +240,7 @@ vcc_ParseFunction(struct vcc *tl)
 		i = vcc_AddDef(tl, tl->t, SYM_SUB);
 		if (i > 1) {
 			VSB_printf(tl->sb,
-			    "Function %.*s redefined\n", PF(tl->t));
+			    "Function '%.*s' redefined\n", PF(tl->t));
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
@@ -276,33 +271,28 @@ vcc_ParseFunction(struct vcc *tl)
 /*--------------------------------------------------------------------
  */
 
-static void
-vcc_ParseDirector(struct vcc *tl)
-{
-	VSB_printf(tl->sb, "\ndirectors are now in directors VMOD.\n");
-	vcc_ErrWhere(tl, tl->t);
-}
-
-/*--------------------------------------------------------------------
- * TODO when bumping the vcl version:
- * #1603 remove return(fetch) from vcl_hit{}
- */
-
 static void
 vcc_ParseVcl(struct vcc *tl)
 {
-	double ver;
 	struct token *tok;
 
 	assert(vcc_IdIs(tl->t, "vcl"));
 	vcc_NextToken(tl);
 	tok = tl->t;
-	ver = vcc_DoubleVal(tl);
+	tok->src->syntax = vcc_DoubleVal(tl);
 	ERRCHK(tl);
-	if (ver != 4.0) {
-		// see TODO above
-		VSB_printf(tl->sb, "VCL version %.1f not supported.\n", ver);
-		vcc_ErrWhere(tl, tok);
+	if (tl->t->e - tok->b > 4) {
+		VSB_printf(tl->sb,
+		    "Don't play silly buggers with VCL version numbers\n");
+		vcc_ErrWhere2(tl, tok, tl->t);
+		ERRCHK(tl);
+	}
+	if (tl->syntax != 0.0 && tok->src->syntax > tl->syntax) {
+		VSB_printf(tl->sb,
+		    "VCL version %.1f higher than"
+		    " the top level version %.1f\n",
+		    tok->src->syntax, tl->syntax);
+		vcc_ErrWhere2(tl, tok, tl->t);
 		ERRCHK(tl);
 	}
 	ExpectErr(tl, ';');
@@ -329,7 +319,6 @@ static struct toplev {
 	{ "acl",		vcc_ParseAcl },
 	{ "sub",		vcc_ParseFunction },
 	{ "backend",		vcc_ParseBackend },
-	{ "director",		vcc_ParseDirector },
 	{ "probe",		vcc_ParseProbe },
 	{ "import",		vcc_ParseImport },
 	{ "vcl",		vcc_ParseVcl },
@@ -340,24 +329,33 @@ void
 vcc_Parse(struct vcc *tl)
 {
 	struct toplev *tp;
+	struct token *tok;
 
 	if (tl->t->tok != ID || !vcc_IdIs(tl->t, "vcl")) {
 		VSB_printf(tl->sb,
 		    "VCL version declaration missing\n"
 		    "Update your VCL to Version 4 syntax, and add\n"
 		    "\tvcl 4.0;\n"
-		    "on the first line the VCL files.\n"
+		    "on the first line of the VCL files.\n"
 		);
 		vcc_ErrWhere(tl, tl->t);
 		ERRCHK(tl);
 	}
+	tok = tl->t;
 	vcc_ParseVcl(tl);
+	if (tok->src->syntax != 4.0) {
+		VSB_printf(tl->sb, "VCL version %.1f not supported.\n",
+		    tok->src->syntax);
+		vcc_ErrWhere2(tl, tok, tl->t);
+		ERRCHK(tl);
+	}
+	tl->syntax = tl->t->src->syntax;
 	ERRCHK(tl);
 	while (tl->t->tok != EOI) {
 		ERRCHK(tl);
 		switch (tl->t->tok) {
 		case CSRC:
-			if (tl->param->allow_inline_c) {
+			if (tl->allow_inline_c) {
 				Fc(tl, 0, "%.*s\n",
 				    (int) (tl->t->e - (tl->t->b + 4)),
 				    tl->t->b + 2);
