@@ -37,32 +37,31 @@
 
 /*--------------------------------------------------------------------*/
 
-struct symbol *
-vcc_Var_Wildcard(struct vcc *tl, const struct token *t, const struct symbol *wc)
+void __match_proto__(sym_wildcard_t)
+vcc_Var_Wildcard(struct vcc *tl, struct symbol *parent,
+    const char *b, const char *e)
 {
 	struct symbol *sym;
 	struct var *v;
 	const struct var *vh;
 	unsigned u;
-	const char *p, *leaf;
+	const char *p;
 	struct vsb *vsb;
 
-	vh = wc->var;
+	vh = parent->wildcard_priv;
 	assert(vh->fmt == HEADER);
 
 	v = TlAlloc(tl, sizeof *v);
 	AN(v);
-	v->name = TlDupTok(tl, t);
 	v->r_methods = vh->r_methods;
 	v->w_methods = vh->w_methods;
 	v->fmt = vh->fmt;
-	leaf = v->name + vh->len;
 
 	/* Create a C-name version of the header name */
 	vsb = VSB_new_auto();
 	AN(vsb);
 	VSB_printf(vsb, "&VGC_%s_", vh->rname);
-	for (p = leaf, u = 1; *p != '\0'; p++, u++)
+	for (p = b, u = 1; p < e; p++, u++)
 		if (vct_isalpha(*p) || vct_isdigit(*p))
 			VSB_putc(vsb, *p);
 		else
@@ -71,7 +70,8 @@ vcc_Var_Wildcard(struct vcc *tl, const struct token *t, const struct symbol *wc)
 
 	/* Create the static identifier */
 	Fh(tl, 0, "static const struct gethdr_s %s =\n", VSB_data(vsb) + 1);
-	Fh(tl, 0, "    { %s, \"\\%03o%s:\"};\n", vh->rname, u, leaf);
+	Fh(tl, 0, "    { %s, \"\\%03o%.*s:\"};\n",
+	    vh->rname, u, (int)(e - b), b);
 
 	/* Create the symbol r/l values */
 	v->rname = TlDup(tl, VSB_data(vsb));
@@ -79,33 +79,29 @@ vcc_Var_Wildcard(struct vcc *tl, const struct token *t, const struct symbol *wc)
 	VSB_printf(vsb, "VRT_SetHdr(ctx, %s,", v->rname);
 	AZ(VSB_finish(vsb));
 	v->lname = TlDup(tl, VSB_data(vsb));
-	VSB_delete(vsb);
+	VSB_destroy(&vsb);
 
-	sym = VCC_AddSymbolTok(tl, t, SYM_VAR);
+	sym = VCC_Symbol(tl, parent, b, e, SYM_VAR, 1);
 	AN(sym);
-	sym->var = v;
 	sym->fmt = v->fmt;
 	sym->eval = vcc_Eval_Var;
 	sym->r_methods = v->r_methods;
-	return (sym);
+	sym->rname = v->rname;
+	sym->w_methods = v->w_methods;
+	sym->lname = v->lname;
 }
 
 /*--------------------------------------------------------------------*/
 
-const struct var *
+const struct symbol *
 vcc_FindVar(struct vcc *tl, const struct token *t, int wr_access,
     const char *use)
 {
-	const struct var *v;
 	const struct symbol *sym;
 
-	AN(tl->vars);
-	sym = VCC_FindSymbol(tl, t, SYM_VAR);
+	sym = VCC_SymbolTok(tl, NULL, t, SYM_VAR, 0);
 	if (sym != NULL) {
-		v = sym->var;
-		AN(v);
-
-		if (wr_access && v->w_methods == 0) {
+		if (wr_access && sym->w_methods == 0) {
 			VSB_printf(tl->sb, "Variable ");
 			vcc_ErrToken(tl, t);
 			VSB_printf(tl->sb, " is read only.");
@@ -113,8 +109,8 @@ vcc_FindVar(struct vcc *tl, const struct token *t, int wr_access,
 			vcc_ErrWhere(tl, t);
 			return (NULL);
 		} else if (wr_access) {
-			vcc_AddUses(tl, t, v->w_methods, use);
-		} else if (v->r_methods == 0) {
+			vcc_AddUses(tl, t, sym->w_methods, use);
+		} else if (sym->r_methods == 0) {
 			VSB_printf(tl->sb, "Variable ");
 			vcc_ErrToken(tl, t);
 			VSB_printf(tl->sb, " is write only.");
@@ -122,9 +118,9 @@ vcc_FindVar(struct vcc *tl, const struct token *t, int wr_access,
 			vcc_ErrWhere(tl, t);
 			return (NULL);
 		} else {
-			vcc_AddUses(tl, t, v->r_methods, use);
+			vcc_AddUses(tl, t, sym->r_methods, use);
 		}
-		return (v);
+		return (sym);
 	}
 	VSB_printf(tl->sb, "Unknown variable ");
 	vcc_ErrToken(tl, t);
