@@ -47,7 +47,7 @@ struct priv_vcl {
 	unsigned		magic;
 #define PRIV_VCL_MAGIC		0x8E62FA9D
 	char			*foo;
-	uintptr_t		exp_cb;
+	uintptr_t		obj_cb;
 	struct vcl		*vcl;
 	struct vclref		*vclref;
 };
@@ -101,9 +101,19 @@ vmod_test_priv_task(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	if (priv->priv == NULL) {
+	if (s == NULL || *s == '\0') {
+		return priv->priv;
+	} else if (priv->priv == NULL) {
 		priv->priv = strdup(s);
 		priv->free = free;
+	} else {
+		char *n = realloc(priv->priv,
+		    strlen(priv->priv) + strlen(s) + 2);
+		if (n == NULL)
+			return NULL;
+		strcat(n, " ");
+		strcat(n, s);
+		priv->priv = n;
 	}
 	return (priv->priv);
 }
@@ -186,11 +196,11 @@ vmod_rot52(VRT_CTX, VCL_HTTP hp)
 
 VCL_STRING
 vmod_argtest(VRT_CTX, VCL_STRING one, VCL_REAL two, VCL_STRING three,
-    VCL_STRING comma)
+    VCL_STRING comma, VCL_INT four)
 {
 	char buf[100];
 
-	bprintf(buf, "%s %g %s %s", one, two, three, comma);
+	bprintf(buf, "%s %g %s %s %ld", one, two, three, comma, four);
 	return (WS_Copy(ctx->ws, buf, -1));
 }
 
@@ -201,8 +211,8 @@ vmod_vre_limit(VRT_CTX)
 	return (cache_param->vre_limits.match);
 }
 
-static void __match_proto__(exp_callback_f)
-exp_cb(struct worker *wrk, struct objcore *oc, enum exp_event_e ev, void *priv)
+static void __match_proto__(obj_event_f)
+obj_cb(struct worker *wrk, void *priv, struct objcore *oc, unsigned event)
 {
 	const struct priv_vcl *priv_vcl;
 	const char *what;
@@ -210,26 +220,28 @@ exp_cb(struct worker *wrk, struct objcore *oc, enum exp_event_e ev, void *priv)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	CAST_OBJ_NOTNULL(priv_vcl, priv, PRIV_VCL_MAGIC);
-	switch (ev) {
-	case EXP_INSERT: what = "insert"; break;
-	case EXP_INJECT: what = "inject"; break;
-	case EXP_REMOVE: what = "remove"; break;
-	default: WRONG("Wrong exp_event");
+	switch (event) {
+	case OEV_INSERT: what = "insert"; break;
+	case OEV_EXPIRE: what = "expire"; break;
+	default: WRONG("Wrong object event");
 	}
-	VSL(SLT_Debug, 0, "exp_cb: event %s 0x%jx", what,
+
+	/* We cannot trust %p to be 0x... format as expected by m00021.vtc */
+	VSL(SLT_Debug, 0, "Object Event: %s 0x%jx", what,
 	    (intmax_t)(uintptr_t)oc);
 }
 
 VCL_VOID __match_proto__()
-vmod_register_exp_callback(VRT_CTX, struct vmod_priv *priv)
+vmod_register_obj_events(VRT_CTX, struct vmod_priv *priv)
 {
 	struct priv_vcl *priv_vcl;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
-	AZ(priv_vcl->exp_cb);
-	priv_vcl->exp_cb = EXP_Register_Callback(exp_cb, priv_vcl);
-	VSL(SLT_Debug, 0, "exp_cb: registered");
+	AZ(priv_vcl->obj_cb);
+	priv_vcl->obj_cb = ObjSubscribeEvents(obj_cb, priv_vcl,
+		OEV_INSERT|OEV_EXPIRE);
+	VSL(SLT_Debug, 0, "Subscribed to Object Events");
 }
 
 VCL_VOID __match_proto__()
@@ -249,9 +261,9 @@ priv_vcl_free(void *priv)
 	CAST_OBJ_NOTNULL(priv_vcl, priv, PRIV_VCL_MAGIC);
 	AN(priv_vcl->foo);
 	free(priv_vcl->foo);
-	if (priv_vcl->exp_cb != 0) {
-		EXP_Deregister_Callback(&priv_vcl->exp_cb);
-		VSL(SLT_Debug, 0, "exp_cb: deregistered");
+	if (priv_vcl->obj_cb != 0) {
+		ObjUnsubscribeEvents(&priv_vcl->obj_cb);
+		VSL(SLT_Debug, 0, "Unsubscribed from Object Events");
 	}
 	AZ(priv_vcl->vcl);
 	AZ(priv_vcl->vclref);
@@ -513,3 +525,13 @@ vmod_barrier_sync(VRT_CTX, VCL_STRING addr)
 		VSLb(ctx->vsl, SLT_Error, "Barrier unexpected data (%zdB)", sz);
 	return (0);
 }
+
+VCL_VOID
+vmod_test_probe(VRT_CTX, VCL_PROBE probe, VCL_PROBE same)
+{
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(probe, VRT_BACKEND_PROBE_MAGIC);
+	CHECK_OBJ_ORNULL(same, VRT_BACKEND_PROBE_MAGIC);
+	AZ(same == NULL || probe == same);
+}
