@@ -37,6 +37,7 @@
 #include <stdlib.h>
 #include <syslog.h>
 
+#include "vrnd.h"
 #include "vrt.h"
 #include "vtcp.h"
 #include "vsa.h"
@@ -125,34 +126,38 @@ vmod_random(VRT_CTX, VCL_REAL lo, VCL_REAL hi)
 	double a;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	a = drand48();
+	a = VRND_RandomTestableDouble();
 	a *= hi - lo;
 	a += lo;
 	return (a);
 }
 
 VCL_VOID __match_proto__(td_std_log)
-vmod_log(VRT_CTX, const char *fmt, ...)
+vmod_log(VRT_CTX, const char *s, ...)
 {
+	txt t;
 	unsigned u;
 	va_list ap;
-	txt t;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	va_start(ap, fmt);
-	if (ctx->vsl != NULL) {
-		u = WS_Reserve(ctx->ws, 0);
-		t.b = ctx->ws->f;
-		t.e = VRT_StringList(ctx->ws->f, u, fmt, ap);
-		if (t.e != NULL) {
-			assert(t.e > t.b);
-			t.e--;
-			VSLbt(ctx->vsl, SLT_VCL_Log, t);
-		}
-		WS_Release(ctx->ws, 0);
-	} else
-		VSLv(SLT_VCL_Log, 0, fmt, ap);
+	WS_Assert(ctx->ws);
+
+
+	u = WS_Reserve(ctx->ws, 0);
+	t.b = ctx->ws->f;
+	va_start(ap, s);
+	t.e = VRT_StringList(ctx->ws->f, u, s, ap);
 	va_end(ap);
+
+	if (t.e != NULL) {
+		assert(t.e > t.b);
+		t.e--;
+		if (ctx->vsl != NULL)
+			VSLbt(ctx->vsl, SLT_VCL_Log, t);
+		else
+			VSL(SLT_VCL_Log, 0, "%s", t.b);
+	}
+	WS_Release(ctx->ws, 0);
 }
 
 VCL_VOID __match_proto__(td_std_syslog)
@@ -231,18 +236,15 @@ vmod_timestamp(VRT_CTX, VCL_STRING label)
 	}
 }
 
-VCL_VOID __match_proto__(td_std_cache_req_body)
+VCL_BOOL __match_proto__(td_std_cache_req_body)
 vmod_cache_req_body(VRT_CTX, VCL_BYTES size)
 {
-	int result;
-	ssize_t ss;
-
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	if (size < 0)
 		size = 0;
-	ss = (ssize_t)size;
-	result = VRT_CacheReqBody(ctx, ss);
-	VSLb(ctx->vsl, SLT_Debug, "VRT_CacheReqBody(%zd): %d", ss, result);
+	if (VRT_CacheReqBody(ctx, (size_t)size) < 0)
+		return (0);
+	return (1);
 }
 
 VCL_STRING __match_proto__(td_std_strstr)
